<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Api\AppBaseController;

use App\Repositories\GeographicPointRepository;
use App\Repositories\AdresseRepository;
use App\Repositories\CommandStateRepository;
use App\Repositories\CommandRepository;
use App\Repositories\UserRepository;
use App\Repositories\ClientRepository;
use App\Repositories\DeliveryCommandRepository;

class DeliveryController extends AppBaseController
{

    protected $geographicPointRepository;
    protected $adresseRepository;
    protected $commandStateRepository;
    protected $commandRepository;
    protected $userRepository;
    protected $clientRepository;
    protected $deliveryCommandRepository;

    public function __construct(
        GeographicPointRepository $geographicPointRepository,
        AdresseRepository $adresseRepository,
        CommandStateRepository $commandStateRepository,
        CommandRepository $commandRepository,
        UserRepository $userRepository,
        DeliveryCommandRepository $deliveryCommandRepository,
        ClientRepository $clientRepository
    ){
        $this->geographicPointRepository = $geographicPointRepository;
        $this->adresseRepository = $adresseRepository;
        $this->commandStateRepository = $commandStateRepository;
        $this->commandRepository = $commandRepository;
        $this->userRepository = $userRepository;
        $this->clientRepository = $clientRepository;
        $this->deliveryCommandRepository = $deliveryCommandRepository;
    }

    public function getCommandPosition(Request $request, $commandId) {

        $user = \JWTAuth::toUser();

        $command = $this->commandRepository->find($commandId);
        $positions = $command->positions;

        $position = null;
        if($positions->count() > 0) {
            $position = $positions->last()->geographicPoint;
        }

        return response()->json([
            'position' => $position
        ], 200);

    }

    public function markCommandDelivered(Request $request, $commandId) {

        $user = \JWTAuth::toUser();

        $command = $this->commandRepository->find($commandId);

        $command->stateId = $this->commandStateRepository->getByTag("delivered")->id;

        $command->save();

        return response()->json([
            'status' => "ok"
        ], 200);

    }

    public function trackDeliverMan(Request $request, $commandId) {

        $user = \JWTAuth::toUser();

        $command = $this->commandRepository->find($commandId);

        if($command->state->tag == "ready") {
            $command->stateId = $this->commandStateRepository->getByTag("delivering")->id;
            $command->save();
        }

        $geographicPoint = $this->geographicPointRepository->create([
            "latitude" => $request->lat,
            "longitude" => $request->lng
        ]);

        $commandList = collect([ $command ]);
        if($command->invitedCommands->count() > 0){
            $commandList = $commandList->merge($command->invitedCommands);
        }

        $commandList->map(function($cmd) use ($user, $geographicPoint) {
            $deliveryCommand = $this->deliveryCommandRepository->create([
                "userId" => $user->id,
                "commandId" => $cmd->id,
                "geographicPointId" => $geographicPoint->id
            ]);
        });

        return response()->json([
            'status' => "ok"
        ], 200);

    }

    public function getCommands(Request $request) {

        $user = \JWTAuth::toUser();

        $state = $this->commandStateRepository->allQuery(
            ["tag" => $request->tag]
        )->first();

        $commands = $this->commandRepository->getDeliveryManCommands(
            $user->id,
            $state->id
        );

        return response()->json([
            'commands' => $commands
        ], 200);

    }

}
