<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use App\Models\Product;

use Illuminate\Http\Request;
use App\Models\ProductDetailKey;
use App\Models\ProductDescription;
use Illuminate\Support\Facades\DB;
use App\Repositories\PriceRepository;
use App\Repositories\EntrepriseRepository;
use App\Repositories\StockOperationRepository;
use App\Http\Controllers\Api\AppBaseController;
use App\Repositories\ReductionIntervalRepository;

class HomeController extends AppBaseController {

    protected $priceRepository;
    protected $entrepriseRepository;
    protected $stockOperationRepository;

    public function __construct(StockOperationRepository $stockOperationRepository, PriceRepository $priceRepository, EntrepriseRepository $entrepriseRepository){
        $this->stockOperationRepository = $stockOperationRepository;
        $this->priceRepository = $priceRepository;
        $this->entrepriseRepository = $entrepriseRepository;
    }
    
    public function getDescriptionIds_dependsOnStock($stock_type) 
    {
        $DescriptionsIds = [];

        if($stock_type == 'in-stock')
        {
            $DescriptionsIds = DB::select('SELECT `ProductDescription`.id FROM `ProductDescription` 
            JOIN `Product` ON `Product`.product_description_id = `ProductDescription`.id
            JOIN `StockOperationProduct` ON `Product`.id = `StockOperationProduct`.productId
            GROUP BY `ProductDescription`.id
            HAVING MAX(`StockOperationProduct`.quantity) > 0;');
        }
        else if($stock_type == 'out-stock'){
            $DescriptionsIds = DB::select('SELECT `ProductDescription`.id FROM `ProductDescription` 
            JOIN `Product` ON `Product`.product_description_id = `ProductDescription`.id
            JOIN `StockOperationProduct` ON `Product`.id = `StockOperationProduct`.productId
            GROUP BY `ProductDescription`.id
            HAVING MAX(`StockOperationProduct`.quantity) <= 0;');
        }

        return collect($DescriptionsIds)->pluck('id');
    }



    public function getProductsGuests(Request $request){
        
        $offset = $request->offset ?? 0;
        $limit = $request->limit ?? 6;

        $colors = $sizes = $price = $category = null;

        if($filters = $request->filters)
        {
            $colors = array_key_exists('colors', $filters) ? $filters['colors'] : null;
            $sizes  = array_key_exists('sizes', $filters) ? $filters['sizes'] : null;
            $price = array_key_exists('price', $filters) ? $filters['price'] : null;
            $category = array_key_exists('category', $filters) ? $filters['category'] : null;
        }

        $products = Product::select('id', 'file_path', 'product_description_id')

        ->whereHas('prices', function ($query) use($price){
            $query
            ->where("isHidden", false)
            ->where("isStandard", false)
            ->where(function($query){
                $query->whereHas('tags', function ($query) {
                    $query->where('tag', '!=' ,'delivery_fees');
                })
                ->orWhereDoesntHave('tags');
            });
            $price && $query->whereBetween('promoPrice', $price);
        })
        

        ->whereHas('description', function ($query) use($category){
            $query->where('label', '!=', 'Livraison')
            ->whereHas('availability', function($query) {
                $query->where('is_available', true);
            });

            $category && $query->whereHas('category', function($query) use($category){
                $query
                ->where('tag', $category)
                ->orWhereHas('parent', function($query) use($category){
                    $query->where('tag', $category);
                });
            });
        })

        ->whereHas('stock', function($query) {
            $query->where('quantity', '>', 0);
        });

        $colors && $products = $products->whereHas('details', function($query) use($colors){
            $query->whereIn('value', $colors);
        });

        $sizes && $products = $products->whereHas('details', function($query) use($sizes){
            $query->whereIn('value', $sizes);
        });
        
        $products = $products->groupBy('product_description_id')
        ->latest('created_at')
        ->with('prices')
        ->offset($offset)
        ->limit($limit)
        ->get();

        $data = [];
        foreach ($products as $product) {
            $price = $product->prices[0];
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $product->description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $product->description->category->tag;
            $item['file_path'] = $product->file_path;
            $item['stock'] = $product->stock->quantity;

            if($colors) $item['color'] = $product->detailKeys->whereIn('value', $colors)->first();
            else {
                $item['color'] = $product->detailKeys()->select('value', 'id')->whereHas('detailKey', function ($query) {
                    $query->whereIn('tag', ['couleurs', 'color']);
                })->first();
            }
            $item['color'] && $item['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $item['color']->value)->with('product.stock')->get();

            $item['sizes'] = [];
     
            $item['productColors'] = $this->getColorsByProductDescription($product->product_description_id);
            array_push($data, $item);
        }
        return response()->json([
            'products' => $data,
        ], 200);
    }

    public function getProductsGuests_old_2(Request $request) {

        return $this->getPaginatedSoldesProducts($request->offset ?? 0);

        $filters        = $request->filters ?? [];
        $stock = $color = $size = $age = $price = $category = $parentCategory = $searchValue = null;

        $category       = array_key_exists('category', $filters) ? $filters['category'] : null;
        $parentCategory = array_key_exists('parentCategories', $filters) ? $filters['parentCategories'] : null;
        $stock          = array_key_exists('stock', $filters) ? $filters['stock'] : null;
        $color          = array_key_exists('color', $filters) ? $filters['color'] : null;
        $size           = array_key_exists('size', $filters) ? $filters['size'] : null;
        $age            = array_key_exists('age', $filters) ? $filters['age'] : null;
        $price          = array_key_exists('price', $filters) ? $filters['price'] : null;
        $searchValue    = array_key_exists('searchValue', $filters) ? $filters['searchValue'] : null;


        $query = ProductDescription::select('id');


        if($searchValue) $query = $query->where('label', 'LIKE', '%'. $searchValue .'%');

        $query = $query->where('label', '!=', 'Livraison')
        ->whereHas('availability', function($query) {
            $query->where('is_available', true);
        });

        if($parentCategory){
            $query = $query->whereHas('category', function ($catQuery) use($parentCategory){
                $catQuery->where('id', $parentCategory['id']);
                $catQuery->orWhere('parent_id', $parentCategory['id']);
            });
        }
        else if($category != 'all') $query->whereHas('category', function ($query) use ($category){
            $query = $query->where('tag', $category);
       });




        if($stock == 'in-stock' || $stock == 'out-stock')
        {
            $description_ids = $this->getDescriptionIds_dependsOnStock($stock);
            $query = $query->whereIn('id', $description_ids);
        }

        $DescriptionIds = $query->get();

        
        $products = Product::select('id', 'image', 'product_description_id')
        ->whereIn('product_description_id', $DescriptionIds)
        ->groupBy('product_description_id')
        ->orderBy('created_at', 'desc');

    
        if(isset($color) && count($color) > 0){
            $products = $products->whereHas('detailKeys', function($colorQuery) use($color){
                $colorQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'color');
                    $keyQuery->orWhere('tag', 'couleurs');
                })
                ->whereIn('value', $color);
            });
        }
        if(isset($size) && count($size) > 0){
            $products = $products->whereHas('detailKeys', function($sizeQuery) use($size){
                $sizeQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'size');
                    $keyQuery->orWhere('tag', 'taille');
                })
                ->whereIn('value', $size);
            });
        }
        if(isset($age) && count($age) > 0){
            $products = $products->whereHas('detailKeys', function($ageQuery) use($age){
                $ageQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'age');
                })
                ->whereIn('value', $age);
            });
        }
        $products = $products->whereHas('prices', function ($subQuery) use($price){
            $subQuery->where("isHidden", false)

            ->where("isStandard", false)
            ->where(function($priceQuery){
                $priceQuery->whereHas('tags', function ($tagQuery) {
                    $tagQuery->where('tag', '!=' ,'delivery_fees');
                })
                ->orWhereDoesntHave('tags');
            })
            ->whereBetween('price', $price);
        })
        ->paginate(12);

        $productsItems = $products->getCollection();

        $productsItems =  $productsItems->map(function ($product) {
            $price = $product->prices[0];
            $productStock = $product->description->products->sum('stock.quantity');
            return [
                'id' => $price->id,
                'price' => $price->price,
                'promoPrice' => $price->promoPrice,
                'pivot' => $price->pivot,
                'description' => $product->description()->select('id', 'label', 'file_path', 'ar_label')->first(),
                'stock' => $productStock > 0 ? $productStock : 0
            ];
        });
        $products->setCollection($productsItems);
        
        return response()->json(['prices' => $products], 200);
    }

    public function getProductsGuests_old(Request $request) {

        // $item['stock'] = $product->description->products->sum('stock.quantity');
        // $item['description'] = $product->description()->select('id', 'label', 'file_path', 'ar_label')->first();


        $productListId = $this->stockOperationRepository->getPlanifiedProducts();

        $whereHasQuery = [
            "products" => function($query) use ($productListId) {
                $query->whereIn("Product.id", $productListId);
            }
        ];

        if($request->tags && $request->tags != "") {
            $tags = explode(",", $request->tags);
            $whereHasQuery["tags"] = function($query) use ($tags) {
                $query->whereIn("tag", $tags);
            };
        }
        $with = ["products.description.category", "products.description.products.detailKeys.detailKey", "products.element", "tags", "category", "products.detailKeys.detailKey", "reviews:rate,priceId"];

        // $prices = $this->priceRepository->getPricesQuery(
        //     $entrepriseId = null,
        //     $with,
        //     $whereHasQuery
        // )
        // ->paginate(10);
        $filters        = $request->filters;
        $stock = $color = $size = $age = $price = $category = $parentCategory = $searchValue = null;
        if($filters){
            $category       = array_key_exists('category', $filters) ? $filters['category'] : null;
            $parentCategory = array_key_exists('parentCategories', $filters) ? $filters['parentCategories'] : null;
            $stock          = array_key_exists('stock', $filters) ? $filters['stock'] : null;
            $color          = array_key_exists('color', $filters) ? $filters['color'] : null;
            $size           = array_key_exists('size', $filters) ? $filters['size'] : null;
            $age            = array_key_exists('age', $filters) ? $filters['age'] : null;
            $price          = array_key_exists('price', $filters) ? $filters['price'] : null;
            $searchValue    = array_key_exists('searchValue', $filters) ? $filters['searchValue'] : null;
        }

        $products = Product::select('id', 'image', 'product_description_id')
            ->orderBy('created_at', 'desc')
            ->groupBy('product_description_id');

        $products = $products->whereHas('description', function ($query) use($category, $parentCategory, $searchValue) {
            if(isset($searchValue) && $searchValue){
                $query->where('label', 'LIKE', '%'. $searchValue .'%');
            }
            $query->where('label', '!=', 'Livraison')
            ->whereHas('availability', function($query) {
                $query->where('is_available', true);
            });
            if(isset($parentCategory) && $parentCategory){
                $query->whereHas('category', function ($catQuery) use($parentCategory){
                    $catQuery->where('id', $parentCategory['id']);
                    $catQuery->orWhere('parent_id', $parentCategory['id']);
                });
            }
            else if(isset($category) && $category != 'all') $query->whereHas('category', function ($query) use ($category){
                 $query->where('tag', $category);
            });
        });
        
        if($stock == 'in-stock') $products = $products->whereHas('stock', function($stockQuery) {
            $stockQuery->where('quantity', '>', 0);
        });
        if($stock == 'out-stock') $products = $products->whereHas('stock', function($stockQuery) {
                $stockQuery->where('quantity', '=', 0);
                $stockQuery->orWhere('quantity', '<', 0);
            })->orWhereDoesntHave('stock');
        
        if(isset($color) && count($color) > 0){
            $products = $products->whereHas('detailKeys', function($colorQuery) use($color){
                $colorQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'color');
                    $keyQuery->orWhere('tag', 'couleurs');
                })
                ->whereIn('value', $color);
            });
        }
        if(isset($size) && count($size) > 0){
            $products = $products->whereHas('detailKeys', function($sizeQuery) use($size){
                $sizeQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'size');
                    $keyQuery->orWhere('tag', 'taille');
                })
                ->whereIn('value', $size);
            });
        }

        if(isset($age) && count($age) > 0){
            $products = $products->whereHas('detailKeys', function($ageQuery) use($age){
                $ageQuery->whereHas('detailKey', function($keyQuery) {
                    $keyQuery->where('tag', 'age');
                })
                ->whereIn('value', $age);
            });
        }

        $products = $products->whereHas('prices', function ($subQuery) use($with, $whereHasQuery, $price){
            $subQuery->where("isHidden", false);
            // if(sizeOf($with) > 0) {
            //     array_map(function($relation) use ($subQuery) {
            //         $subQuery = $subQuery->with($relation);
            //     }, $with);
            // }
            // if(sizeOf($whereHasQuery) > 0) {
            //     $relationNames = array_keys($whereHasQuery);
            //     array_map(function($name) use ($subQuery, $whereHasQuery) {
            //         $subQuery = $subQuery->whereHas($name, $whereHasQuery[$name]);
            //     }, $relationNames);
            // }
            $subQuery->where("isStandard", false)
            ->whereHas('tags', function ($tagQuery) {
                $tagQuery->where('tag', '!=' ,'delivery_fees');
            })
            ->orWhereDoesntHave('tags');
            // ->has("products", "=", 1);

            if(isset($price) && count($price) > 0){
                $subQuery->whereBetween('price', $price);
            }
        })
        ->paginate(12);

        $data = [];
        foreach ($products as $product) {
            $price = $product->prices[0];
            $item['id'] = $price->id;
            $item['price'] = $price->price;
            $item['promoPrice'] = $price->promoPrice;
            $item['pivot'] = $price->pivot;
            $item['description'] = $product->description()->select('id', 'label', 'file_path', 'ar_label')->first();
            $item['stock'] = $product->description->products->sum('stock.quantity');
            array_push($data, $item);
        }

        return response()->json([
            'prices' => $data,
            'descriptions' => $products
        ], 200);

    }

    public function getProductsDetailKeys($id)
    {
        $productDescription = Product::find($id)->description;

        $productDetailKeys = ProductDetailKey::select('value', 'id', 'fr_value', 'product_id', 'detail_key_id')
        ->whereHas('product', function ($query) use($productDescription){
            $query->where('product_description_id', $productDescription->id);
            $query->whereNotNull('file_path');
        })
        ->whereHas('detailKey', function ($query) {
            $query->whereIn('tag', ['taille', 'couleurs']);
        })
        ->with('detailKey')
        ->with('product')
        // ->groupBy('product_id')
        ->get();

        $productColorsDetails = [];
        $productSizeDetails = [];

        foreach ($productDetailKeys as $detail) {
            
            $data = [
                'id' => $detail->id,
                'product_id' => $detail->product_id,
                'value' => $detail->value,
                'tag' => $detail->detailKey->tag,
                'file_path' => $detail->product->file_path,
                'label' => $detail->fr_value
            ];

            if($detail->detailKey->tag == 'couleurs') $productColorsDetails[] = $data;
            else if($detail->detailKey->tag == 'taille') $productSizeDetails[] = $data;
        }

        return (object) [
            'colors' => $productColorsDetails,
            'sizes' => $productSizeDetails
        ];
    }

    public function getAllProductsDetailKeys_old(Request $request)
    {
        $products_ids = $request->data;
        $productsDetailKeys = [];
        

        foreach ($products_ids as $id)
        {
            $productDetailKeys = $this->getProductsDetailKeys($id);
            $exist_colors = $exist_sizes = $productColors = $productSizes = [];

            foreach ($productDetailKeys->colors as $color)
            {
                if(!in_array($color["value"], $exist_colors)){
                    $productColors[] = $color; 
                    $hex_colors[] = $color["value"];
                } 
            }

            foreach ($productDetailKeys->sizes as $size)
            {
                if(!in_array($size["value"], $exist_sizes)){
                    $productSizes[] = $size;
                    $exist_sizes[] = $size["value"];
                } 
            }

            $productsDetailKeys[] = [
                "productId" => $id,
                "productColors" => $productColors,
                "productSizes" => $productSizes
            ];
        }

        return response()->json(['productsDetailKeys' => $productsDetailKeys]);
    }

    public function getAllProductsDetailKeys(Request $request) {
        
        function getDetails($tags) {
            return ProductDetailKey::select('value', 'fr_value as label')
            ->whereHas('product', function ($query) {
                $query            
                ->whereHas('prices', function ($query) {
                    $query
                    ->where("isHidden", false)
                    ->where("isStandard", false)
                    ->where(function($query){
                        $query->whereHas('tags', function ($query) {
                            $query->where('tag', '!=' ,'delivery_fees');
                        })
                        ->orWhereDoesntHave('tags');
                    });
                });
            })
            ->whereHas('detailKey', function ($query) use($tags){
                $query->whereIn('tag', $tags);
            })
            ->groupBy('value');
        }

        return response()->json([
            "colorsKeys" => getDetails(['couleurs', 'color'])->get(),
            "sizesKeys" => getDetails(['taille'])->get()
        ], 200);
    }


    public function getPaginatedSoldesProducts($offset){
        
        $stock = 'in-stock';
        
        $products = Product::select('id', 'image', 'product_description_id')
        ->whereHas('prices', function ($query) {
            $query
            ->where("isHidden", false)
            ->where("isStandard", false)
            ->where(function($query){
                $query->whereHas('tags', function ($query) {
                    $query->where('tag', '!=' ,'delivery_fees');
                })
                ->orWhereDoesntHave('tags');
            });
        })
        ->whereHas('description', function ($query) use($stock) {
            $query
            ->where('label', '!=', 'Livraison');

            $description_ids = $this->getDescriptionIds_dependsOnStock($stock);

            $query
            ->whereIn('id', $description_ids)
            ->whereHas('availability', function($query) {
                $query
                ->where('is_available', true);
            });
        })
        ->orderBy('created_at', 'desc')
        ->groupBy('product_description_id')
        ->offset($offset)
        ->limit(12)
        ->get();

        $data = [];
        foreach ($products as $product) {
            $price = $product->prices[0];
            $item['id'] = $price->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['promoPrice'] = $price->promoPrice;
            $item['pivot'] = $price->pivot;
            $item['category'] = $product->description->category->tag;
            $item['description'] = $product->description()->select('id', 'label', 'file_path', 'ar_label')->first();
            $item['stock'] = $product->description->products->sum('stock.quantity');
            array_push($data, $item);
        }

        return response()->json([
            'prices' => $data,
        ], 200);
    }

    public function getProductsHome(Request $request)
    {
        if($request->tags && $request->tags != "") {
            $tags = explode(",", $request->tags);
        }

        if($request->tags == "news") $tags = null;

        $products = Product::select('id', 'file_path', 'product_description_id')
        ->whereHas('prices', function ($query) use($tags){
            $query
            ->where("isHidden", false)
            ->where("isStandard", false);
            
            isset($tags) && $tags && $query->whereHas('tags', function ($query) use($tags){
                $query->whereIn('tag', $tags);
            });
        })
        ->whereHas('description', function ($query) {
            $query->where('label', '!=', 'Livraison')
            ->whereHas('availability', function($query) {
                $query->where('is_available', true);
            });
        })
        ->whereHas('stock', function($query) {
            $query->where('quantity', '>', 0);
        })
        ->groupBy('product_description_id')
        ->latest('created_at')
        ->limit(4)
        ->get();

        $data = [];
        foreach ($products as $product) {
            $price = $product->prices[0];
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $product->description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $product->description->category->tag;
            $item['file_path'] = $product->file_path;
            $item['stock'] = $product->stock->quantity;

            $item['color'] = $product->detailKeys()->select('value', 'id', 'fr_value as label')->whereHas('detailKey', function ($query) {
                $query->whereIn('tag', ['couleurs', 'color']);
            })->first();
            
            $item['color'] && $item['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $item['color']->value)->with('product.stock')->get();
    
            $item['productColors'] = $this->getColorsByProductDescription($product->product_description_id);
            array_push($data, $item);
        }

        return response()->json([
            'products' => $data,
        ], 200);
    }


    public function getColorsByProductDescription($product_description_id){

        return ProductDetailKey::select('value', 'id', 'product_id')
        ->whereHas('product', function ($query) use($product_description_id){
            $query
            ->where('product_description_id', $product_description_id)

            ->whereHas('prices', function ($query) {
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
            })

            ->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->whereHas('detailKey', function ($query) {
            $query->whereIn('tag', ['couleurs', 'color']);
        })
        ->get();
    }


    public function getSizesByProductDescription($product_description_id, $color_value = null, $in_stock = null){

        return ProductDetailKey::select('value', 'id', 'product_id')
        ->whereHas('product', function ($query) use($product_description_id, $color_value, $in_stock){
            $query
            ->where('product_description_id', $product_description_id)

            ->whereHas('prices', function ($query) {
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
            });

            $color_value && $query->whereHas('detailKeys', function($query) use($color_value){
                $query->where('value', $color_value);
            });

            $in_stock && $query->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->whereHas('detailKey', function ($query) {
            $query->whereIn('tag', ['taille']);
        });
    }

    
    public function getImagesByProductDescription($product_description_id){

        return Product::select('id as product_id', 'file_path')->where('product_description_id', $product_description_id)
        ->whereNotNull('file_path')
        ->whereHas('stock', function($query) {
            $query->where('quantity', '>', 0);
        })->get();  
    }

    

    public function getProductById(Request $request){

        $product_id = (int) $request->product_id;
        $only_swichting_color = $request->query('onlySwitchingColor');

        $product = Product::select('id', 'file_path', 'product_description_id')
        ->where('id', $product_id)
        ->first();

        if(is_null($product_id)) throw new \Exception("We couldn't find any product id $product_id");

        $data = [];
        $price = $product->prices[0];

        $data['id'] = $product->id;
        $data['file_path'] = $product->file_path;
        $data['stock'] = $product->stock->quantity;
        $data['price_id'] = $price->id;
        $data['price'] = $price->price;
        $data['promoPrice'] = $price->promoPrice;
        $data['tags'] = $price->tags->pluck('tag');

        $color = $data['color'] = $product->detailKeys()->select('value', 'id', 'fr_value as label')->whereHas('detailKey', function ($query) {
            $query->whereIn('tag', ['couleurs', 'color']);
        })->first();    
        
        $color && $data['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $color->value, $in_stock = true)->with('product.stock')->get();

        if(is_null($only_swichting_color)){

            $data['label'] = $product->description->label;
            $data['description'] = $product->description->description;
            $data['productColors'] = $this->getColorsByProductDescription($product->product_description_id);
            $data['productSizes'] = $this->getSizesByProductDescription($product->product_description_id)->get();
            $data['productImages'] = $this->getImagesByProductDescription($product->product_description_id);
        }

        return response()->json(['product' => $data], 200);
    }


    public function getSelectedProduct(Request $request){

        $description_id = (int) $request->description_id;
        $color_id = (int) $request->color_id;

        $product = Product::select('id', 'file_path', 'product_description_id')
        ->where('product_description_id', $description_id)

        ->whereHas('details', function($query) use($color_id){
            $query->where('ProductDetailkey.id', $color_id);
        })
        ->first();

        if(is_null($product)) throw new \Exception("We couldn't find any product with description id $description_id and color $color_id");

        $data = [];

        $data['id'] = $product->id;
        $data['file_path'] = $product->file_path;
        $data['price_id'] = $product->prices[0]->id;
        $data['stock'] = $product->stock->quantity;
        $data['color'] = $product->detailKeys()->select('value', 'id', 'fr_value as label')->whereHas('detailKey', function ($query) {
            $query->whereIn('tag', ['couleurs', 'color']);
        })->first();

        // $data['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $data['color']->value, $in_stock = true)
        // ->with('product.stock')->get();

        return response()->json(['product' => $data], 200);
    }

    public function getProducts(Request $request) {

        $user = \JWTAuth::toUser();
        // $nowDate = Carbon::parse($request->header('User-Date'));
        $entreprise = $user->entreprise;

        $productListId = $this->stockOperationRepository->getPlanifiedProducts();

        $whereHasQuery = [
            // "products" => function($query) use ($productListId) {
            //     $query->whereIn("Product.id", $productListId);
            // }
        ];

        if($request->tags && $request->tags != "") {
            $tags = explode(",", $request->tags);
            $whereHasQuery["tags"] = function($query) use ($tags) {
                $query->whereIn("tag", $tags);
            };
        }

        $withQuery = ["products", "tags", "category"];

        if($entreprise) {
            $pricesQuery = $this->priceRepository->getPricesQuery(
                $entreprise,
                $withQuery,
                $whereHasQuery
            );
        } else {
            $pricesQuery = $this->priceRepository->getPricesQuery(
                $entrepriseId = null,
                $withQuery,
                $whereHasQuery
            );
        }

        $prices = $pricesQuery->get();

        if($entreprise) $prices = $entreprise->updatePrices($prices);

        return response()->json([
            'prices' => $prices
        ], 200);

    }

    public function findProducts(Request $request) {

        $search = $request->search;
        $withQuery = ["products.description.category", "products.description.products", "tags", "category", "products.detailKeys.detailKey"];
        $pricesQuery = $this->priceRepository->findPricesQuery($withQuery, $search);
        // ProductDescription::whereHas('products', function ($query) 
        // use($with, $whereHasQuery, $stock, $color, $size, $age, $price){
        //     if($stock == 'in-stock') $query->whereHas('stock', function($stockQuery) {
        //             $stockQuery->where('quantity', '>', 0);
        //         });


        //     $query->whereHas('prices', function ($subQuery) use(){
        //         $subQuery->where("isHidden", false);
        //         $subQuery->where("isStandard", false)
        //         ->has("products", "=", 1);
                
        //     });
        // })->with('products.prices.products.description.products.detailKeys.detailKey', 'products.prices.products.description.category',
        //     'products.prices.tags', 'products.prices.products.element', 'products.prices.category', 'products.prices.products.detailKeys.detailKey',
        //     'products.prices.reviews:rate,priceId', 'products.prices.products.description.products.details', 'products.prices.products.description.products.prices',
        //     'products.prices.products.description.products.description', 'products.prices.products.stock', 'products.prices.products.description.products.stock')
        // ->paginate(10);

        return response()->json([
            'prices' => $pricesQuery
        ], 200);
    }
}
