<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;
use App\Models\DeliveryState;
use App\Models\EtatCmd;
use App\Models\ProductAvailability;
use App\Models\StockOperationProduct;
use App\Models\TransportCompany;
use App\Repositories\PriceRepository;
use App\Repositories\BrandRepository;
use App\Repositories\PriceReviewRepository;
use Carbon\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Response as GuzzleResponse;
use Illuminate\Support\Facades\Schema;

use App\Repositories\StockOperationRepository;
use App\Repositories\StockOperationProductRepository;
use App\Repositories\StockOperationTypeRepository;
use Illuminate\Support\Facades\Cache;

class ProductController extends AppBaseController
{

    protected $priceRepository;
    protected $priceReviewRepository;
    protected $stockOperationRepository;
    protected $stockOperationProductRepository;
    protected $stockOperationTypeRepository;

    public function __construct(
        PriceRepository $priceRepository,
        PriceReviewRepository $priceReviewRepository,
        StockOperationRepository $stockOperationRepository,
        StockOperationProductRepository $stockOperationProductRepository,
        StockOperationTypeRepository $stockOperationTypeRepository

    ){
        $this->priceRepository = $priceRepository;
        $this->priceReviewRepository = $priceReviewRepository;
        $this->stockOperationRepository = $stockOperationRepository;
        $this->stockOperationProductRepository = $stockOperationProductRepository;
        $this->stockOperationTypeRepository = $stockOperationTypeRepository;
    }

    public function getProductDetail(Request $request, $priceId) {
        $product = $this->priceRepository->advancedFind($priceId);
        return response()->json([
            "product" => $product
        ]);
    }

    public function addReview(Request $request, $priceId) {

        $review = $this->priceReviewRepository->create([
            "rate" => $request->rate,
            "name" => $request->name,
            "email" => $request->email,
            "title" => $request->title,
            "comment" => $request->comment,
            "priceId" => $priceId
        ]);

        return response()->json([
            "review" => $review
        ]);

    }


    public function syncData()
    {
        \Log::debug('syncData called at : '. Carbon::now());
        // $user = \JWTAuth::toUser();
        
        $lastSyncedDate = \DB::table('settings')->where('key', 'site.last_sync_date')->first();
        $newDate = Carbon::now();
        $firstDate = Carbon::parse($lastSyncedDate->value);
        $now = Carbon::parse($newDate);
        $difference_minutes = $firstDate->diffInMinutes($now);
        if($difference_minutes < 60){
            return response()->json([
                "status" => false,
                'message' => "La différence du temps entre 2 synchronisations doit etre min 60 minutes"
            ]);
        }
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION')]
        ]);
        $criterias['isWeb'] = true;
        $query = '?withoutFilteringOtherStores=true';
        if($lastSyncedDate){
            $criterias['lastUpdatedAt'] = (new Carbon($lastSyncedDate->value))->toISOString();
            $syncDate = (new Carbon($lastSyncedDate->value))->utc()->subHour()->format('Y-m-d\TH:i:s');
            $query .= '&lastUpdatedAt=' . $syncDate;
        }

        $response = $client->get('/catalogueV2AP/PointOfSell/sync'. $query, $criterias);

        $data = json_decode($response->getBody());
        \DB::statement('SET FOREIGN_KEY_CHECKS=0;');
        $operationType = $this->stockOperationTypeRepository->getDefault();

        $stockOperation = \DB::table('StockOperation')->first();
        if(!$stockOperation) $stockOperation = $this->stockOperationRepository->create([
                "magasinId" => 1,
                "userId" => 1,
                "typeId" => $operationType->id,
                'planifiedFor' => 0
            ]);

        foreach ($data as $model => $modelData) {
            \Log::debug($model);
            if($model == "PriceCategory") continue;

            foreach ($modelData as $data) {
                $id = $data->id;
                $recordExist = \DB::table($model)->where('id', $id)->first();
                if($model == "ProductCategory") $priceCatExist = \DB::table('PriceCategory')->where('id', $id)->first();

                $data->created_at = Carbon::parse($data->created_at)->format('Y-m-d h:i:s');
                $data->updated_at = Carbon::parse($data->updated_at)->format('Y-m-d h:i:s');
                if($model != "PriceTagPrice" && $data->deleted_at) $data->deleted_at = Carbon::parse($data->deleted_at)->format('Y-m-d h:i:s');

                if ( property_exists($data, "file_id") && $data->file_id > 0) {
                    try {
                        $url = str_replace("{1}", $data->file_id, config('urls.get-file'));
                        file_put_contents('php://stderr', print_r([
                            "message" => "Calling URL " . $url
                        ], TRUE));
                        $response = $client->get($url);
                        $headers = $response->getHeaders();
                        $fileName = (string) \Str::uuid();
                        $path = '/pos-images/' .$fileName . '.' . $headers["file-extension"][0];
                        \Storage::put('public' . $path, $response->getBody());
                        $data->file_path = $path;
                    } catch (\Exception $ex) {
                        file_put_contents('php://stderr', print_r([
                            "error" => $ex->getMessage()
                        ], TRUE));
                        \Log::info("Exception on getting image file " . $ex->getMessage());
                    }
                }

                $allData = (array) $data;

                if($recordExist || isset($priceCatExist)) \DB::table($model)->where('id', $id)->update($allData);
                else \DB::table($model)->insert($allData);

                if($model == "ProductCategory"){
                    if(isset($priceCatExist)) \DB::table('PriceCategory')->where('id', $id)->update($allData);
                    else \DB::table('PriceCategory')->insert($allData);
                }

                if($model == 'Product' && !$this->stockOperationProductRepository->getProducts($data->id)){
                    $this->stockOperationProductRepository->create([
                        'productId' => $data->id,
                        'operationId' => $stockOperation->id,
                        'quantity' => 0
                    ]);
                }

                if($model == 'ProductDescription'){
                    $productDescriptionAv = ProductAvailability::where('product_description_id', $data->id)->first();
                    if(!$productDescriptionAv) ProductAvailability::create([
                        'is_available' => true,
                        'product_description_id' => $data->id
                    ]);
                }

            }

        }
        \DB::statement('SET FOREIGN_KEY_CHECKS=1;');

        $commandTables = json_decode($client->get(config('urls.commandTables'))->getBody());
        $transportCompanies = $commandTables->TransportCompany;
        $deliveryStates = $commandTables->DeliveryState;
        $commandStates = $commandTables->EtatCmd;
        foreach ($deliveryStates as $state) {
            DeliveryState::updateOrCreate([
                'intitule' => $state->intitule,
                'tag' => $state->tag,
            ], [
                'id' => $state->id,
            ]);
        }
        foreach ($transportCompanies as $company) {
            $recordExist = \DB::table('TransportCompany')->where('id', $company->id)->first();
            if(isset($recordExist)) \DB::table('TransportCompany')->where('id', $company->id)->update([
                    'name' => $company->name,
                    'created_at' => $company->created_at,
                    'updated_at' => $company->updated_at,
                    'deleted_at' => $company->deleted_at
                ]);
            else \DB::table('TransportCompany')->insert([
                'id' => $company->id,
                'name' => $company->name,
                'created_at' => $company->created_at,
                'updated_at' => $company->updated_at,
                'deleted_at' => $company->deleted_at
            ]);
        }
        foreach ($commandStates as $commandState) {
            EtatCmd::updateOrCreate([
                'id' => $commandState->id,
                'intitule' => $commandState->intitule,
                'tag' => $commandState->tag,
                'commentaire' => $commandState->commentaire ?? "---",
                'created_at' => $commandState->createdAt,
                'updated_at' => $commandState->updatedAt,
                'deleted_at' => $commandState->deletedAt
            ], [
                'id' => $commandState->id,
            ]);
        }

        \DB::table('settings')->where('key', 'site.last_sync_date')->update([
            'value' => Carbon::now()->format('Y-m-d H:i:s'),
        ]);

        syncProductsStock();

        return response()->json([
            "status" => "Ok",
            'message' => "La synchronisation à été traitée"
        ]);
    }
}
