<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;
use App\Repositories\GeographicPointRepository;
use App\Repositories\AdresseRepository;
use App\Repositories\CommandRepository;
use App\Repositories\UserRepository;
use App\Repositories\RoleRepository;
use App\Repositories\ClientRepository;

class ProfileController extends AppBaseController
{

    protected $geographicPointRepository;
    protected $adresseRepository;
    protected $commandRepository;
    protected $userRepository;
    protected $roleRepository;
    protected $clientRepository;

    public function __construct(
        GeographicPointRepository $geographicPointRepository,
        AdresseRepository $adresseRepository,
        CommandRepository $commandRepository,
        UserRepository $userRepository,
        RoleRepository $roleRepository,
        ClientRepository $clientRepository
    ){
        $this->geographicPointRepository = $geographicPointRepository;
        $this->adresseRepository = $adresseRepository;
        $this->commandRepository = $commandRepository;
        $this->userRepository = $userRepository;
        $this->roleRepository = $roleRepository;
        $this->clientRepository = $clientRepository;
    }

    public function updateProfile(Request $request) {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $role = $user->role;
        if($role->name == "prospet") {
            $role = $this->roleRepository->allQuery([
                "name" => "client"
            ])->first();
        }

        $user = $this->userRepository->update([
            "name" => $request->name,
            "email" => $request->email,
            "password" => $request->password,
            "role_id" => $role->id
        ], $user->id);

        $client = $this->clientRepository->update([
            "phone" => $request->phone
        ], $client->id);

        return response()->json([
            "status" => "ok"
        ]);

    }

    public function getFavoritProducts() {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $likedProducts = $client->likedPrices()
            ->with("products.description.products.detailKeys.detailKey", "products.detailKeys.detailKey")
            ->get();

        return response()->json([
            "likedPrices" => $likedProducts
        ]);

    }

    public function getCommands() {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $commands = $this->commandRepository->getClientCommands(
            $client->id
        );

        $commands->each->append("total");

        return response()->json([
            'commands' => $commands
        ], 200);

    }

    public function deleteAddress($id) {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $addressListId = $this->adresseRepository->delete($id);

        return response()->json([
            'status' => 'ok'
        ], 200);

    }

    public function getAddresses() {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $addressListId = $this->commandRepository->all(
            $search = [ "clientId" => $client->id ],
            $skip = null,
            $limit = null,
            $columns = ['adresseId']
        )->toArray();

        $addressObjs = $this->adresseRepository->inFilter(
            $search = [ "id" => $addressListId ]
        );

        return response()->json([
            'addresses' => $addressObjs
        ], 200);

    }

    public function getCommand($id) {

        $command = $this->commandRepository->find($id);

        $cart = $command->prices->map(function($price) {
            if($price->products->count() == 0) { return null; } else {
                return [
                    "price" => $price,
                    "products" => $price->products,
                    "quantity" => $price->pivot->quantity
                ];
            }
        })->toArray();

        return response()->json([
            'cart' => array_values(array_filter($cart))
        ], 200);

    }

}
