<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Tymon\JWTAuth\Exceptions\JWTException;
use Illuminate\Validation\ValidationException;
use App\Repositories\UserRepository;
use App\Repositories\CommandRepository;

use App\Mail\ForgotPasswordMail;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    protected $userRepository;
    protected $commandRepository;

    public function __construct(
        UserRepository $userRepository,
        CommandRepository $commandRepository
    ){
        $this->userRepository = $userRepository;
        $this->commandRepository = $commandRepository;
    }

    public function forgotPassword(Request $request) {

        $email = $request->get('email');
        $hash = $request->get('hash');
        $password = $request->get('password');

        if($email) {

            $user = $this->userRepository->all([
                "email" => $email
            ])->first();

            if(!$user) {
                return response()->json([
                    "status" => "error",
                    "input" => "email",
                    "message" => "Votre email n'existe pas"
                ], 422);
            }

            $user->hash = (string) \Str::uuid();
            $user->save();

            $message = (new ForgotPasswordMail(
                $user->hash
            ))->onQueue('emails');

            \Mail::to($email)
                ->cc([])
                ->queue($message);

            return response()->json([
                'input' => 'email',
                "status" => "OK",
                "message" => "Nous vous avons envoyé un e-mail pour reinitialiser votre mot de passe"
            ]);

        } else if ($password && $hash) {

            $user = $this->userRepository->all([
                "hash" => $hash
            ])->first();

            if(!$user) {
                return response()->json([
                    "status" => "error",
                    'input' => 'hash',
                    "message" => "votre hash est expiré"
                ], 422);
            }

            $user->password = \Hash::make($password);
            $user->hash = null;

            $user->save();

            return response()->json([
                "status" => "OK",
                "message" => "Vous avez réussi à réinitialiser votre mot de passe",
                'input' => 'hash'
            ]);

        }

        return response()->json([
            "status" => "error",
            "message" => "Checkout please your inputs validations"
        ], 422);

    }

    public function loginWithToken(Request $request) {

        $token = "";

        $email = $request->get('email');
        $phone = $request->get('phone');
        $password = $request->get('password');
        $remember_me = $request->get("remember_me", 0);

        $validator = \Validator::make($request->all(), [
            'password' => 'required'
        ]);

        if ($validator->fails()){
            throw new ValidationException($validator);
        }

        try {
            $token = $this->userRepository->getToken(
                $email,
                $phone,
                $password,
                $remember_me
            );
        } catch (\Exception $ex) {
            return response()->json([
                'message' => $ex->getMessage()
            ], 422);
        }


        return response()->json($token);

    }
}
