<?php

namespace App\Http\Controllers\Voyager;

use Illuminate\Http\Request;

use Voyager;
use Carbon\Carbon;
use App\Repositories\CommandRepository;
use App\Repositories\ProductRepository;

class ReportingController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController
{

    protected $commandRepository;
    protected $productRepository;

    public function __construct(CommandRepository $commandRepository, ProductRepository $productRepository){
        $this->commandRepository = $commandRepository;
        $this->productRepository = $productRepository;
    }

    public function index(Request $request) {

        $from = $request->from;
        $to = $request->to;

        $fromDate = Carbon::parse($from);
        $toDate = Carbon::parse($to);

        if($request->has("productId") && $request->get("productId") != "") {
            $commands = $this->commandRepository->findBetween(
                [$fromDate, $toDate],
                [$request->get("productId")]
            );
        } else {
            $commands = $this->commandRepository->findBetween(
                [$fromDate, $toDate]
            );
        }

        $diffrentInDays = $fromDate->diffInDays($toDate);

        $reportedRecap = [];

        if ($diffrentInDays < 15) {

            $chiffreAffaire = $commands->reduce(function ($carry, $command) {
                return $carry + $command->total;
            }, 0);

            $commandsCount = $commands->count();

            $cartAvg = $chiffreAffaire/($commandsCount != 0 ? $commandsCount : 1);

            $budget = $commands->reduce(function ($carry, $command) {
                return $carry + $command->buying;
            }, 0);

            $reportedRecap = [[
                "chiffreAffaire" => $chiffreAffaire,
                "commandsCount" => $commandsCount,
                "cartAvg" => $cartAvg,
                "budget" => $budget
            ]];

        } else if($diffrentInDays == 15) {

            $groupedCommandsByDay = $commands->groupBy(function ($command, $key) {
                return $command->created_at->toDateString();
            });

            $reportedRecap = $groupedCommandsByDay->keys()->map(function ($createdAtKey) use ($groupedCommandsByDay) {

                $commands = $groupedCommandsByDay[$createdAtKey];

                $chiffreAffaire = $commands->reduce(function ($carry, $command) {
                    return $carry + $command->total;
                }, 0);

                $commandsCount = $commands->count();

                $cartAvg = $chiffreAffaire/($commandsCount != 0 ? $commandsCount : 1);

                $budget = $commands->reduce(function ($carry, $command) {
                    return $carry + $command->buying;
                }, 0);

                return [
                    "chiffreAffaire" => $chiffreAffaire,
                    "commandsCount" => $commandsCount,
                    "cartAvg" => $cartAvg,
                    "budget" => $budget,
                    "createdAt" => $createdAtKey
                ];

            });

        } else if($diffrentInDays > 15) {

            $groupedCommandsByDay = $commands->groupBy(function ($command, $key) {
                return $command->created_at->format('F\/Y');
            });

            $reportedRecap = $groupedCommandsByDay->keys()->map(function ($createdAtKey) use ($groupedCommandsByDay) {

                $commands = $groupedCommandsByDay[$createdAtKey];

                $chiffreAffaire = $commands->reduce(function ($carry, $command) {
                    return $carry + $command->total;
                }, 0);

                $commandsCount = $commands->count();

                $cartAvg = $chiffreAffaire/($commandsCount != 0 ? $commandsCount : 1);

                $budget = $commands->reduce(function ($carry, $command) {
                    return $carry + $command->buying;
                }, 0);

                return [
                    "chiffreAffaire" => $chiffreAffaire,
                    "commandsCount" => $commandsCount,
                    "cartAvg" => $cartAvg,
                    "budget" => $budget,
                    "createdAt" => $createdAtKey
                ];

            });

        }

        $reportedCommands = $commands->map(function($command) {
            $codePromo = $command->codeReductions()->first();
            return [
                "id" => $command->id,
                "state" => $command->state->toArray(),
                "user" => $command->client->user->toArray(),
                "entreprise" => $command->client->user->entreprise ? $command->client->user->entreprise->toArray() : [],
                "total" => $command->total,
                "total_reduction" => $command->total_reduction,
                "code_promo" => $codePromo ? $codePromo->tag : "---"
            ];
        });

        $products = $this->productRepository->all();

        return Voyager::view("voyager::report.index", compact(
            "from",
            "to",
            "products",
            "reportedCommands",
            "reportedRecap"
        ));
    }

}
