<?php

use App\Models\StockOperationProduct;

function is_true($val, $return_null=false){
    \Log::debug($val);
    $boolval = ( is_string($val) ? filter_var($val, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE) : (bool) $val );
    return ( $boolval===null && !$return_null ? false : $boolval );
}

function sendOneSignalNotification($message) {
    try {
        $headings = preg_replace('/\d/', '', $message->subject);
        $content = $message->message;
        \Log::debug('HERE WE ARE');
        \OneSignal::sendNotificationToAll($content, $url = null, $data = null, $buttons = null, $schedule = null, $headings, $subtitle = null);

    } catch (\Exception $ex) {
        \Log::error($ex);
    }
}

function syncProductsStock()
{
    $client = new \GuzzleHttp\Client([
        'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
        'defaults' => [
            'exceptions' => false
        ],
        'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
    ]);

    $selectedStoreId = getStoreIdByRelation('in');
    $condition = $selectedStoreId == config('constants.CENTRAL_ACHAT_ID');

    if($condition) $entityType = config('constants.CENTRAL_ACHAT');
    else $entityType = config('constants.MAGASIN');

    $query = "/?isLive=true&type=$entityType";
    $url = config('urls.sync-stock.subUrl1') . $selectedStoreId . config('urls.sync-stock.subUrl2') . $query;
        
    $response = $client->get($url)->getBody();
    $stockStatus = json_decode($response);

    $allOperationProducts = StockOperationProduct::select('productId')->get();
    $stockProducts = collect($stockStatus->stockStatus);

    foreach ($allOperationProducts as $oprationProduct) {

        $stock = $stockProducts->where('produitInboundId', $oprationProduct->productId)->first();
        $quantity = $stock && $stock->quantite > 0 ? $stock->quantite : 0;

        StockOperationProduct::where('productId', $oprationProduct->productId)->update([
            'quantity' => $quantity
        ]);
    }
    
    return true;
}

 function getStoreIdByRelation($type, $product_id = null){
        
    $magasin_id = null;
    $key = "site.magasin_client_id_$type";

    // we store stock relations in json inside settings table
    $magasin_stock_relation = \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->first();
    if($magasin_stock_relation) $magasin_stock_relation = $magasin_stock_relation->value;

    // stock relation can be null if settings table emty and product_id can be null when we call him from another block
    if(!is_null($magasin_stock_relation) && !is_null($product_id)) {
        $relation = collect(json_decode($magasin_stock_relation))->where('product_id', $product_id)->first();
        $magasin_id = $relation ? $relation->magasin_id : null;
        
    }
    // if stock have no relation with product livraison
    if(is_null($magasin_id)) {
        $magasin_client_id = \DB::table('settings')->where('key', $key)->first();
        $magasin_id = $magasin_client_id ?  $magasin_client_id->value : 'online';
    }

    // if stock have no relations and have no default magasin id
    if(is_null($magasin_id) || $magasin_id == 'online'){

        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $url = config('urls.get-store-by-name').'?name=out&isOnline=true';
        $data = $client->get($url)->getBody();
        $magasin_id = json_decode($data)->id;
    }

    return $magasin_id;
}