<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;

use App\Repositories\BlogRepository;
use App\Repositories\BlogCommentRepository;

class BlogController extends AppBaseController
{

    protected $blogRepository;
    protected $blogCommentRepository;

    public function __construct(
        BlogRepository $blogRepository,
        BlogCommentRepository $blogCommentRepository
    ){
        $this->blogRepository = $blogRepository;
        $this->blogCommentRepository = $blogCommentRepository;
    }

    public function getBlogs(Request $request) {

        $paginatedBlogs = $this->blogRepository->getPaginatedBlogs(10);

        return response()->json($paginatedBlogs);

    }

    public function getBlogDetail(Request $request, $slug) {

        $blog = $this->blogRepository->advancedFind($slug);

        $this->blogRepository->update([
            "visitor" => $blog->visitor + 1
        ], $blog->id);

        return response()->json([
            "blog" => $blog
        ]);

    }

    public function addComment(Request $request, $blogId) {

        $blog = $this->blogRepository->find($blogId);

        $comment = $this->blogRepository->addComment($blog->id, [
            "name" => $request->name,
            "email" => $request->email,
            "comment" => $request->commentText
        ]);

        return response()->json([
            "comment" => $comment
        ]);

    }

}
