<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Validation\Rule;

use App\Mail\InviteColleguesMail;
use App\Mail\NewCommandMail;

use App\Http\Controllers\Api\AppBaseController;
use App\Jobs\SyncCommandsToPos;
use App\Mail\WelcomeMail;
use App\Models\Command;
use App\Models\CommandPrice;
use App\Models\DeliveryState;
use App\Models\Price;
use App\Models\ProductDescription;
use App\Models\StockOperationProduct;
use App\Models\TransportCompany;
use App\Services\Interfaces\CommandService;

use App\Repositories\PriceRepository;
use App\Repositories\ProductRepository;
use App\Repositories\CombinedProductRepository;
use App\Repositories\PriceCategoryRepository;
use App\Repositories\ProductCategoryRepository;

use App\Repositories\CommandRepository;
use App\Repositories\CommandPriceRepository;
use App\Repositories\CommandReductionRepository;
use App\Repositories\CommandStateRepository;
use App\Repositories\ReductionCodeRepository;

use App\Repositories\UserRepository;
use App\Repositories\ClientRepository;
use App\Repositories\RoleRepository;
use App\Repositories\AdresseRepository;
use App\Repositories\GeographicPointRepository;
use App\Repositories\PayementModeRepository;

use App\Repositories\InviteCodeRepository;
use App\Repositories\ReductionIntervalRepository;
use App\User;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Hash;
use GuzzleHttp\Psr7\Utils;
class CheckoutController extends AppBaseController
{
    protected $commandService;

    protected $priceRepository;
    protected $productRepository;
    protected $combinedProductRepository;
    protected $priceCategoryRepository;
    protected $productCategoryRepository;

    protected $commandRepository;
    protected $commandPriceRepository;
    protected $commandReductionRepository;
    protected $reductionCodeRepository;
    protected $reductionIntervalRepository;

    protected $userRepository;
    protected $clientRepository;
    protected $roleRepository;
    protected $clientAdresseRepository;
    protected $adresseRepository;
    protected $commandStateRepository;
    protected $geographicPointRepository;
    protected $payementModeRepository;

    protected $inviteCodeRepository;

    public function __construct(
        CommandService $commandService,
        CommandStateRepository $commandStateRepository,
        PriceRepository $priceRepository,
        ProductRepository $productRepository,
        CombinedProductRepository $combinedProductRepository,
        PriceCategoryRepository $priceCategoryRepository,
        ProductCategoryRepository $productCategoryRepository,
        CommandRepository $commandRepository,
        CommandPriceRepository $commandPriceRepository,
        CommandReductionRepository $commandReductionRepository,
        ReductionCodeRepository $reductionCodeRepository,
        UserRepository $userRepository,
        ClientRepository $clientRepository,
        RoleRepository $roleRepository,
        AdresseRepository $adresseRepository,
        GeographicPointRepository $geographicPointRepository,
        PayementModeRepository $payementModeRepository,
        InviteCodeRepository $inviteCodeRepository,
        ReductionIntervalRepository $reductionIntervalRepository
    ){
        $this->commandService = $commandService;
        $this->commandStateRepository = $commandStateRepository;
        $this->priceRepository = $priceRepository;
        $this->productRepository = $productRepository;
        $this->combinedProductRepository = $combinedProductRepository;
        $this->priceCategoryRepository = $priceCategoryRepository;
        $this->productCategoryRepository = $productCategoryRepository;
        $this->commandRepository = $commandRepository;
        $this->commandPriceRepository = $commandPriceRepository;
        $this->commandReductionRepository = $commandReductionRepository;
        $this->reductionCodeRepository = $reductionCodeRepository;
        $this->userRepository = $userRepository;
        $this->clientRepository = $clientRepository;
        $this->roleRepository = $roleRepository;
        $this->adresseRepository = $adresseRepository;
        $this->geographicPointRepository = $geographicPointRepository;
        $this->payementModeRepository = $payementModeRepository;
        $this->inviteCodeRepository = $inviteCodeRepository;
        $this->reductionIntervalRepository = $reductionIntervalRepository;
    }

    public function inviteCollegues(Request $request) {

        $inviter = \JWTAuth::toUser();
        $client = $inviter->client;
        $command = null;

        if( array_key_exists("commandId", $request->inviteForm) && $request->inviteForm["commandId"] ) {
            $command = $this->commandRepository->find($request->inviteForm["commandId"]);
        } else {

            $request->validate([
                // 'addressForm.planifiedFor' => 'required',
                'addressForm.lat' => 'sometimes',
                'addressForm.lng' => 'sometimes',
                'addressForm.address' => 'required',
                'commandForm.commands' => 'array'
            ]);

            $commandState = $this->commandStateRepository->getWiatingInvitationState();

            $paymentMode = $this->payementModeRepository->getDefault();

            // $planifiedFor = Carbon::parse($request->addressForm["planifiedFor"]);

            // geolocation of the client
            $geolaction = null;
            if ($request->addressForm["lat"] && $request->addressForm["lng"]) {
                $geolaction = $this->geographicPointRepository->create([
                    "latitude" => round($request->addressForm["lat"], 2),
                    "longitude" => round($request->addressForm["lng"], 2)
                ]);
            }

            // Adresse
            $address = $this->adresseRepository->create([
                "geographicPointId" => $geolaction ? $geolaction->id : null,
                "description" => $request->addressForm["address"]
            ]);

            $command = $this->commandRepository->create([
                'grouped' => array_key_exists("grouped", $request->inviteForm) ? $request->inviteForm["grouped"] : 1,
                'adresseId' => $address->id,
                'stateId' => $commandState->id,
                'paymentModeId' => $paymentMode->id,
                'clientId' => $client->id,
                // 'planifiedFor' => $planifiedFor,
                'comment' => " --- ",
            ]);
        }

        $inviteComment = array_key_exists("comment", $request->inviteForm) ? $request->inviteForm["comment"] : "---";

        $users = array_map(function($email) use ($command, $inviter, $inviteComment) {

            $hash = (string) \Str::uuid();

            $role = $this->roleRepository->allQuery([
                "name" => "prospet"
            ])->first();

            $user = $this->userRepository->create([
                "name" => " annonyme",
                "email" => $email,
                "password" => $hash,
                "role_id" => $role->id
            ]);

            $client = $this->clientRepository->create([
                "userId" => $user->id,
                "phone" => "000 00 00 00"
            ]);

            $inviteCode = $this->inviteCodeRepository->create([
                "commandId" => $command->id,
                "hash" => $hash
            ]);

            $message = (
                new InviteColleguesMail($inviter, $email, $inviteComment, $hash)
            )->onQueue('emails');

            \Mail::to($email)
                ->cc([])
                ->queue($message);

            $userArray = $user->toArray();
            $userArray["client"] = $client->toArray();
            return $userArray;

        }, $request->inviteForm["emails"]);

        return response()->json([
            'command' => $command,
            'address' => $command->address,
            'users' => $users
        ]);

    }

    public function addClient(Request $request) {
        
        $default_password = '123456';

        $request->validate([
            'name' => 'required',
            'phone' => ['nullable', 'string', Rule::unique('users', 'phone')->whereNotNull('verified')],
            'email' => ['nullable', 'email', Rule::unique('users', 'email')->whereNotNull('verified')],
            'password' => 'sometimes|nullable',
            'verified' => 'nullable'
        ]);
        

        $role = $this->roleRepository->getByTag("client");
        $password =  $request->password ? $request->password : $default_password;
        $userFound = User::where('phone', $request->phone)->first();

        $user = null;
        if(!$userFound){
            $user = User::create([
                "name" => $request->name,
                'entrepriseId' => null,
                "role_id" => $role->id,
                "email" => $request->email,
                "phone" => $request->phone,
                "password" => Hash::make($password),
                "verified" => $request->verified ? now() : null
            ]);

            $this->clientRepository->create([
                'userId' => $user->id,
                'phone' => $request->phone
            ]);
            // dd($user->verified);

            // dd($this->userRepository->find($user->id));
            
        }else{
            if($request->name) $userFound->name = $request->name;
            if($request->email) $userFound->email = $request->email;
            if($request->phone) $userFound->phone = $request->phone;
            $userFound->phone = $request->phone;
            $userFound->verified = $request->verified ? now() : null;
            $userFound->save();
            
           // dd($request->verified);

            $token = $this->userRepository->getTokenFromUser($userFound);
            return response()->json($token);
        }

        $token = "";
        try {
            $token = $this->userRepository->getToken(
                $request->email,
                $request->phone,
                $password
            );
        } catch (\Exception $ex) {
            return response()->json([
                'message' => $ex
            ], 422);
        }

        //if($user && $user->email && !$userFound) {
        //    $message = (new WelcomeMail())->onQueue('emails');
        //    \Mail::to($user->email)
        //        ->cc([])
        //        ->queue($message);
        //}

        return response()->json(['token' => $token]);

    }

    public function getRecommendedProducts() {

        $user = \JWTAuth::toUser();
        $entreprise = $user->entreprise;

        if($entreprise) {
            $prices = $this->priceRepository->getRecommendedPricesQuery($entreprise);
        } else {
            $prices = $this->priceRepository->getRecommendedPricesQuery();
        }

        $prices = $prices->with("products.category")->get();

        if($entreprise) $prices = $entreprise->updatePrices($prices);

        return response()->json([
            'prices' => $prices
        ], 200);

    }

    public function getRecommendedProductsGuests() {

        $prices = $this->priceRepository->getRecommendedPricesQuery()
            ->with("products.category")
            ->get();

        return response()->json([
            'prices' => $prices
        ], 200);

    }

    public function getUnauthentifiedPromoCodeReduction($promoCode) {

        $reductionCode = $this->commandReductionRepository->getCodeIfAvailable(
            $promoCode
        );

        return response()->json([
            'reduction' => $reductionCode
        ], 200);

    }


    public function getPromoCodeReduction($promoCode) {
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $response = json_decode($client->get(config('urls.verify-promo-code') .'?code=' . $promoCode)->getBody());


        return response()->json([
            'reduction' => $response->remise
        ], 200);

    }


    public function varify_products_stock($commands){
        
        foreach ($commands as $line) {
        
            $product = Price::find($line['price_id'])->products->first();

            $product_stock = $product->stock->quantity;
            if($product_stock < $line['quantity']) {
                return response()->json(['tag' => 'product_stock_out', 'product_id' => $product->id, 'label' => $product->description->label], 500);
            }
        }
        return false;
    }


    public function addCartcheckOut(Request $request){
        
        // set_time_limit(5);
        
        $user = \JWTAuth::toUser();

        $client = $user->client;

        $reduction = $request->commandForm["reduction"] ? $request->commandForm["reduction"]["valeur"] : 0;
        $codePromo = $request->commandForm["reduction"] ? $request->commandForm["reduction"]["intitule"] : null;

        $request->validate([
            'addressForm.address' => 'sometimes',
            'addressForm.country' => 'sometimes',
            'addressForm.city' => 'sometimes',
            'addressForm.region' => 'sometimes',
            'addressForm.paymentModeTag' => 'required',
            'addressForm.code_postal' => 'sometimes',
            'commandForm.commands' => 'required|array',
            'commandForm.reduction' => 'sometimes',
        ]);
        
        $requestCommandes = $request->commandForm["commands"];
        // \Log::info("requestCommandes : ".json_encode($requestCommandes,JSON_PRETTY_PRINT));
        $stock_out = $this->varify_products_stock($requestCommandes);

        if($stock_out) return $stock_out;
        
        \Log::info("step-validation-passed 1");
        // dd("step-validation-passed");

        $paymentMode = $this->payementModeRepository->getByTag($request->addressForm['paymentModeTag']);
        $commandState = $this->commandStateRepository->getByTag("pending");

        $address = $this->adresseRepository->create([
            "geographicPointId" => null,
            "description" => $request->addressForm["address"] ??  "---"
        ]);
                \Log::info("step-validation-passed 2");

        // dd("step-adresse-passed");

        $command = $this->commandService->addCommand([
            'adresseId' => $address->id,
            'stateId' => $commandState->id,
            'paymentModeId' => $paymentMode->id,
            'clientId' => $client->id,
            'comment' => $codePromo ? "[C.PROMO : $codePromo]" : "",
            'country' => $request->addressForm["country"] ?? 'Maroc',
            'region' => $request->addressForm["region"] ?? 'Casablanca-Settat',
            'city' => $request->addressForm["city"] ?? 'Casablanca',
            'code_postal' => $request->addressForm["code_postal"],
            'totalAmount' => $request->totalAmount,
        ]);
        
        
        $command->trackingNumber = 'w/' . Carbon::now()->format('Y') . '/' . $command->id;
        if($command->clientId == $client->id) $command->stateId = $commandState->id;
        
        $command->save();
        \Log::info("step-validation-passed 3");
        
        \App\Models\Command::where('id', $command->id)->update([
            'reduction' => $reduction,
            'codePromo' => $codePromo
        ]);

        // dd("step-command-passed");
        
        if($request->commandForm["reduction"]) {
            
            $commandRemise = $request->commandForm["reduction"];
            
            \App\Models\CmdRemise::create([
                'cmdId' => $command->id,
                "remiseId" => $commandRemise["id"],
                "localKey" => $commandRemise["local_key"],
                "intitule" => $commandRemise["intitule"],
                "valeur" => $commandRemise["valeur"]
            ]);	
        }

        $local_key = Str::uuid()->toString();
        $deliveryCompany = null;

        foreach ($requestCommandes as $commandLine) {
        

            $transportCompany = TransportCompany::where('price_id', $commandLine["price_id"])->first();

            if($transportCompany) $deliveryCompany = $transportCompany;

            $priceModel = Price::find($commandLine["price_id"]);

            $commandPrice = $this->commandPriceRepository->create([
                "clientId" => $commandLine["clientId"] && $commandLine["clientId"] != "" ?  $commandLine["clientId"] : $client->id,
                "commandId" => $command->id,
                "priceId" => $priceModel->id,
                "quantity" => $commandLine["quantity"]
            ]);
            
            $lineQuantity = $commandPrice->quantity;
            $linePriceId = $commandPrice->priceId;

            $commandLines[] = [
                'parent_id' => null,
                'id' => 0,
                'created_at' => Carbon::now(),
                'cmd_remise_id' => null,
                'cmd_client_id' => $local_key,
                'local_key' => Str::uuid()->toString(),
                'updated_at' => Carbon::now(),
                'quantite' => $lineQuantity,
                'prix_prd_cns_prcpl_id' => $linePriceId,
                'modified_at' => null
            ];
        }
        \Log::info("step-validation-passed 4");

        // dd("step-commandPrice-passed");

        $remise = $request->commandForm["reduction"] != "" ? $request->commandForm["reduction"] : null;

        if ($client->id == $command->clientId) {

            if($remise) {
                $this->commandReductionRepository->addCodeReduction([
                    "clientId" => $command->clientId,
                    "commandId" => $command->id,
                    "code" => $remise
                ]);
            }
            
            $role = $this->roleRepository->getByTag("teleporter");
            
            $admins = $this->userRepository->allQuery(["role_id" => $role->id])->get();
            
            // if($user && $user->email) {
            //     $message = (new NewCommandMail($command, false))->onQueue('emails');
                
            //     \Mail::to($user->email)->cc([])->queue($message);
            // }
            
            // $message = (new NewCommandMail($command))->onQueue('emails');
        
            // foreach ($admins as $admin) {
            //     \Mail::to($admin->email)->cc([])->queue($message);
            // }
        }
        \Log::info("step-validation-passed 5");

        // dd("step-remise-mail-passed");

        $command = $this->commandRepository->find($command->id);
        
        // $setting_stock_in = \DB::table('settings')->where('key', 'site.magasin_client_id_in')->first();
        // $setting_stock_out = \DB::table('settings')->where('key', 'site.magasin_client_id_out')->first();
        // $condition = $setting_stock_in && $setting_stock_out && $setting_stock_in->value == $setting_stock_out->value;
        // if(!$condition) {
        foreach ($command->lines as $line) {
            $product = $line->price->products->first();
            \Log::info("Price Id: ".$line->price->id);
            $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->price->id);
            $product->stock->quantity -= $commandPrice->quantity;
            \Log::info("Product Id: ".$product->id);

            $product->stock->save();
        }
        // }
                \Log::info("step-validation-passed 6");

        // dd("step-stock-operation-passed");
        
        try{
            if($client->id == 5108) SyncCommandsToPos::dispatchNow($command, $commandLines, $local_key, $deliveryCompany, false, $remise);
            else SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, false, $remise)->onQueue('sync-commands');
        }catch(\Exception $error){
            dd($error->getMessage());
        }
        
        
        return response()->json([
            "status" => "ok",
            "id" => $command->id,
            "trackingNumber" => $command->trackingNumber,
            "total" => $command->total,
            'paymentMode' => $paymentMode
        ], 200);
    }


    public function addCartcheckOut_old(Request $request) {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $commandState = $this->commandStateRepository->getByTag("pending");

        if($request->commandId) {

            $command = $this->commandRepository->find(
                $request->commandId
            );

            if($client->id != $command->clientId) {

                \OneSignal::async()->sendNotificationUsingTags(
                    $message = "Vous avez une nouvelle commande sur votre invitation",
                    $tags = [[
                            "key" => "email",
                            "relation" => "=",
                            "value" => $command->client->user->email
                    ]],
                    $url = null,
                    $data = null,
                    $buttons = null,
                    $schedule = null
                );

                if(!$command->grouped) {
                    $invitedCommand = $this->commandService->addCommand([
                        'parentId' => $command->id,
                        'grouped' => $command->grouped,
                        'adresseId' => $command->adresseId,
                        'stateId' => $command->stateId,
                        'paymentModeId' => $command->paymentModeId,
                        'clientId' => $client->id,
                        // 'planifiedFor' => $command->planifiedFor,
                        'codePromo' => null,
                        'comment' => "INVITED COMMAND"
                    ]);
                }

            }

        } else {

            $request->validate([
                'addressForm.planifiedFor' => 'required',
                'addressForm.lat' => 'sometimes',
                'addressForm.lng' => 'sometimes',
                'addressForm.address' => 'required',
                'commandForm.commands' => 'array',
                'addressForm.country' => 'required',
                'addressForm.city' => 'required',
                'addressForm.paymentModeTag' => 'required',
                // 'addressForm.code_postal' => 'required'
            ]);
            
            $paymentMode = $this->payementModeRepository->getByTag($request->addressForm['paymentModeTag']);
            // $planifiedFor = Carbon::parse(
            //     $request->addressForm["planifiedFor"]
            // );

            // geolocation of the client
            $geolaction = null;
            if ($request->addressForm["lat"] && $request->addressForm["lng"]) {
                $geolaction = $this->geographicPointRepository->create([
                    "latitude" => round($request->addressForm["lat"], 2),
                    "longitude" => round($request->addressForm["lng"], 2)
                ]);
            }

            // Adresse
            $address = $this->adresseRepository->create([
                "geographicPointId" => $geolaction ? $geolaction->id : null,
                "description" => $request->addressForm["address"]
            ]);

            $command = $this->commandService->addCommand([
                'adresseId' => $address->id,
                'stateId' => $commandState->id,
                'paymentModeId' => $paymentMode->id,
                'clientId' => $client->id,
                // 'planifiedFor' => $planifiedFor,
                'comment' => $request->addressForm["comment"],
                'country' => $request->addressForm["country"],
                'city' => $request->addressForm["city"],
                'region' => $request->addressForm["region"],
                'code_postal' => $request->addressForm["code_postal"],
                'trackingNumber' => null,
                'totalAmount' => $request->totalAmount
            ]);

            $command->trackingNumber = 'w/' . Carbon::now()->format('Y') . '/' . $command->id;
            $command->save();
        }

        if ($client->id == $command->clientId && ($command->state->tag == "canceled" || !$command->grouped)) {
            $command->lines()->delete();
        }

        $local_key = Str::uuid()->toString();
        $deliveryCompany = null;

        $commandLines = array_map(function($commandLine) use ($command, $client, $local_key) {

            $priceModel = $this->priceRepository->find(
                $commandLine["price"]["id"]
            );

            if(sizeOf($commandLine["products"]) > 1) {
                $productListId = array_map(function($product){
                    return $product["id"];
                }, $commandLine["products"]);
                $priceModel = $this->priceRepository->firstOrCreateCompositPrice(
                    $priceModel,
                    $productListId
                );
            }

            if(!$command->grouped && array_key_exists("clientId", $commandLine) && $commandLine["clientId"] != "" &&  $commandLine["clientId"] != $command->clientId && $client->id == $command->clientId) {

                $invitedCommand = $this->commandRepository->all([
                    "parentId" => $command->id,
                    "clientId" => $commandLine["clientId"]
                ])->first();

                $commandPrice = $this->commandPriceRepository->create([
                    "clientId" => $commandLine["clientId"],
                    "commandId" => $invitedCommand->id,
                    "priceId" => $priceModel->id,
                    "quantity" => $commandLine["quantity"]
                ]);
                $lineQuantity = $commandPrice->quantity;
                $linePriceId = $commandPrice->priceId;

            } else {
                $commandPrice = $this->commandPriceRepository->create([
                    "clientId" => array_key_exists("clientId", $commandLine) && $commandLine["clientId"] != "" ? $commandLine["clientId"] : $client->id,
                    "commandId" => $command->id,
                    "priceId" => $priceModel->id,
                    "quantity" => $commandLine["quantity"]
                ]);
                $lineQuantity = $commandPrice->quantity;
                $linePriceId = $commandPrice->priceId;
            }

            return [
                'parent_id' => null,
                'id' => 0,
                'created_at' => Carbon::now(),
                'cmd_remise_id' => null,
                'cmd_client_id' => $local_key,
                'local_key' => Str::uuid()->toString(),
                'updated_at' => Carbon::now(),
                'quantite' => $lineQuantity,
                'prix_prd_cns_prcpl_id' => $linePriceId,
                'modified_at' => null
            ];

        }, $requestCommandes);

        foreach ($requestCommandes as $key => $line) {
            \Log::debug($line['price']['id']);
            $priceId = $line['price']['id'];
            $transportCompany = TransportCompany::where('price_id', $priceId)->first();

            if($transportCompany) $deliveryCompany = $transportCompany;
        }

        if ($client->id == $command->clientId) {

            $command->stateId = $commandState->id;
            $command->save();

            if(array_key_exists("codePromo", $request->commandForm) && $request->commandForm["codePromo"] != "") {
                $this->commandReductionRepository->addCodeReduction([
                    "clientId" => $command->clientId,
                    "commandId" => $command->id,
                    "code" => $request->commandForm["codePromo"]
                ]);
            }

            //remove append default delivering 
            // $command = $this->commandService->checkDeliveringPrice(
            //     $command->id
            // );

            $role = $this->roleRepository->getByTag("teleporter");

            $admins = $this->userRepository->allQuery(
                ["role_id" => $role->id]
            )->get();

            

            if($user && $user->email) {
                $message = (new NewCommandMail(
                    $command,false
                ))->onQueue('emails');
                \Mail::to($user->email)
                    ->cc([])
                    ->queue($message);
            }

            $message = (new NewCommandMail($command))->onQueue('emails');

            foreach ($admins as $key => $admin) {
                \Mail::to($admin->email)
                    ->cc([])
                    ->queue($message);
            }
        }

        $command = $this->commandRepository->find(
            $command->id
        );
        $remise = null;
        if(array_key_exists("codePromo", $request->commandForm)) {
            $remise = $request->commandForm["codePromo"];
        }

        foreach ($command->lines as $line) {
            $product = $line->price->products->first();
            $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->price->id);
            $productStock = StockOperationProduct::where('productId', $product->id)->first();
            $productStock->quantity -=  $commandPrice->quantity;
            $productStock->save();
        }

        SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, false, $remise)->onQueue('sync-commands');

        return response()->json([
            'status' => "ok",
            "id" => $command->id,
            "command" => $command,
            "trackingNumber" => $command->trackingNumber,
            "total" => $command->total,
            'paymentMode' => $paymentMode
        ], 200);

    }

    public function addPaymentFile(Request $request)
    {
        $extension = $request->paymentImage->extension();
        $fileName = (string) \Str::uuid() . "." . $extension;
        $request->paymentImage->storeAs('/public/Command', $fileName);
        $path = '/Command/' . $fileName;
        
        $this->commandRepository->update(
            [
                'image' => $path
            ], $request->commandId
        );
        

        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);
        // \Log::debug($request->all());
        
        // $response = $client->post(config('urls.send-file'), [
        //     'multipart' => [
        //         [
        //             'name'     => $path,
        //             'contents' => fopen(\Storage::disk('local')->get($path), 'r')
        //         ],
        //     ]
        // ]);

        return response()->json(['status' => true]);
    }

    public function cancelCommand(Request $request, $commandId) {

        $user = \JWTAuth::toUser();
        $client = $user->client;

        $command = $this->commandRepository->find($commandId);

        if($command->clientId == $client->id) {
            $commandState = $this->commandStateRepository->getByTag("canceled");

            $command = $this->commandRepository->update([
                "stateId" => $commandState->id
            ], $commandId);

            return response()->json([
                "command" => $command
            ]);

        } else {
            return response()->json([
                'status' => "error",
                'message' => "You are not the owner of this command"
            ], 422);
        }

    }

    public function getCommandDetails(Request $request)
    {
        
        if($request->trackingNumber){
            $command = $this->commandRepository->getCommandByNumber($request->trackingNumber);
            if(is_null($command)) return response()->json(['tag' => 'not_found'], 422);
            if(is_null($command->pos_command_id)) return response()->json(['tag' => 'not_sync_yet'], 422);

        }
        else if($request->phoneNumber) {
            $user = User::where('phone', $request->phoneNumber)->first();
            if(is_null($user)) return response()->json(['tag' => 'user_not_found'], 422);
            $commandes =  Command::whereNotNull('pos_command_id')->where('clientId', $user->client->id)->get()->pluck('pos_command_id');
            return $this->getAllCommands($commandes);
        }
        else {
            return;
        }


        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' =>[ $command->pos_command_id]
            ]
        ])->getBody());

        $commandLines = [];
        $priceListIds = [];
        foreach ($commandDetails->CmdPrdCnsPrcpl as $key => $commandPrices) {
            $commandLines[$key]['quantity'] = $commandPrices->quantite;
            $commandLines[$key]['priceId'] = $commandPrices->prix_prd_cns_prcpl_id;
            array_push($priceListIds, $commandPrices->prix_prd_cns_prcpl_id);
        }

        $prices = $this->priceRepository->getPricesByIds($priceListIds);
        \Log::debug(json_encode($commandDetails));
        $commandPos = $commandDetails->CmdClient[0];
        
        foreach($prices as $price){
            if(is_null($price->products[0]->file_path)){
                $price->products[0]->file_path = $price->products[0]->description->file_path;
            }
        }

        $command = $this->commandRepository->getCommandByPosId($commandPos->id);

        if(isset($commandPos->delivery_state_id)) 
            $command->state = DeliveryState::where('id', $commandPos->delivery_state_id);
        
        return response()->json([
            'prices' => $prices, 
            'commandLines' => $commandLines, 
            'command' => $command,
            'total' => $command->total
        ]);
    }
    
    public function getAllCommands($commandListIds = null){
        
        $commandListIds = $commandListIds ?? $this->commandRepository->getCommandsListIds();
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);
        
        $commandsDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' => $commandListIds
            ]
        ])->getBody());
    
        $posCommands = $commandsDetails->CmdClient;
        $CmdsPrdCnsPrcpl = collect($commandsDetails->CmdPrdCnsPrcpl);
        
        $commands = [];
    
        foreach ($posCommands as $posCommand) {

            Command::where('pos_command_id', $posCommand->id)->update([
                'delivery_state_id' => isset($posCommand->cmd_delivery_info) ? $posCommand->cmd_delivery_info->state_id : null
            ]);

            $CmdPrdCnsPrcpl = $CmdsPrdCnsPrcpl->where('cmd_client_id', $posCommand->id);
            $command = $this->commandRepository->getCommandByPosId($posCommand->id);
    
            $commandLines = [];
            $prices = [];
            $priceListIds = [];
    
            foreach ($CmdPrdCnsPrcpl as $key => $commandPrices) {

                CommandPrice::updateOrCreate([
                    'priceId' => $commandPrices->prix_prd_cns_prcpl_id,
                    'commandId' => $command->id,
                    'quantity' => $commandPrices->quantite,
                    'clientId' => $command->clientId,
                ], [
                    'priceId' => $commandPrices->prix_prd_cns_prcpl_id
                ]);

                $commandLines[] = (object) [
                    'quantity' => $commandPrices->quantite,
                    'priceId' => $commandPrices->prix_prd_cns_prcpl_id
                ];

                array_push($priceListIds, $commandPrices->prix_prd_cns_prcpl_id);
            }
        
            $commandLines = collect($commandLines);
            $prices = $this->priceRepository->getPricesByIds($priceListIds);
            
            $tag_method = null;

            if($command->payementMode->tag == 'bank-transfer') $tag_method = 'Virement bancaire';
            else if($command->payementMode->tag == 'cash-on-delivery') $tag_method = 'Paiement à la livraison';

            $data = [
                'cm_id' => $command->id,
                'stateLabel' => $command->deliveryState && $command->deliveryState->intitule ? $command->deliveryState->intitule : 'En attente',
                'stateTag' => $command->deliveryState && $command->deliveryState->tag ? $command->deliveryState->tag : 'pending',
                'clientPhone' => $command->client->phone,
                'image' => $command->image,
                'addresse' => $command->address->description,
                'trackNumber' => $command->trackingNumber,
                'canceled' => $command->canceled,
                'country' => $command->country,
                'city' => $command->city,
                'payementMode' => $tag_method,
                'paymentTag' => $command->payementMode->tag,
                'created' => $command->created_at,
                'deliveryFees' => 0,
                'timestamp' => $command->created_at->timestamp,
                'products' => [],
                'totalPrice' => 0,
                'totalAmount' => $command->totalAmount
            ];
            
            if($command->canceled) $data['stateLabel'] = "Annulé";


            foreach ($prices as $price) {
                if($price->products[0]->description->label != 'LIVRAISON') {

                    $productQuantity = $commandLines->where('priceId', $price->id)->first()->quantity;
                    
                    $productPrice = collect($price->products[0]);
                    
                    $keys = array_map(function ($e) {
                        return $e['fr_value'] ? $e['fr_value'] : $e['value'];
                    }, $productPrice->get('detail_keys'));
                    
                    $data['products'][] = [
                        'id' => $price->products[0]->id,
                        'label'=> $price->products[0]->description->label,
                        'file_path'=> $price->products[0]->file_path ?? $price->products[0]->description->file_path,
                        'price'=> $price->promoPrice,
                        'quantity'=> $productQuantity,
                        'keys'=> implode('/', $keys)

                    ];
                }else{
                    $data['deliveryFees'] = $price->promoPrice;
                }

                $data['totalPrice'] += $price->promoPrice;
            }
    
            array_push($commands, $data);
        }
        
        return response()->json(['commands' => $commands]);
    }

    public function getAllCommands_old()
    {
        $commandListIds = $this->commandRepository->getCommandsListIds();
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandsDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' => $commandListIds
            ]
        ])->getBody());
        
        return response()->json(['commands' => $commandsDetails]);

        $allLines = $commandsDetails->CmdPrdCnsPrcpl;
        
        $posCommands = $commandsDetails->CmdClient;

        $commands = array_map(function($posCommand) use ($allLines){
            Command::where('pos_command_id', $posCommand->id)->update([
                'delivery_state_id' => isset($posCommand->cmd_delivery_info) ? $posCommand->cmd_delivery_info->state_id : null
            ]);
            $command = $this->commandRepository->getCommandByPosId($posCommand->id);
            \Log::debug(json_encode($command->id));

            $command->totalPrice = 0;
            $command->prices = [];
            $commandLines = [];
            $linesCommand = [];
            
            foreach ($allLines as $line) {
                if($line->cmd_client_id == $posCommand->id){
                    $cmdLine = CommandPrice::updateOrCreate([
                        'priceId' => $line->prix_prd_cns_prcpl_id,
                        'commandId' => $command->id,
                        'quantity' => $line->quantite,
                        'clientId' => $command->clientId,
                    ], [
                        'priceId' => $line->prix_prd_cns_prcpl_id
                    ]);
                    $linesCommand = $cmdLine;
                    $command->totalPrice += $cmdLine->price->price;
                    array_push($commandLines, $line->prix_prd_cns_prcpl_id);
                }
            }
            $command->prices = $this->priceRepository->getPricesByIds($commandLines);
            
            $command->deliveryState = $command->deliveryState;
            $command->totalAmount = $command->totalAmount;
            return $command;
        
        }, $posCommands);

        $notSyncedCommands = $this->commandRepository->getNotSyncedCommands();
        $data = array_merge($notSyncedCommands, $commands);

        return response()->json(['commands' => $data]);

    }

    public function getShippingPrices()
    {        
        $shippingPrices = Price::whereHas('tags', function ($query) {
            $query->where('tag', 'delivery_fees');
        })
        //->has("products", "=", 1)
        // ->with('products.description', 'products.detailKeys.detailKey')
        ->get();

        return response()->json(['shippingPrices' => $shippingPrices]);
    }
}
