<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;

use App\Repositories\PriceRepository;
use App\Repositories\BrandRepository;
use App\Repositories\PriceReviewRepository;
use App\Repositories\PriceCategoryRepository;
use App\Repositories\ProductDetailKeyRepository;
use App\Repositories\ProductElementRepository;
use App\Repositories\ProductCategoryRepository;

class CollectionController extends AppBaseController
{
    protected $priceRepository;
    protected $brandRepository;
    protected $priceReviewRepository;
    protected $priceCategoryRepository;
    protected $productDetailKeyRepository;
    protected $productElementRepository;
    protected $productCategoryRepository;

    public function __construct(
        PriceRepository $priceRepository,
        BrandRepository $brandRepository,
        PriceReviewRepository $priceReviewRepository,
        PriceCategoryRepository $priceCategoryRepository,
        ProductDetailKeyRepository $productDetailKeyRepository,
        ProductElementRepository $productElementRepository,
        ProductCategoryRepository $productCategoryRepository
    ) {
        $this->priceRepository = $priceRepository;
        $this->brandRepository = $brandRepository;
        $this->priceReviewRepository = $priceReviewRepository;
        $this->priceCategoryRepository = $priceCategoryRepository;
        $this->productDetailKeyRepository = $productDetailKeyRepository;
        $this->productElementRepository = $productElementRepository;
        $this->productCategoryRepository = $productCategoryRepository;
    }

    public function getFilteres(Request $request)
    {

        $categories = $this->priceCategoryRepository->getFilledCategories();
        $product_category = $this->productCategoryRepository->getProductCategory();
        $brands = $this->brandRepository->getFilledBrands();
        $detailKeys = $this->productDetailKeyRepository->getFilledDetail();
        $maxPrice = $this->priceRepository->getMaxPrice();
        $minPrice = $this->priceRepository->getMinPrice();

        return response()->json([
            "categories" => $categories,
            "product_category" => $product_category,
            "brands" => $brands,
            "detail_keys" => $detailKeys,
            "min_price" => $minPrice,
            "max_price" => $maxPrice,
        ]);
    }

    public function getProducts(Request $request)
    {

        $paginatedPrices = $this->priceRepository->getPaginatedPrices($request->all());

        return response()->json($paginatedPrices);
    }

    public function getIngredients()
    {
        $ingredients = $this->productElementRepository->getIngredientsElements();
        return response()->json($ingredients);
    }
}
