<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;
use App\Repositories\PriceRepository;
use App\Repositories\ProductRepository;
use App\Repositories\CombinedProductRepository;
use App\Repositories\PriceCategoryRepository;
use App\Repositories\ProductCategoryRepository;
use App\Repositories\EntrepriseRepository;
use App\Repositories\ClientRepository;

class SelectController extends AppBaseController {

    public function __construct(
        PriceRepository $priceRepository,
        ProductRepository $productRepository,
        CombinedProductRepository $combinedProductRepository,
        PriceCategoryRepository $priceCategoryRepository,
        ProductCategoryRepository $productCategoryRepository,
        EntrepriseRepository $entrepriseRepository,
        ClientRepository $clientRepository
    ){
        $this->priceRepository = $priceRepository;
        $this->productRepository = $productRepository;
        $this->combinedProductRepository = $combinedProductRepository;
        $this->priceCategoryRepository = $priceCategoryRepository;
        $this->productCategoryRepository = $productCategoryRepository;
        $this->entrepriseRepository = $entrepriseRepository;
        $this->clientRepository = $clientRepository;
    }

    public function getClientList(Request $request) {
        
        
        $search = [];

        if ($request->keyword) {
            $search["phone"] = $request->keyword;
            $search["user.name"] = $request->keyword;
            $search["user.email"] = $request->keyword;
        }

        $clients = $this->clientRepository->all(
            $search = $search,
            $skip = null,
            $limit = 10,
            ['*'],
            ['user']
        )->toArray();

        return response()->json($clients);

    }

    public function getEntrepriseList(Request $request) {

        $search = [];

        if ($request->keyword) $search["name"] = $request->keyword;

        $products = $this->entrepriseRepository->all(
            $search = $search,
            $skip = null,
            $limit = 10
        )->toArray();

        return response()->json($products);

    }

    public function getPriceList(Request $request) {

        $priceQuery = $this->priceRepository->getBasedQuery(null);

        if ($request->categoryId) $priceQuery = $priceQuery->where("categoryId", $request->categoryId);

        $priceQuery = $priceQuery->with("products");

        return response()->json($priceQuery->get());

    }

    public function getProductList(Request $request) {

        $search = [];

        if ($request->categoryId) $search["categoryId"] = $request->categoryId;

        $products = $this->productRepository->all(
            $search = $search,
            $skip = null,
            $limit = null,
            $columns = ["id", "label AS text"]
        )->toArray();

        return response()->json($products);

    }

    public function getCategoryList(Request $request) {

        $search = [];

        $categories = $this->productCategoryRepository->all(
            $search = $search,
            $skip = null,
            $limit = null,
            $columns = ["id", "label AS text"]
        )->toArray();

        return response()->json($categories);

    }

}