<?php

namespace App\Http\Controllers\Voyager;
use Illuminate\Http\Request;
use Voyager;
use App\Repositories\EntrepriseRepository;
use App\Repositories\ClientRepository;

class ClientController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController {

    protected $entrepriseRepository;
    protected $clientRepository;

    public function __construct(
        EntrepriseRepository $entrepriseRepository,
        ClientRepository $clientRepository
    ){
        $this->entrepriseRepository = $entrepriseRepository;
        $this->clientRepository = $clientRepository;
    }
    
    
    public function resetPassword(Request $request) {
        
        $id = $request->id;
        $password = "twistana123";
        
        \App\User::where('id', $id)->update([
            'password' => \Hash::make($password)
        ]);
        
        $response = [
            'message'    => 'Le mot de passe à été réinitialisé',
            'alert-type' => 'success',
        ];

        return redirect()->back()->with($response);
        
    }

    public function getPaginatedClients(Request $request) {
        
        $keyword = $request->key;
        $pagination = $request->pagination ?? 10;
        
        $clients = \App\User::with('client');
        
        if($keyword) {
            $clients = $clients
                ->where('phone', 'like', '%' . $keyword . '%')
                ->orWhere('email', 'like', '%' . $keyword . '%')
                ->orWhere('name', 'like', '%' . $keyword . '%');
        }
        
        
        $clients = $clients
            ->latest('created_at')
            ->paginate($pagination);
            
            
        $view = 'voyager::client.index';
        
        $clients->appends($request->query());
        
        return Voyager::view($view, [
            'dataTypeContent' => $clients,
            'key' => $keyword
        ]);
    }
    
    
    public function addClientToEntreprise(Request $request){

        $user = auth()->user();

        $entreprise = $this->entrepriseRepository->find($user->entrepriseId);
        if($entreprise->clientsTotal() >= $entreprise->maxEmployes) {
            return back()->withErrors(
                'Vous avez atteint le nombre maximum des clients'
            );
        }

        $client = $this->clientRepository->findClientWithEmail(
            $request->email
        );

        if(!$client) {
            return back()->withErrors(
                'Nous ne pouvons trouver aucun client avec cet e-mail'
            );
        } else {
            $toggle = $this->clientRepository->addClientToEntreprise(
                $client->id,
                $user->entrepriseId
            );
            return redirect()->back();
        }

    }

    public function toggleEntrepriseClient(Request $request, $clientId)
    {
        $user = auth()->user();

        $toggle = $this->clientRepository->addClientToEntreprise(
            $clientId,
            $user->entrepriseId
        );

        return response()->json([
            'status' => 'ok'
        ], 200);
    }

    public function index(Request $request)
    {
        return redirect()->back();
        
        // GET THE SLUG, ex. 'posts', 'pages', etc.
        $slug = $this->getSlug($request);

        // GET THE DataType based on the slug
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('browse', app($dataType->model_name));

        $getter = $dataType->server_side ? 'paginate' : 'get';

        $search = (object) ['value' => $request->get('s'), 'key' => $request->get('key'), 'filter' => $request->get('filter')];
        $searchable = $dataType->server_side ? array_keys(SchemaManager::describeTable(app($dataType->model_name)->getTable())->toArray()) : '';
        $orderBy = $request->get('order_by', $dataType->order_column);
        $sortOrder = $request->get('sort_order', null);
        $usesSoftDeletes = false;
        $showSoftDeleted = false;
        $orderColumn = [];
        if ($orderBy) {
            $index = $dataType->browseRows->where('field', $orderBy)->keys()->first() + 1;
            $orderColumn = [[$index, 'desc']];
            if (!$sortOrder && isset($dataType->order_direction)) {
                $sortOrder = $dataType->order_direction;
                $orderColumn = [[$index, $dataType->order_direction]];
            } else {
                $orderColumn = [[$index, 'desc']];
            }
        }

        // Next Get or Paginate the actual content from the MODEL that corresponds to the slug DataType
        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);

            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $query = $model->{$dataType->scope}();
            } else {
                $query = $model::select('*');
            }

            $user = auth()->user();

            if($user->role->name == "entrepreneur") {
                $entrepriseId = $user->entrepriseId;
                $query = $query->whereHas('user', function ($query) use ($entrepriseId) {
                    $query->where('entrepriseId', $entrepriseId);
                });
            }

            // Use withTrashed() if model uses SoftDeletes and if toggle is selected
            if ($model && in_array(SoftDeletes::class, class_uses($model)) && app('VoyagerAuth')->user()->can('delete', app($dataType->model_name))) {
                $usesSoftDeletes = true;

                if ($request->get('showSoftDeleted')) {
                    $showSoftDeleted = true;
                    $query = $query->withTrashed();
                }
            }

            // If a column has a relationship associated with it, we do not want to show that field
            $this->removeRelationshipField($dataType, 'browse');

            if ($search->value != '' && $search->key && $search->filter) {
                $search_filter = ($search->filter == 'equals') ? '=' : 'LIKE';
                $search_value = ($search->filter == 'equals') ? $search->value : '%'.$search->value.'%';
                $query->where($search->key, $search_filter, $search_value);
            }

            if ($orderBy && in_array($orderBy, $dataType->fields())) {
                $querySortOrder = (!empty($sortOrder)) ? $sortOrder : 'desc';
                $dataTypeContent = call_user_func([
                    $query->orderBy($orderBy, $querySortOrder),
                    $getter,
                ]);
            } elseif ($model->timestamps) {
                $dataTypeContent = call_user_func([$query->latest($model::CREATED_AT), $getter]);
            } else {
                $dataTypeContent = call_user_func([$query->orderBy($model->getKeyName(), 'DESC'), $getter]);
            }

            // Replace relationships' keys for labels and create READ links if a slug is provided.
            $dataTypeContent = $this->resolveRelations($dataTypeContent, $dataType);
        } else {
            // If Model doesn't exist, get data from table name
            $dataTypeContent = call_user_func([DB::table($dataType->name), $getter]);
            $model = false;
        }

        // Check if BREAD is Translatable
        if (($isModelTranslatable = is_bread_translatable($model))) {
            $dataTypeContent->load('translations');
        }

        // Check if server side pagination is enabled
        $isServerSide = isset($dataType->server_side) && $dataType->server_side;

        // Check if a default search key is set
        $defaultSearchKey = $dataType->default_search_key ?? null;

        $view = 'voyager::bread.browse';

        if (view()->exists("voyager::$slug.browse")) {
            $view = "voyager::$slug.browse";
        }

        return Voyager::view($view, compact(
            'dataType',
            'dataTypeContent',
            'isModelTranslatable',
            'search',
            'orderBy',
            'orderColumn',
            'sortOrder',
            'searchable',
            'isServerSide',
            'defaultSearchKey',
            'usesSoftDeletes',
            'showSoftDeleted'
        ));
    }

    public function store(Request $request) {


        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        // Check permission
        $this->authorize('add', app($dataType->model_name));

        // Validate fields with ajax
        $user = auth()->user();
        if($user->role->name == "entrepreneur") {
            $entrepriseId = $user->client->entrepriseId;
            $request->request->add(['entrepriseId' => $entrepriseId]);
        }
        $val = $this->validateBread($request->all(), $dataType->addRows)->validate();
        $data = $this->insertUpdateData($request, $slug, $dataType->addRows, new $dataType->model_name());

        event(new BreadDataAdded($dataType, $data));

        return redirect()
        ->route("voyager.{$dataType->slug}.index")
        ->with([
                'message'    => __('voyager::generic.successfully_added_new')." {$dataType->display_name_singular}",
                'alert-type' => 'success',
            ]);
    }

}