<?php

namespace App\Http\Controllers\Voyager;

use App\Models\DetailKey;
use App\Models\Price;
use App\Models\Product;
use App\Models\ProductDescription;
use App\Models\ProductDetailKey;
use Illuminate\Http\Request;
use TCG\Voyager\Events\BreadDataAdded;
use TCG\Voyager\Events\BreadDataUpdated;
use Voyager;

use App\Repositories\EntrepriseRepository;
use App\Repositories\PriceRepository;
use App\Repositories\ProductRepository;
use App\Repositories\CombinedProductRepository;
use App\Repositories\PriceCategoryRepository;
use App\Repositories\ProductCategoryRepository;
use App\Repositories\PriceEntrepriseRepository;
use App\Repositories\ReductionEntrepriseRepository;
use Illuminate\Support\Facades\Storage;

class ProductController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController {

    protected $priceRepository;
    protected $entrepriseRepository;
    protected $productRepository;
    protected $combinedProductRepository;
    protected $priceCategoryRepository;
    protected $priceEntrepriseRepository;
    protected $productCategoryRepository;
    protected $reductionEntrepriseRepository;

    public function __construct(
        PriceRepository $priceRepository,
        ProductRepository $productRepository,
        CombinedProductRepository $combinedProductRepository,
        PriceCategoryRepository $priceCategoryRepository,
        ProductCategoryRepository $productCategoryRepository,
        PriceEntrepriseRepository $priceEntrepriseRepository,
        ReductionEntrepriseRepository $reductionEntrepriseRepository,
        EntrepriseRepository $entrepriseRepository
    ){
        $this->priceRepository = $priceRepository;
        $this->productRepository = $productRepository;
        $this->combinedProductRepository = $combinedProductRepository;
        $this->priceCategoryRepository = $priceCategoryRepository;
        $this->productCategoryRepository = $productCategoryRepository;
        $this->priceEntrepriseRepository = $priceEntrepriseRepository;
        $this->reductionEntrepriseRepository = $reductionEntrepriseRepository;
        $this->entrepriseRepository = $entrepriseRepository;
    }

    public function index(Request $request)
    {
        // GET THE SLUG, ex. 'posts', 'pages', etc.
        $slug = $this->getSlug($request);

        // GET THE DataType based on the slug
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        

        // Check permission
        $this->authorize('browse', app($dataType->model_name));

        $getter = $dataType->server_side ? 'paginate' : 'get';

        $entreprise = $request->has("entreprise") ? $this->entrepriseRepository->find($request->get("entreprise")) : null;

        $search = (object) ['value' => $request->get('s'), 'key' => $request->get('key'), 'filter' => $request->get('filter')];
        $searchable = $dataType->server_side ? array_keys(SchemaManager::describeTable(app($dataType->model_name)->getTable())->toArray()) : '';
        $orderBy = $request->get('order_by', $dataType->order_column);
        $sortOrder = $request->get('sort_order', null);
        $usesSoftDeletes = false;
        $showSoftDeleted = false;
        $orderColumn = [];
        if ($orderBy) {
            $index = $dataType->browseRows->where('field', $orderBy)->keys()->first() + 1;
            $orderColumn = [[$index, 'desc']];
            if (!$sortOrder && isset($dataType->order_direction)) {
                $sortOrder = $dataType->order_direction;
                $orderColumn = [[$index, $dataType->order_direction]];
            } else {
                $orderColumn = [[$index, 'desc']];
            }
        }

        // Next Get or Paginate the actual content from the MODEL that corresponds to the slug DataType
        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);

            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $query = $model->{$dataType->scope}();
            } else {
                $query = $model::select('*');
            }

            if($entreprise == null){
                $query = $query->whereHas("prices", function($subQuery) {
                    $subQuery->where("isStandard", false);
                });
            } else {
                $query = $query->whereHas("prices", function($subQuery) use ($entreprise) {
                    $subQuery->where("isStandard", false)->orWhereIn("Price.id", $entreprise->prices->pluck("priceId"));
                });
            }

            // Use withTrashed() if model uses SoftDeletes and if toggle is selected
            if ($model && in_array(SoftDeletes::class, class_uses($model)) && app('VoyagerAuth')->user()->can('delete', app($dataType->model_name))) {
                $usesSoftDeletes = true;

                if ($request->get('showSoftDeleted')) {
                    $showSoftDeleted = true;
                    $query = $query->withTrashed();
                }
            }

            // If a column has a relationship associated with it, we do not want to show that field
            $this->removeRelationshipField($dataType, 'browse');

            if ($search->value != '' && $search->key && $search->filter) {
                $search_filter = ($search->filter == 'equals') ? '=' : 'LIKE';
                $search_value = ($search->filter == 'equals') ? $search->value : '%'.$search->value.'%';
                $query->where($search->key, $search_filter, $search_value);
            }

            if ($orderBy && in_array($orderBy, $dataType->fields())) {
                $querySortOrder = (!empty($sortOrder)) ? $sortOrder : 'desc';
                $dataTypeContent = call_user_func([
                    $query->orderBy($orderBy, $querySortOrder),
                    $getter,
                ]);
            } elseif ($model->timestamps) {
                $dataTypeContent = call_user_func([$query->latest($model::CREATED_AT), $getter]);
            } else {
                $dataTypeContent = call_user_func([$query->orderBy($model->getKeyName(), 'DESC'), $getter]);
            }

            // Replace relationships' keys for labels and create READ links if a slug is provided.

            $dataTypeContent = $this->resolveRelations($dataTypeContent, $dataType);
        } else {
            // If Model doesn't exist, get data from table name
            $dataTypeContent = call_user_func([DB::table($dataType->name), $getter]);
            $model = false;
        }

        // Check if BREAD is Translatable
        if (($isModelTranslatable = is_bread_translatable($model))) {
            $dataTypeContent->load('translations');
        }

        // Check if server side pagination is enabled
        $isServerSide = isset($dataType->server_side) && $dataType->server_side;

        // Check if a default search key is set
        $defaultSearchKey = $dataType->default_search_key ?? null;

        $view = 'voyager::bread.browse';

        if (view()->exists("voyager::$slug.browse")) {
            $view = "voyager::$slug.browse";
        }

        return Voyager::view($view, compact(
            'dataType',
            'dataTypeContent',
            'isModelTranslatable',
            'search',
            'orderBy',
            'orderColumn',
            'sortOrder',
            'searchable',
            'isServerSide',
            'defaultSearchKey',
            'usesSoftDeletes',
            'showSoftDeleted'
        ));
    }

    public function edit(Request $request, $id)
    {
        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        if (strlen($dataType->model_name) != 0) {
            $model = app($dataType->model_name);
            if ($model && in_array(SoftDeletes::class, class_uses($model))) {
                $model = $model->withTrashed();
            }
            if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
                $model = $model->{$dataType->scope}();
            }
            $dataTypeContent = call_user_func([$model, 'findOrFail'], $id);
        } else {
            $dataTypeContent = DB::table($dataType->name)->where('id', $id)->first();
        }

        foreach ($dataType->editRows as $key => $row) {
            $dataType->editRows[$key]['col_width'] = isset($row->details->width) ? $row->details->width : 100;
        }

        $this->removeRelationshipField($dataType, 'edit');

        $this->authorize('edit', $dataTypeContent);

        $isModelTranslatable = is_bread_translatable($dataTypeContent);

        $view = 'voyager::bread.edit-add';

        if (view()->exists("voyager::$slug.edit-add")) {
            $view = "voyager::$slug.edit-add";
        }

        $combinedProduct = $this->combinedProductRepository->allQuery(
            $search = [ "product_id" => $id ]
        )->first();

        $price = $this->priceRepository->allQuery(
            $search = [ "id" => $combinedProduct->price_id ]
        )->first();

        return Voyager::view($view, compact('price', 'dataType', 'dataTypeContent', 'isModelTranslatable'));
    }

    public function update(Request $request, $id)
    {
        $slug = $this->getSlug($request);
        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();
        // Compatibility with Model binding.
        $id = $id instanceof \Illuminate\Database\Eloquent\Model ? $id->{$id->getKeyName()} : $id;
        $model = app($dataType->model_name);
        if ($dataType->scope && $dataType->scope != '' && method_exists($model, 'scope'.ucfirst($dataType->scope))) {
            $model = $model->{$dataType->scope}();
        }
        if ($model && in_array(SoftDeletes::class, class_uses($model))) {
            $data = $model->withTrashed()->findOrFail($id);
        } else {
            $data = call_user_func([$dataType->model_name, 'findOrFail'], $id);
        }

        // Check permission
        $this->authorize('edit', $data);

        // Validate fields with ajax
        $val = $this->validateBread($request->all(), $dataType->editRows, $dataType->name, $id)->validate();

        $validatedPriceData = \Validator::make($request->all(), [
            'price' => 'required|numeric',
            'image' => 'required'
        ]);

        if ($validatedPriceData->fails()) {
            return response()->json([
                'errors' => $validatedPriceData->messages()
            ]);
        }

        $combinedProduct = $this->combinedProductRepository->allQuery(
            $search = [ "product_id" => $id ]
        )->first();

        $price = $this->priceRepository->allQuery(
            $search = [ "id" => $combinedProduct->price_id ]
        )->first();

        $productCategory = $this->productCategoryRepository->find(
            $request->categoryId
        );

        $priceCategory = $this->priceCategoryRepository->create(
            $productCategory->toArray()
        );

        // if($request->image){
        $extension = $request->image->extension();
        $fileName = (string) \Str::uuid() . "." . $extension;
        $request->image->storeAs('/public/Product', $fileName);
        $path = '/Product/' . $fileName;

        if($request->variantImage){
            $extension = $request->variantImage->extension();
            $fileName = (string) \Str::uuid() . "." . $extension;
            $request->variantImage->storeAs('/public/Product/Variant', $fileName);
            $variantPath = '/Product/Variant/' . $fileName;
            
            Product::where('id', $id)->update([
                'image' => $variantPath,
            ]);
        }
        
        // }

        ProductDescription::where('id', $data->product_description_id)->update([
            'image' => $path,
            'label' => $request->label,
            'category_id' => $request->categoryId,
            'buyingPrice' => $request->buyingPrice,
            'weight'      => $request->weight,
            'description' => $request->description,
        ]);

        

        $detailKeySize = DetailKey::where('tag', 'size')->first('id');
        $detailKeyAge = DetailKey::where('tag', 'age')->first('id');
        $detailKeyColor = DetailKey::where('tag', 'color')->first('id');
        ProductDetailKey::where('product_id', $id)->where('detailKeyId', $detailKeySize->id)->update([
            'value' => $request->size
        ]);

        ProductDetailKey::where('product_id', $id)->where('detailKeyId', $detailKeyAge->id)->update([
            'value' => $request->age
        ]);

        ProductDetailKey::where('product_id', $id)->where('detailKeyId', $detailKeyColor->id)->update([
            'value' => $request->color
        ]);

        // $this->insertUpdateData($request, $slug, $dataType->editRows, $data);
        event(new BreadDataUpdated($dataType, $data));

        $price->category_id = $request->categoryId;
        $price->image = $path;
        $price->isRecommended = $request->has("isRecommended") && $request->get("isRecommended") == "on" ? 1 : 0;
        $price->order = $request->has("order") ? $request->get("order") : 0;
        $price->label = $request->label . '(' . $request->size . '-' . $request->age . '-' . $request->color . ')';

        if($request->has("entreprise")) {
            $priceEntreprise = $this->priceEntrepriseRepository->create([
                'priceId' => $price->id,
                'entrepriseId' => $request->get("entreprise")
            ]);
            $percent = ($request->price/($price->promoPrice != 0 ? $price->promoPrice : 1));
            $reductionEntreprise = $this->reductionEntrepriseRepository->create([
                'priceEntrepriseId' => $priceEntreprise->id,
                'value' => 100 - (($percent != 1 ? $percent : 0) * 100)
            ]);
        } else {
            $price->price = $request->price;
            $price->promoPrice = $request->promoPrice;
        }

        $price->save();

        $urlQuery = [];
        if ($request->has("entreprise")) {
            $urlQuery = ["entreprise" => $request->has("entreprise")];
        }

        return redirect()
            ->route("voyager.{$dataType->slug}.index", $urlQuery)
            ->with([
                'message'    => __('voyager::generic.successfully_updated')." {$dataType->getTranslatedAttribute('display_name_singular')}",
                'alert-type' => 'success',
            ]);
    }


    public function store(Request $request) {

        \Log::debug($request->all());

        $slug = $this->getSlug($request);

        $dataType = Voyager::model('DataType')->where('slug', '=', $slug)->first();

        $this->authorize('add', app($dataType->model_name));

        $validatedPriceData = \Validator::make($request->all(), [
            'variantPrice' => 'required',
            'image' => 'required'
        ]);

        $val = $this->validateBread($request->all(), $dataType->addRows)->validate();;

        if ($validatedPriceData->fails()) {
            return response()->json([
                'errors' => $validatedPriceData->messages()
            ]);
        }

        $productCategory = $this->productCategoryRepository->find(
            $request->categoryId
        );

        $priceCategory = $this->priceCategoryRepository->create(
            $productCategory->toArray()
        );

        if (!$request->has('_validate')) {

            $extension = $request->image->extension();
            $fileName = (string) \Str::uuid() . "." . $extension;
            $request->image->storeAs('/public/Product', $fileName);
            $path = '/Product/' . $fileName;

            $productDescription = ProductDescription::create([
                'label' => $request->label,
                'image' => $path,
                'category_id' => $request->categoryId,
                'buyingPrice' => $request->buyingPrice,
                'weight'      => $request->weight,
                'description' => $request->description,
                'nature' => ''
            ]);
            $detailKeySize = DetailKey::where('tag', 'size')->first('id');
            $detailKeyAge = DetailKey::where('tag', 'age')->first('id');
            $detailKeyColor = DetailKey::where('tag', 'color')->first('id');
            $variantPath = $path;
            if((isset($request->size) && count($request->size) > 0) || (isset($request->age) && count($request->age) > 0) || (isset($request->color) && count($request->color)) > 0){
                if(isset($request->size )) $data = count($request->size); 
                elseif(isset($request->age) && is_array($request->age)) $data = count($request->age) ; 
                else $data = count($request->color);
                for($k = 0 ; $k < $data ; $k++){
                    if($request->variantImage && $request->variantImage[$k]){
                        $extension = $request->variantImage[$k]->extension();
                        $fileName = (string) \Str::uuid() . "." . $extension;
                        $request->variantImage[$k]->storeAs('/public/Product/Variant', $fileName);
                        $variantPath = '/Product/Variant/' . $fileName;
                    }
                    $product = Product::create([
                        'product_description_id' => $productDescription->id,
                        'model_number' => $request->modelNumber && $request->modelNumber[$k] ? $request->modelNumber[$k] : '',
                        'bar_code' => $request->barCode && $request->barCode[$k] ? $request->barCode[$k] : '',
                        'image' => $variantPath,
                    ]);

                    if($request->size && $request->size[$k]){
                        $size = ProductDetailKey::create([
                            'product_id' => $product->id,
                            'detail_key_id' => $detailKeySize->id,
                            'value' => $request->size[$k]
                        ]);
                    }

                    if($request->age && $request->age[$k]){
                        $age = ProductDetailKey::create([
                            'product_id' => $product->id,
                            'detail_key_id' => $detailKeyAge->id,
                            'value' => $request->age[$k]
                        ]);
                    }

                    if($request->color && $request->color[$k]){
                        $color = ProductDetailKey::create([
                            'product_id' => $product->id,
                            'detail_key_id' => $detailKeyColor->id,
                            'value' => $request->color[$k]
                        ]);
                    }
                    $size = $request->size ? $request->size[$k] : '';
                    $age = $request->age ? $request->age[$k] : '';
                    $color = $request->color ? $request->color[$k] : '';

                    $price = Price::create([
                        'label' => $request->label . '(' . $size . '-' . $age . '-' . $color . ')',
                        'price' => $request->variantPrice[$k],
                        'category_id' => $request->categoryId,
                        'image' => $path,
                        'isStandard' => $request->has("entreprise") ? 1 : 0,
                        'isRecommended' => $request->has("isRecommended") && $request->get("isRecommended") == "on" ? 1 : 0,
                        'order' => $request->has("order") ? $request->get("order") : 0,
                        'promoPrice' => $request->variantPromoPrice[$k]
                    ]);

                    $combinedProduct = $this->combinedProductRepository->create([
                        'product_id' => $product->id,
                        'price_id' => $price->id,
                        'quantity' => 1
                    ]);
                }
            }

            ///insert the product
            // $data = $this->insertUpdateData(
            //     $request,
            //     $slug,
            //     $dataType->addRows,
            //     new $dataType->model_name()
            // );

            // ///insert the price
            // $price = $this->priceRepository->create([
            //     'image' => $data->image,
            //     'label' => $request->label,
            //     'price' => $request->price,
            //     'isStandard' => $request->has("entreprise") ? 1 : 0,
            //     'isRecommended' => $request->has("isRecommended") && $request->get("isRecommended") == "on" ? 1 : 0,
            //     'order' => $request->has("order") ? $request->get("order") : 0,
            //     'categoryId' => $priceCategory->id,
            //     'promoPrice' => $request->promoPrice
            // ]);

            
            $urlQuery = [];
            // if($request->has("entreprise")) {
            //     $urlQuery = ["entreprise" => $request->get("entreprise")];
            //     $this->priceEntrepriseRepository->create([
            //         'priceId' => $price->id,
            //         'entrepriseId' => $request->get("entreprise")
            //     ]);
            // }

            ///insert the combinedProduct
            

            // event(new BreadDataAdded($dataType, $data));

            if ($request->ajax()) {
                return response()->json(['success' => true]);
            }

            return redirect()
                ->route("voyager.{$dataType->slug}.index", $urlQuery)
                ->with([
                    'message'    => __('voyager::generic.successfully_added_new')." {$dataType->display_name_singular}",
                    'alert-type' => 'success',
                ]);

        }

    }
}