<?php

namespace App\Http\Controllers\Voyager;

use Illuminate\Http\Request;
use App\Models\ProductCategory;

use Voyager;
use Carbon\Carbon;
use Illuminate\Support\Facades\Cache;

class SliderController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController
{
    const SETTING_KEY = "site.sliders.settings";
    const SETTING_KEY_2 = "site.sliders.keys";


    public function index(Request $request) {
        
        $content = \DB::table('settings')->where('key', self::SETTING_KEY_2)->first()->value;
        $content = $content ? collect(json_decode($content)) : collect([]);
        
        $content = (object) $content->all();
        
        return Voyager::view("voyager::slider.index", compact(
            "content"
        ));
    }
    
    public function setImageSetting(Request $request) {
        
        $image = $request->file('image');
        $imageKey = $request->imagekey;
        
        $fileName = (string) \Str::uuid();
        $fileExtension = $image->getClientOriginalExtension();

        $imageContent = file_get_contents($image->getPathname());
        
        $path = "/slider/$fileName.$fileExtension";
        \Storage::put('public'.$path, $imageContent);
        
        $sliders = \DB::table('settings')->where('key', self::SETTING_KEY)->first()->value;
        $sliders = $sliders ? collect(json_decode($sliders)) : collect([]);
        

        $sliders->put($imageKey, $path);
        
        \DB::table('settings')->where('key', self::SETTING_KEY)->update([
            'value' => $sliders
        ]);
        
        $response = [
            'message'    => 'Slider à été traitée',
            'alert-type' => 'success',
        ];
        
        $this->clearSliderCache();

        return redirect()->back()->with($response);
    }
    
    public function getSliders() {
        
        $sliders = \DB::table('settings')->where('key', self::SETTING_KEY)->first()->value;
        $content = \DB::table('settings')->where('key', self::SETTING_KEY_2)->first()->value;

        $sliders = $sliders ? collect(json_decode($sliders)) : collect([]);
        $content = $content ? collect(json_decode($content)) : collect([]);

        return response()->json([
            'sliders' => $sliders,
            'content' => $content
        ]);
    }
    
    public function resetSliders() {
        
        \DB::table('settings')->where('key', self::SETTING_KEY)->update([
            'value' => json_encode(collect([]))
        ]);
        
        $response = [
            'message'    => 'Slider à été traitée',
            'alert-type' => 'success',
        ];
        
        $this->clearSliderCache();

        return redirect()->back()->with($response);
    }
    
    public function updateContent(Request $request) {
        
        $key = $request->key;
        $value = $request->value;
        
        if($value == "false") $value = false;
        if($value == "true") $value = true;
        
        $content = \DB::table('settings')->where('key', self::SETTING_KEY_2)->first()->value;
        
        $content = $content ? collect(json_decode($content)) : collect([]);
        
        $content->put($key, $value);
        
        \DB::table('settings')->where('key', self::SETTING_KEY_2)->update([
            'value' => json_encode($content)
        ]);
        
        $response = [
            'message'    => 'Content à été traitée',
            'alert-type' => 'success',
        ];
        
        $this->clearSliderCache();

        return redirect()->back()->with($response);
    }
    
    
    
    public function setSliderKeys() {
        
        $sliders = \DB::table('settings')->where('key', self::SETTING_KEY_2)->first()->value;
        
        $sliders = $sliders ? collect(json_decode($sliders)) : collect([]);
        
        $sliders->put("sliderPercent", "50");
        $sliders->put("parrallaxContent", "LA 2ÈME À -50%");
        
        $sliders->put("showSliderContent", true);
        $sliders->put("showParrallaxContent", true);
        
        \DB::table('settings')->where('key', self::SETTING_KEY_2)->update([
            'value' => json_encode($sliders)
        ]);
        
    }
    
    
    public function clearSliderCache(){
        Cache::forget('homeSliders');
    }


}   