<?php

namespace App\Http\Controllers\Voyager;

use Illuminate\Http\Request;

use Voyager;
use Carbon\Carbon;

class StockController extends \TCG\Voyager\Http\Controllers\VoyagerBaseController
{
    
    public function index(Request $request) {
        
        $magasin_client_id_in = \DB::table('settings')->where('key', 'site.magasin_client_id_in')->first()->value;
        $magasin_client_id_out = \DB::table('settings')->where('key', 'site.magasin_client_id_out')->first()->value;
        
        $last_in_stock_update = \DB::table('settings')->where('key', 'site.magasin_client_id_in_last_update')->first()->value;
        $last_out_stock_update = \DB::table('settings')->where('key', 'site.magasin_client_id_out_last_update')->first()->value;
        
        $stores = $this->getAllStores();
        
        $cities = $this->getAllCities();
        $cities = collect(collect($cities)->whereIn('id', collect($stores)->pluck('city_id')));
        
        foreach($stores as $store) {
            if(!isset($store->city_id)) continue;
            $city = $cities->where('id', $store->city_id)->first();
            $store->city_name = $city ? $city->intitule : "----";
            if(!isset($store->currency_tag)) $store->currency_tag = 'MAD';
        }
        
        $selectedStores = (object) [
            'in' => $magasin_client_id_in,
            'out' => $magasin_client_id_out
        ];
        
        $settingKeys = (object) [
            'in' => "site.magasin_client_id_in",
            'out' => "site.magasin_client_id_out"
        ];
        
        $lastUpdates = (object) [
            'in' => $last_in_stock_update,
            'out' => $last_out_stock_update
        ];
        
        $stockRelations = $this->geStockRelations($stores);
        
        $products = $this->getProductsLivraison();
        
        return Voyager::view("voyager::stock.index", compact(
            "stores",
            "selectedStores",
            "settingKeys",
            "lastUpdates",
            "stockRelations",
            "products"
        ));
    }
    
    public function setMagasinStock(Request $request) {
        
        $auth = auth()->user();
        $input_password = $request->authPassword;
    
        if(!\Hash::check($input_password , $auth->password)){
            $response = [
                'message'    => 'invalide Password',
                'alert-type' => 'error',
            ];

            return redirect()->back()->with($response);
        }
        
        $settingKey = $request->settingKey;
        $settingValue = $request->settingValue;
        
        \DB::table('settings')->where('key', $settingKey)->update([
            'value' => $settingValue
        ]);
        
        if($settingKey == 'site.magasin_client_id_in') $updateKey = "site.magasin_client_id_in_last_update";
        else $updateKey = "site.magasin_client_id_out_last_update";
        
        \DB::table('settings')->where('key', $settingKey)->update([
            'value' => $settingValue
        ]);
        
        \DB::table('settings')->where('key', $updateKey)->update([
            'value' => Carbon::now()
        ]);
        
        if($settingKey == 'site.magasin_client_id_in') syncProductsStock();

        
        $response = [
            'message'    => 'Le stock à été traitée',
            'alert-type' => 'success',
        ];

        return redirect()->back()->with($response);
    }
    
    public function setStockRelation(Request $request) {
        
        $auth = auth()->user();
        $input_password = $request->authPassword;
    
        if(!\Hash::check($input_password , $auth->password)){
            $response = [
                'message'    => 'invalide Password',
                'alert-type' => 'error',
            ];

            return redirect()->back()->with($response);
        }
        
        $productId = $request->productId;
        $magasinId = $request->magasinId;
        
        if($productId == 0 || $magasinId == 0) {
            $response = [
                'message'    => 'invalide Attributes',
                'alert-type' => 'info',
            ];
            
            return redirect()->back()->with($response);
        }
        
        $now = Carbon::now()->format('Y-m-d H:i:s');
        $magasin_stock_livraison = \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->first()->value;
        
        $data = collect([]);
        
        if($magasin_stock_livraison) $data = collect(json_decode($magasin_stock_livraison));
        
        $existRelation = $data->where('product_id', $productId)->first();
        
        if($existRelation) {
            $data = $data->reject(function ($el) use($productId){
                return $el->product_id == $productId;
            });
        }
        
        $data->prepend((object) [
            'product_id' => $productId,
            'magasin_id' => $magasinId,
            'date' => $now
        ]);
        
        \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->update([
            'value' => json_encode($data)
        ]);
        
        $response = [
            'message'    => 'Le stock à été traitée',
            'alert-type' => 'success',
        ];
        
        return redirect()->back()->with($response);
    }
    
    public function deleteStockRelation(Request $request) {
        
        $auth = auth()->user();
        $input_password = $request->authPassword;
        
        if(!\Hash::check($input_password , $auth->password)){
            $response = [
                'message'    => 'invalide Password',
                'alert-type' => 'error',
            ];

            return redirect()->back()->with($response);
        }
        
        $product_id = $request->productId;
        $this->deleteStockRelationByProductId($product_id);
        
        $response = [
            'message'    => 'la relation à été supprimée',
            'alert-type' => 'success',
        ];
        
        return redirect()->back()->with($response);
    }
    
    public function getProductsLivraison() {
        
        $products = \App\Models\Product::whereHas('prices', function($query){
            $query->whereHas('tags', function($query){
                $query->where('tag', 'delivery_fees');
            });
        })->get();
        
        $response = collect([]);
        
        foreach($products as $product) {
            $product_label = $product->detailKeys[0]->value;
        
            $response->push((object)[
                'id' => $product->id,
                'label' => $product_label
            ]);
        }
        
        return $response;
    }
    
    public function geStockRelations($stores = null) {
        
        if(is_null($stores)) $stores = $this->getAllStores();
        
        $magasin_stock_livraison = \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->first()->value;
        $stockLivraison = $magasin_stock_livraison ? json_decode($magasin_stock_livraison) : [];
        
        $response = collect([]);
        $stores = collect($stores);
        
        foreach($stockLivraison as $relation) {
            $product = \App\Models\Product::find($relation->product_id);
            $magasin = $stores->where('id', $relation->magasin_id)->first();
            
            if(is_null($magasin)) {
                $this->deleteStockRelationByMagasinId($relation->magasin_id);
                continue;
            }
            
            if(is_null($product)) {
                $this->deleteStockRelationByProductId($relation->product_id);
                continue;
            }
            
            $product_label = $product->detailKeys[0]->value;
            $magasin_label = $magasin->intitule;
            
            $response->push((object)[
                'product_id' => $product->id,
                'productLabel' => $product_label,
                'magasinLabel' => $magasin_label,
                'date' => $relation->date
            ]);
        }
        
        return $response;
    }
    
    public function getAllStores($with_out_central_achat = false) {
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);
        
        $url = config('urls.allStores');
        
        $response = collect(json_decode($client->get($url)->getBody()));
        
        if(!$with_out_central_achat) {
            $response->prepend((object) [
                'id' => config('constants.CENTRAL_ACHAT_ID'),
                'intitule' => "Central Achat",
            ]);
        }
        
        return $response;
    }
    
    public function getAllCities() {
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);
        
        $url = "/gatewayAP/cities";
        
        $response = json_decode($client->get($url)->getBody());
        
        return $response;
    }

    public function deleteStockRelationByProductId($product_id) {
        
        $magasin_stock_livraison = \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->first()->value;
        $stockLivraison = $magasin_stock_livraison ? json_decode($magasin_stock_livraison) : collect([]);
        
        $stockLivraison = collect($stockLivraison)->reject(function ($el) use($product_id){
            return $el->product_id == $product_id;
        });
        
        \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->update([
            'value' => json_encode($stockLivraison)
        ]);
    }
    
    public function deleteStockRelationByMagasinId($magasinId) {
        
        $magasin_stock_livraison = \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->first()->value;
        $stockLivraison = $magasin_stock_livraison ? json_decode($magasin_stock_livraison) : collect([]);
        
        $stockLivraison = collect($stockLivraison)->reject(function ($el) use($magasinId){
            return $el->magasin_id == $magasinId;
        });
        
        \DB::table('settings')->where('key', 'site.magasin_stock_livraison')->update([
            'value' => json_encode($stockLivraison)
        ]);
    }











}