<?php

namespace App\Jobs;

use App\Models\CalendarDayEntity;
use App\Models\Command;
use App\Models\CmdRemise;
use App\Models\EtatCmd;
use App\Models\Product;
use Carbon\Carbon;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Str;

class SyncCommandsToPos implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $command;
    private $commandLines;
    private $local_key;
    private $transportCompany;
    private $validating;
    private $remise;
    private $remiseAmount;
    
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($command, $commandLines, $local_key, $transportCompany, $validating = false, $remise = null, $remiseAmount = null)
    {
        $this->command = $command;
        $this->commandLines = $commandLines;
        $this->local_key = $local_key;
        $this->transportCompany = $transportCompany;
        $this->validating = $validating;
        $this->remise = $remise;
        $this->remiseAmount = $remiseAmount;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
      
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $cmdRemise = CmdRemise::where('cmdId', $this->command->id)->first();
        
        
        $hasRemise = !is_null($cmdRemise);
        
       

        $cmdClient = $this->command->client;
        
        if(!$this->validating){
            if(!$this->command->client->external_client_id) {
                $newClient = json_decode($client->post(config('urls.add-client'), [
                    'json' => [
                        'name'  => $cmdClient->user->name,
                        'email' => $cmdClient->user->email,
                        'tel' => $cmdClient->phone,
                        'address' => '---',
                        'sex' => 'homme'
                    ]
                ])->getBody());

                $this->command->client->external_client_id = $newClient->id;
                $this->command->client->save();
                
            }else {
                $newClient = (object) [
                    'id' => $this->command->client->external_client_id
                ];
            }
        }
        

        $product = Product::whereHas('feesPrices', function($query){
            $query->whereHas('commandPrice', function($subQuery){
                $subQuery->where('commandId', $this->command->id);
            })->whereHas('tags', function ($tagQuery) {
                $tagQuery->where('tag', 'delivery_fees');
            });
        })->first();

        $product_livraison_id = $product ? $product->id : null;

        $selectedStoreId = getStoreIdByRelation('out', $product_livraison_id);
    
        $commandState = EtatCmd::where('tag', 'credite')->first();
        $selectedCommandId = $commandState->id;

        if($this->validating && $this->validating['stateTag']){
            if($this->validating['stateTag'] == 'validee'){
                $commandState = EtatCmd::where('tag', 'validee')->first();
                $selectedCommandId = $commandState->id;
            }
        }
        
        $entityId = config('constants.CENTRAL_ACHAT_ID') == $selectedStoreId ? config('constants.ENTITY_CENTRAL_ACHAT_ID') : config('constants.ENTITY_MAGASIN_ID');

        $url = config('urls.start-times-v2');
        $url = str_replace("{1}", $selectedStoreId, $url);
        $url = str_replace("{2}", $entityId, $url);

        $startTimes = json_decode($client->get($url)->getBody());
        
        $entityStartTime = $startTimes->EntityStartTime[count($startTimes->EntityStartTime) - 1];
        
        $todaySession = \DB::table('settings')->where('key', 'site.session_date')->first();
        $today = Carbon::today()->format('Y-m-d');
        
        $key = 'store-' . $selectedStoreId . '-session';
        if(($todaySession && $todaySession->value != $today) || !Cache::has($key)) {
            
            $selectedCalendarDayId = CalendarDayEntity::where('date_year', $today)->first()->calendarDayEntityId;

            $startDay = json_decode($client->post(config('urls.start-day'), [
                'json' => [
                    'local_key' => Str::uuid()->toString(),
                    'updated_at' => Carbon::now()->add(1, 'hour'),
                    'created_at' => Carbon::now()->add(1, 'hour'),
                    'calendar_day_id' => (int) $selectedCalendarDayId,
                    'entity_start_time_id' => $entityStartTime->id,
                    'device_id' => null
                ]
            ])->getBody());
            $calendarDayEntityId = $startDay->id;
            
            Cache::put($key, $calendarDayEntityId);

            \DB::table('settings')->where('key', 'site.session_date')->update([
                'value' => $today,
            ]);
        }else $calendarDayEntityId = Cache::get($key);

        
        $unique_remise_local_key = Str::uuid()->toString();
        
        $cmdClients = [[
            'id' => $this->validating ? $this->command->pos_command_id : 0,
            'etat_cmd_id' => $selectedCommandId,
            'send_mail' => false,
            'created_at' => Carbon::now()->add(1, 'hour'),
            'ticket' => 'w/' . Carbon::now()->format('Y') . '/' . $this->command->id,
            'mode_paiements' => [],
            'user_id' => env('USER_API_ID'),
            'local_key' => $this->local_key,
            'mode_paiement_id' => $this->command->paymentModeId,
            'bipper' => '---',
            'updated_at' => Carbon::now()->add(1, 'hour'),
            'client_id' => $this->validating ? $this->validating['client_id'] : $newClient->id,
            'types' => $this->validating ? ["online", "validated"] : ["online", "not-validated"],
            'transport_company_id' => $this->transportCompany ? $this->transportCompany->id : null,
            "cmd_remise_id" => $cmdRemise ? $unique_remise_local_key : null,
            "commentaire" => $this->command->city. ' / '. $this->command->address->description. ' / '.$this->command->comment
        ]];        
    


       $precisionRemise = [];
        
        if($cmdRemise ){
            
            $livraisonPrice = 0;
            
            foreach($this->commandLines as $line){
                
                try{
                    $price = \App\Models\Price::find($line->prix_prd_cns_prcpl_id);
                    $product = $price->products[0];
                    
                    $description_label = $product->description->label;
                    if(strtoupper($description_label) == 'LIVRAISON') $livraisonPrice = intval($price->price);
                
                }catch(\Exception $error){
                    continue;
                }
            }
            
            $command = \App\Models\Command::find($this->command->id);
            
            $const_remise_valeur = intval(($command->total - $livraisonPrice) * $cmdRemise->valeur / 100);
            
            $addRemiseUrl = "/commandeAP/Remise/addV2";
            try{
            
                $addRemiseResponse = $client->post($addRemiseUrl, [
                    'json' => [
                        "intitule" => null,
                        "valeur"  => $const_remise_valeur,
                        "typeRemiseId" => 2,//constant Remise 
                    ]
                ]);
                
                $precisionRemise = json_decode($addRemiseResponse->getBody(),true);
                
            }catch(\Exception $e){
               
                \Log::info($e->getMessage());
            }
            
        }
        
        $cmd_remise_precision = [];
        
        if($cmdRemise && $precisionRemise && array_key_exists('local_key',$precisionRemise)){
            
            $cmd_remise_precision =  [
                        'local_key' =>  $unique_remise_local_key,
                        'user_id' => env('USER_API_ID'),
                        'remise_id' => $precisionRemise['local_key'],
                        'modified_at' => Carbon::now()->add(1, 'hour')
                   ];
        }
       
        $data = [
            'cmdClients' => $cmdClients,
            'cmdPrdCnsPrcpl' => $this->commandLines,
            'remise' => array_filter([$precisionRemise]),
            'cmdRemise' => array_filter([$cmd_remise_precision]),
            'localKey' => Str::uuid()->toString(),
            'cmdSupplement' => [],
            "cmdPrdPrcplPrdTiers" => [],
            "cmdPrdCnsTier" => [],
            "cmdEvenement" => [],
            "cmdClientAdditionalInfo" => [],
            "cmdInstruction" => [],
            "cmdPrdCnsTiersSupp" => [],
            "cmdPrdCnsPrcplException" => [],
            "cmdPrdCnsPrcplSupp" => [],
            "isWeb" => true,
            "testRemise" => "11111"
        ];
        
        if(!$this->validating && $cmdRemise) {
           
            $client->post('/gatewayAP/PointOfSell/Remise/code/validate', [
                'json' => [
                    'code' => $cmdRemise->intitule	
                ]
            ]);

            $client->post('/gatewayAP/PointOfSell/Remise/code/validate', [
                'json' => [
                    'cmdClientlocalkey' => $this->local_key
                ]
            ]);
        }
       
        if(!$this->validating) {
           

            $response = $client->post(config('urls.sync-command.subUrl1'). $calendarDayEntityId . config('urls.sync-command.subUrl2'), [
                'json' => $data
            ]);
        }
        else {
            try {
               
                $response = $client->patch(config('urls.sync-command.subUrl1'). $calendarDayEntityId . config('urls.sync-command.subUrl2'), [
                    'json' => $data
                ]);
            }catch(\Exception $error){
                $id =  $this->command->id;
                
                return;
            }

        }
       
        if($response->getStatusCode() == 200){
            try {
                // PUSH COMMAND
                if(!$this->validating){
                    $pos_command_id = json_decode($response->getBody())->CmdClient[0]->id;
                    
                    $syncedCommand = Command::find($this->command->id);
                    $syncedCommand->pos_command_id = $pos_command_id;
                    $syncedCommand->save();
                    
                // CANCEL COMMAND
                }else if($this->validating && $this->validating['stateTag']){
                    $validatedCommand = Command::find($this->command->id);
                    if($validatedCommand) {
                        $validatedCommand->canceled = true;
                        $validatedCommand->save();
                    }
                }
                // VALIDATE COMMAND
                else{
                    $validatedCommand = Command::find($this->command->id);
                    $validatedCommand->validated = true;
                    $validatedCommand->save();
                }
            }catch(\Exception $error) {
                \Log::info("[ERROR]");
            }
        }
    }
}
