<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Builder;

class Command extends Model
{

    protected $table = 'Command';
    // protected $appends = ['main_product', 'total'];
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    // protected $fillable = array('stateId', 'paymentModeId', 'parentId', 'grouped', 'clientId', 'adresseId', 'magasinId', 'planifiedFor', 'totalReduction', 'totalWithoutReduction', 'comment', 'country', 'city', 'region', 'code_postal', 'trackingNumber', 'image', 'validated', 'totalAmount', 'canceled', 'reduction');

    protected $guarded = [];

    protected static function boot() {
        parent::boot();

        // static::addGlobalScope('invited', function (Builder $builder) {
        //     $builder->whereNull('parentId');
        // });

    }

    public function getMainProductAttribute(){
        $price = $this->prices()->where("isHidden", 0)->first();
        if($price) {
            $product = $price->products()->first();
            return $product;
        } else {
            return null;
        }
    }

    public function getTotalReductionAttribute(){

        $total = $this->total_without_reduction;

        $intervalReduction = $this->intervalReductions()->first();
        $codeReduction = $this->codeReductions()->first();

        $totalReduction = 0;

        if($intervalReduction) {
            $totalReduction = $total * ($intervalReduction->value/100);
        }

        if($codeReduction) {
            $explodedValue = explode("%", $codeReduction->value);
            if(sizeOf($explodedValue) > 1) {
                $totalReduction += $total * ($explodedValue[0]/100);
            } else if(sizeOf($explodedValue) == 1) {
                $totalReduction += $explodedValue[0];
            }
        }

        return $totalReduction;

    }

    public function getTotalWithoutReductionAttribute(){

        $entreprise = $this->entreprise()->first();
        $totalBrute = 0;

        $prices = $this->prices()
            ->where("label", "!=", config('constants.DELIVERING_PRICE_LABEL'))
            ->get();

        if($entreprise) $prices = $entreprise->updatePrices($prices);

        $totalBrute = $prices->reduce(function ($carry, $item) {
            return $carry + ($item->promoPrice * $item->pivot->quantity);
        }, 0);

        return $totalBrute;

    }
    
    public function totalWithRemise() {
        
        $cmdRemise = \App\Models\CmdRemise::where('cmdId', $this->id)->first();
        $livraisonPrice = 0;
        $remiseValue = 0;

        if($cmdRemise) {
            foreach($this->lines as $line){
                try{
                    $price = \App\Models\Price::find($line["priceId"]);
                    $product = $price->products[0];
                    
                    $description_label = $product->description->label;
                    if(strtoupper($description_label) == 'LIVRAISON') $livraisonPrice = intval($price->price);
                
                }catch(\Exception $error){
                    continue;
                }
            }
            
            $remiseValue = ($this->total - $livraisonPrice) * $cmdRemise->valeur / 100;
        }

        return intval($this->total - $remiseValue);
    }


    public function getTotalAttribute(){
        $total = 0;
        foreach($this->prices as $price){
            $total += $price->promoPrice * $price->pivot->quantity;
        }
        return $total;
    }

    public function getBuyingAttribute(){
        return $this->prices->reduce(function ($carry, $item) {
            $totalProducts = $item->products->reduce(function ($subCarry, $subItem) {
                return $subCarry + $subItem->buyingPrice;
            }, 0);
            return $totalProducts + $carry;
        }, 0);
    }

    public function state() {
        return $this->belongsTo('App\Models\CommandState', 'stateId');
    }

    public function inviteCodes() {
        return $this->hasMany('App\Models\InviteCode', 'commandId');
    }

    public function payementMode() {
        return $this->belongsTo('App\Models\PayementMode', 'paymentModeId');
    }

    public function client() {
        return $this->belongsTo('App\Models\Client', 'clientId');
    }

    public function magasin() {
        return $this->belongsTo('App\Models\Magasin', 'magasinId');
    }

    public function address() {
        return $this->belongsTo('App\Models\Adresse', 'adresseId');
    }

    public function deliveryMen() {
        // return $this->hasManyThrough('App\Models\User', 'App\Models\DeliveryCommand', 'commandId', 'userId');
        return $this->belongsToMany('App\User', 'App\Models\DeliveryCommand', 'commandId', 'userId');
    }

    public function positions() {
        return $this->hasMany('App\Models\DeliveryCommand', 'commandId');
    }

    public function entreprise() {
        return $this->morphedByMany(
            'App\Models\Entreprise',
            'entityType',
            'App\Models\CommandReduction',
            'commandId',
            'entityId'
        );
    }

    public function intervalReductions() {
        return $this->morphedByMany(
            'App\Models\ReductionInterval',
            'entityType',
            'App\Models\CommandReduction',
            'commandId',
            'entityId'
        );
    }

    public function codeReductions() {
        return $this->morphedByMany(
            'App\Models\ReductionCode',
            'entityType',
            'App\Models\CommandReduction',
            'commandId',
            'entityId'
        );
    }

    public function lines() {
        return $this->hasMany(
            'App\Models\CommandPrice',
            'commandId'
        );
    }

    public function invitedCommands() {
        return $this->hasMany(
            'App\Models\Command',
            'parentId'
        );
    }

    public function inviterCommand() {
        return $this->belongsTo(
            'App\Models\Command',
            'parentId'
        );
    }

    public function prices() {
        return $this->belongsToMany(
            'App\Models\Price',
            'App\Models\CommandPrice',
            'commandId',
            'priceId'
        )
        ->withTrashed()
        ->withPivot('quantity', 'clientId');
        // ->whereNull('CommandPrice.deleted_at');
    }

    public function deliveryState()
    {
        return $this->belongsTo('App\Models\DeliveryState', 'delivery_state_id');
    }

    public function save(array $options = []) {

        parent::save();

        $this->totalReduction = $this->getTotalReductionAttribute();
        $this->totalWithoutReduction = $this->getTotalWithoutReductionAttribute();

        $this->invitedCommands()->update([
            'stateId' => $this->stateId,
            'paymentModeId' => $this->paymentModeId,
            'adresseId' => $this->adresseId,
            'magasinId' => $this->magasinId,
            'planifiedFor' => $this->planifiedFor,
            'grouped' => $this->grouped
        ]);

        parent::save();

    }

}