<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Entreprise extends Model
{

    protected $table = 'Entreprise';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('name', 'maxEmployes');

    public function updatePrices($priceList) {

        return $priceList->map(function($price) {

            $priceEntrepriseModel = $this->prices->first(function($priceEntreprise) use ($price) {
                return $priceEntreprise->priceId == $price->id;
            });

            if($priceEntrepriseModel && $priceEntrepriseModel->reduction) {
                $reductionValue = $priceEntrepriseModel->reduction->value;
                $price->promoPrice = $price->promoPrice - ($price->promoPrice * $reductionValue/100);
            }

            return $price;

        });

    }

    public function users() {
        return $this->hasMany('App\User', 'entrepriseId');
    }

    public function clientsTotal() {
        return $this->users->filter(function ($user, $key) {
            return $user->client == null ? false : true;
        })->count();
    }

    public function prices() {
        return $this->hasMany('App\Models\PriceEntreprise', 'entrepriseId');
    }

    public function entrepriseReductions() {
        return $this->hasManyThrough('App\Models\ReductionEntreprise', 'App\Models\PriceEntreprise', 'entrepriseId', 'priceEntrepriseId');
    }

}