<?php

namespace App\Models;

// use App\Models\ReductionInterval;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Builder;

class Price extends Model {

    protected $table = 'Price';
    public $timestamps = true;
    // public $appends = ["reduced_price"];

    use SoftDeletes;

    protected static function boot() {
        parent::boot();

        static::addGlobalScope('order', function (Builder $builder) {
            $builder->orderBy('order', 'asc');
        });

        static::deleting(function($price) {
            $price->combinedProducts->map(function($join) {
                return $join->delete();
            });
        });
    }

    protected $dates = ['deleted_at'];
    protected $fillable = array('isStandard', 'category_id', 'price', 'promoPrice', 'isHidden', "promotionEndAt", 'description', 'label', 'image');

    public function sync() {
        return $this->morphOne('App\Models\Sync', 'syncable');
    }

    public function category() {
        return $this->belongsTo('App\Models\PriceCategory', 'category_id')->withTrashed();
    }

    public function productCategories() {
        return $this->products->map(function($product) {
            return $product->category;
        })->unique();
    }

    public function products() {
        return $this->belongsToMany(
            'App\Models\Product',
            'App\Models\CombinedProduct',
            'price_id',
            'product_id'
        )->withTrashed()->withPivot(['quantity', 'compositionNumber']);
        // ->whereNull('CombinedProduct.deleted_at');
    }

    public function tags() {
        return $this->belongsToMany(
            'App\Models\PriceTag',
            'PriceTagPrice',
            'price_id',
            'price_tag_id'
        );
    }

    public function combinedProducts($withDeleted = true) {
        $query = $this->hasMany('App\Models\CombinedProduct', 'price_id');
        if($withDeleted) {
            $query = $query->withTrashed();
        }
        return $query;
    }

    public function reductions() {
        return $this->hasManyThrough(
            'App\Models\ReductionEntreprise',
            'App\Models\PriceEntreprise',
            'priceId',
            'priceEntrepriseId'
        );
    }

    public function reductionsByEntreprise($entrepriseId) {
        return $this->hasManyThrough(
            'App\Models\ReductionEntreprise',
            'App\Models\PriceEntreprise',
            'priceId',
            'priceEntrepriseId'
        )->where("entrepriseId", $entrepriseId);
    }

    public function user() {
        return $this->belongsTo('App\Models\PriceCategory', 'category_id');
    }

    public function reviews() {
        return $this->hasMany('App\Models\PriceReview', 'priceId');
    }

    public function detailKeys() {
        return $this->hasMany('App\Models\ProductDetailKey', 'product_id');
    }

    public function element() {
        return $this->hasMany('App\Models\ProductElement', 'product_id');
    }

    public function commandPrice() {
        return $this->hasOne('App\Models\CommandPrice', 'priceId');
    }

    public function save(array $options = []){
        $this->isStandard = ( !isset($this->isStandard) || $this->isStandard == "" ) ? false : $this->isStandard;
        $this->promoPrice = ( !isset($this->promoPrice) || $this->promoPrice == "" ) ? $this->price : $this->promoPrice;
        $result = parent::save();
        $this->sync()->save(New Sync);
        return $result;
    }
}
