<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Builder;

class PriceCategory extends Model
{

    protected $table = 'PriceCategory';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('label', 'order', 'file_id', 'tag', 'tva', 'parent_id', 'description', 'ar_label');

    protected static function boot() {
        parent::boot();

        static::addGlobalScope('order', function (Builder $builder) {
            $builder->orderBy('order', 'asc')->orderBy('created_at', 'desc');
        });

        //DELETE Prices (IF THERE IS ONE), DELETE COMBINEDPRODUCT
        static::deleting(function($category) {
            $category->prices()->delete();
        });

    }

    public function sync() {
        return $this->morphOne('App\Models\Sync', 'syncable');
    }

    public function parent() {
        return $this->belongsTo('App\Models\PriceCategory', 'parent_id');
    }

    public function parentTree() {
        return $this->parent()->with("parentTree");
    }

    public function children() {
        return $this->hasMany('App\Models\PriceCategory', 'parent_id');
    }

    public function childrenTree() {
        return $this->children()->with("childrenTree");
    }

    public function prices() {
        return $this->hasMany('App\Models\Price', 'category_id')->withTrashed();
    }

    public function save(array $options = [])
    {
        $this->tag = ( !isset($this->tag) || $this->tag == "" ) ? \Str::slug($this->label,"_") : $this->tag;
        $result = parent::save();
        $this->sync()->save(New Sync);
        return $result;
    }

}