<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends Model
{

    protected $table = 'Product';
    public $timestamps = true;

    use SoftDeletes;

    protected static function boot()
    {
        parent::boot();

        static::deleting(function ($product) {
            $prices = $product->prices;
            $product->combinedProducts->map(function ($join) {
                return $join->delete();
            });
            $prices->map(function ($price) {
                $totalCombination = $price->combinedProducts(false)->count();
                if ($totalCombination == 0) $price->delete();
            });
        });
    }

    protected $dates = ['deleted_at'];
    protected $fillable = array('file_id', 'product_description_id', 'description', 'image', 'buyingPrice', 'tva', 'barcode', 'file_path');

    public function getFullNameAttribute()
    {
        $attributes = $this->detailKeys->reduce(function ($carry, $item) {
            return $carry . ($carry == "" ? "" : " / ") . $item->value;
        }, "");
        $attributes = $attributes != "" ? " (" . $attributes . ")" : " ";
        return $this->description->label . $attributes;
    }

    public function feesPrices()
    {
        $query = $this->belongsToMany(
            'App\Models\Price',
            'App\Models\CombinedProduct',
            'product_id',
            'price_id'
        )->withTrashed();

        return $query;
    }

    public function sync()
    {
        return $this->morphOne('App\Models\Sync', 'syncable');
    }

    public function combinedProducts()
    {
        return $this->hasMany('App\Models\CombinedProduct', 'product_id');
    }

    public function prices($withDeleted = false)
    {

        $query = $this->belongsToMany(
            'App\Models\Price',
            'App\Models\CombinedProduct',
            'product_id',
            'price_id'
        )->withPivot('quantity');

        if (!$withDeleted) {
            $query = $query->whereNull('CombinedProduct.deleted_at');
        }

        return $query;
    }

    public function detailKeys()
    {
        return $this->hasMany('App\Models\ProductDetailKey', 'product_id');
    }

    public function element()
    {
        return $this->hasMany('App\Models\ProductElement', 'product_id');
    }

    public function description()
    {
        return $this->belongsTo('App\Models\ProductDescription', 'product_description_id')->withTrashed();
    }

    public function details()
    {

        return $this->belongsToMany(
            'App\Models\DetailKey',
            'App\Models\ProductDetailKey',
            'product_id',
            'detail_key_id'
        )->withPivot('value');
    }

    public function save(array $options = [])
    {
        $result = parent::save();
        $this->sync()->save(new Sync);
        return $result;
    }

    public function product_category()
    {
        return $this->belongsToMany(
            'App\Models\ProductProductCategory',
            'App\Models\ProductCategory',
            'productId',
            'categoryId'
        )->withPivot('value');
    }

    public function stock()
    {
        return $this->hasOne('App\Models\StockOperationProduct', 'productId');
    }
}
