<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Builder;

use App\Models\PriceCategory;

class ProductCategory extends Model
{

    protected $table = 'ProductCategory';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('label', 'order', 'file_id', 'tag', 'tva', 'parent_id', 'description', 'ar_label');

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('order', function (Builder $builder) {
            $builder->orderBy('order', 'asc')->orderBy('created_at', 'desc');
        });

        //DELETE PRODUCTS (IF THERE IS ONE), DELETE COMBINEDPRODUCT
        static::deleting(function ($category) {
            // $category->products()->delete();
        });
    }

    public function getFamilyPathAttribute()
    {
        $element = $this;
        $familyPath = "";
        $index = 0;
        while ($element->parent != null) {
            if ($index != 0) $familyPath .= " / ";
            $familyPath .= $element->parent->label;
            $element = $element->parent;
            $index++;
        }
        return $familyPath;
    }

    public function sync()
    {
        return $this->morphOne('App\Models\Sync', 'syncable');
    }

    public function parent()
    {
        return $this->belongsTo('App\Models\ProductCategory', 'parent_id');
    }

    public function childrens()
    {
        return $this->hasMany('App\Models\ProductCategory', 'parent_id');
    }

    public function products()
    {
        return $this->hasMany('App\Models\Product', 'categoryId')->withTrashed();
    }

    public function save(array $options = [])
    {
        $this->tag = (!isset($this->tag) || $this->tag == "") ? \Str::slug($this->label, "_") : $this->tag;
        $result = parent::save();
        $this->sync()->save(new Sync);
        return $result;
    }

    public function otherProducts()
    {
        return $this->belongsToMany(
            'App\Models\Product',
            'App\Models\ProductProductCategory',
            'categoryId',
            'productId'
        );
    }
}
