<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class ProductDescription extends Model {

    protected $table = 'ProductDescription';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('label', 'image', 'weight', 'brand_id', 'category_id', 'buyingPrice', 'returnable', 'criticalQuantity', 'file_id', 'description', 'file_path', 'ar_label');

    public function sync() {
        return $this->morphOne('App\Models\Sync', 'syncable');
    }

    public function brand() {
        return $this->belongsTo('App\Models\Brand', "brand_id");
    }

    public function category() {
        return $this->belongsTo('App\Models\ProductCategory', "category_id");
    }

    public function products() {
        return $this->hasMany('App\Models\Product', 'product_description_id')->withTrashed();
    }

    public function availability()
    {
        return $this->hasOne('App\Models\ProductAvailability', 'product_description_id');
    }

    public function save(array $options = [])
    {
        $result = parent::save();
        $this->sync()->save(New Sync);
        return $result;
    }

}