<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

use Carbon\Carbon;

class StockOperation extends Model
{

    protected $table = 'StockOperation';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('magasinId', 'userId', 'typeId', 'planifiedFor');

    protected static function boot() {
        parent::boot();

        //DELETE PRODUCTS (IF THERE IS ONE), DELETE COMBINEDPRODUCT
        static::deleting(function($operation) {
            $operation->operationProducts()->delete();
        });
    }

    public function operationProducts() {
        return $this->hasMany('App\Models\StockOperationProduct', 'operationId');
    }

    public function products() {
        return $this->belongsToMany(
            'App\Models\Product',
            'App\Models\StockOperationProduct',
            'operationId',
            'productId'
        )->withPivot('quantity')
        ->whereNull('StockOperationProduct.deleted_at');
    }

    public function save(array $options = [])
    {
        $this->planifiedFor = ( !isset($this->planifiedFor) || $this->planifiedFor == "" ) ? Carbon::now()->weekday() : $this->planifiedFor;
        parent::save();
    }

    public function magasin() {
        return $this->belongsTo('App\Models\Magasin', "magasinId");
    }

    public function stockOperationType() {
        return $this->belongsTo('App\Models\StockOperationType', "typeId");
    }

    public function user() {
        return $this->belongsTo('App\User', "userId");
    }

}