<?php

namespace App\Repositories;

use App\Models\Client;
use App\Models\ReductionCodeClient;
use App\Repositories\BaseRepository;

/**
 * Class ClientRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
*/

class ClientRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [

    ];

    public function deleteReductionCodes($codeId) {
        return ReductionCodeClient::where('reductionCodeId', $codeId)->delete();
    }

    public function addReductionCode($clientId, $codeId) {
        return ReductionCodeClient::firstOrCreate(
            ['clientId' => $clientId, 'reductionCodeId' => $codeId]
        );
    }

    public function findByEntreprises($entrepriseListId) {
        return $this->model->whereHas("user", function($query) use ($entrepriseListId) {
            $query->whereIn("entrepriseId", $entrepriseListId);
        })->get();
    }

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }

    public function findClientWithEmail($email) {
        return $this->model->whereHas("user", function($query) use ($email) {
            $query->where("email", $email)->whereNull("entrepriseId");
        })->first();
    }

    public function addClientToEntreprise($clientId, $entrepriseId) {
        $client = $this->find($clientId);
        $user = $client->user;
        $user->entrepriseId = $user->entrepriseId ? null : $entrepriseId;
        $user->save();
    }

    public function create($input)
    {
        $client = $this->model->where("userId", $input["userId"])->first();

        if($client == null) {
            $model = $this->model->newInstance($input);
            $model->save();
            return $model;
        } else {
            return $client;
        }

    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return Client::class;
    }
}
