<?php

namespace App\Repositories;

use App\Models\CommandReduction;
use App\Models\ReductionCode;
use App\Models\ReductionInterval;
use App\Repositories\BaseRepository;

/**
 * Class CommandReductionRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
*/

class CommandReductionRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [

    ];

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }


    public function getCodeIfAvailable($code, $clientId = null) {

        $codeReduction = ReductionCode::where("tag", $code)
            ->whereColumn("used", "<", "maxUsed")
            ->first();

        if($codeReduction && $codeReduction->clients->count() > 0) {
            $authorizedClients = $codeReduction->clients->pluck("id")->toArray();
            if($clientId == null || !in_array($clientId, $authorizedClients)) $codeReduction = null;
        }

        return $codeReduction;

    }

    public function addCodeReduction($input) {

        $codeReduction = $this->getCodeIfAvailable(
            $input["code"],
            $input["clientId"]
        );

        if(!$codeReduction) { return null; } else {
            $codeReduction->used += 1;
            $codeReduction->save();
            return $this->model->firstOrCreate([
                "commandId" => $input["commandId"],
                "entityType_type" => "ReductionCode",
                "entityId" => $codeReduction->id
            ]);
        }

    }

    public function createEntrepriseReduction($input) {
        return $this->model->firstOrCreate([
            "commandId" => $input["commandId"],
            "entityType_type" => "ReductionEntreprise",
            "entityId" => $input["entrepriseId"]
        ]);
    }

    public function createIntervalReduction($input) {
        return $this->model->firstOrCreate([
            "commandId" => $input["commandId"],
            "entityType_type" => "ReductionInterval",
            "entityId" => $input["intervalReductionId"]
        ]);
    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return CommandReduction::class;
    }
}
