<?php

namespace App\Repositories;

use App\Models\Command;
use App\Repositories\BaseRepository;
use Carbon\Carbon;
use ArPHP\I18N\Arabic;
/**
 * Class CommandRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
*/

class CommandRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [

    ];

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable() {
        return $this->fieldSearchable;
    }

    public function generatePdfFile($commandId){

        $command = $this->find($commandId);
        $file = str_slug($command->client->user->name)."-".Carbon::now()->timestamp.".pdf";
        $folderName = 'tickets/';
        $folderPath = storage_path('app/public/'.$folderName);

        $pdfFile = $this->textHTML($command)
        ->setPaper('a5', 'portrait');

        // we dont save the file at this moment ->save($folderPath, $file)

        if(!\File::isDirectory($folderPath)){
            \File::makeDirectory($folderPath, 0777, true, true);
        }

        return $pdfFile;
    }

    public function textHTML($command){
        $reportHtml = view('pdf.validatedCommand', ["command" => $command])->render();
        
        $arabic = new Arabic();
        $p = $arabic->arIdentify($reportHtml);

        for ($i = count($p)-1; $i >= 0; $i-=2) {
            $utf8ar = $arabic->utf8Glyphs(substr($reportHtml, $p[$i-1], $p[$i] - $p[$i-1]));
            $reportHtml = substr_replace($reportHtml, $utf8ar, $p[$i-1], $p[$i] - $p[$i-1]);
        }

        $pdf = \PDF::loadHTML($reportHtml);
        
        return $pdf;
    }

    public function findBetween($between = [], $productIdList = [], $columns = ['*']) {
        $query = $this->model->newQuery();

        $query = $query->whereBetween("created_at", $between);
        if(sizeOf($productIdList) > 0) {
            $query = $query->whereHas("prices", function($priceQuery) use ($productIdList) {
                $priceQuery->whereHas("products", function($productQuery) use ($productIdList) {
                    $productQuery->whereIn("Product.id", $productIdList);
                });
            });
        }

        $query = $query->orderBy("created_at", "desc");

        return $query->get($columns);
    }

    public function getClientCommands($clientId) {
        $waitingTag = config("constants.DEFAULT_WAITING_INVITATION_COMMAND_STATE")["tag"];
        return $this->model
        ->with("state", "payementMode", "address", "prices.products", "prices.products.description")
            // ->with("state", "payementMode", "address", "prices.products", "prices.pivot.client.user")
            ->where("clientId", $clientId)
            ->whereHas("state", function($query) use ($waitingTag) {
                $query->where("tag", "!=", $waitingTag);
            })
            ->orderBy("created_at", "desc")
            ->get();
    }

    public function getDeliveryManCommands($userId, $stateId) {

        $commands = $this->model
            ->with("state", "payementMode", "address.geographicPoint", "prices.products", "prices.pivot.client.user:id,name")
            ->whereHas("deliveryMen", function($query) use ($userId) {
                $query->where("userId", $userId);
            })
            ->whereNull("parentId")
            ->where("stateId", $stateId)
            ->orderBy("created_at", "desc")
            ->get();

        $commands = $commands->map(function($data) {

            $invitedCommands = $this->model->where("parentId", $data->id)
                ->with("prices.products", "prices.pivot.client.user:id,name")
                ->get();

            $total = $data->total + $invitedCommands->reduce(function($total, $command) {
                return $command->total + $total;
            },0);

            $data->customTotal = $total;
            $data->customPrices = $data->prices;

            $invitedCommands->each(function($invitedCommand) use ($data) {
                $invitedCommand->prices->each(function($invitedPrices) use ($data) {
                    $data->customPrices->push($invitedPrices);
                });
            });

            return $data;
        });

        return $commands;
    }

    public function getCommandByNumber($trackingNum)
    {
        return $this->model->where('trackingNumber', $trackingNum)
            ->first();
    }

    public function getCommandByPosId($posId)
    {
        return $this->model->where('pos_command_id', $posId)->with('client.user', 'address', 'payementMode')->first();
    }

    public function getCommandsListIds()
    {
        $user = \JWTAuth::toUser();
        return $this->model->whereNotNull('pos_command_id')->where('clientId', $user->client->id)->get()->pluck('pos_command_id');
    }

    public function getNotSyncedCommands()
    {
        $user = \JWTAuth::toUser();
        return $this->model->whereNull('pos_command_id')
            ->where('clientId', $user->client->id)
            ->with('client.user', 'address', 'payementMode', 'prices.commandPrice', 'prices.products.description', 'prices.products.detailKeys.detailKey')
            ->get()
            ->toArray();
    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return Command::class;
    }
}
