<?php

namespace App\Repositories;

use App\Models\PriceCategory;
use App\Repositories\BaseRepository;

/**
 * Class PriceCategoryRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
*/

class PriceCategoryRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [

    ];

    public function getFilledCategories() {
        return $this->model->whereHas("prices")->get();
    }

    public function getRootCategories($isDefault = null){
        $query = $this->model->newQuery();
        if($isDefault) $query = $query->where('isDefault', true);
        return $query->with("childrenTree")->where("parent_id", null)->orderBy('order', 'desc')->get();
    }

    public function getCategoryTree($id){
        $query = $this->model->newQuery();
        return $query->where("id", $id)->with("childrenTree")->first();
    }

    public function getDefault(){

        $categoryInstance = [
            "label" => config('constants.DEFAULT_PRICE_CATEGORY')
        ];

        return $this->model->firstOrCreate($categoryInstance, []);

    }

    public function getByTag($tag){

        return $this->model->firstOrCreate([
            "tag" => $tag
        ], [
            "label" => $tag,
            'order' => 0,
            'description' => $tag
        ]);

    }

    public function create($input)
    {
        return $this->model->updateOrCreate([
            "id" => key_exists('id', $input) ? $input["id"] : null,
        ],[
            "tag" => $input["tag"],
            "label" => $input["label"],
            "parent_id" => key_exists('parentId', $input) ? $input["parentId"] : null,
            "description" => $input["description"]
        ]);
    }

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return PriceCategory::class;
    }
}
