<?php

namespace App\Repositories;

use Illuminate\Container\Container as Application;
use App\Models\ProductCategory;
use App\Repositories\BaseRepository;

/**
 * Class ProductCategoryRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
 */

class ProductCategoryRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [];

    /**
     * @var array
     */
    protected $rootCategoryTags = [];

    public function __construct(Application $app)
    {
        parent::__construct($app);
        $this->rootCategoryTags = [
            config("constants.SEMIFINI_PRODUCTS_CATEGORY"),
            config("constants.INBOUND_PRODUCTS_CATEGORY"),
            config("constants.PRIMARY_PRODUCTS_CATEGORY"),
            config("constants.SECONDARY_PRODUCTS_CATEGORY"),
            config("constants.SUPPLEMENT_PRODUCTS_CATEGORY")
        ];
    }

    public function getFamiliesByParentTag($tag)
    {
        return $this->model->whereHas("parent", function ($query) use ($tag) {
            $query->where("tag", $tag);
        });
    }

    public function getFamilies()
    {
        return $this->model->whereHas("parent", function ($query) {
            $query->whereIn("tag", $this->rootCategoryTags);
        });
    }

    public function getCategories()
    {
        return $this->model->whereHas("parent", function ($query) {
            $query->whereNotIn("tag", $this->rootCategoryTags);
        });
    }

    public function insertTagedCategories()
    {

        foreach ($this->rootCategoryTags as $key => $tag) {
            $this->model->firstOrCreate([
                "tag" => $tag
            ], [
                "label" => $tag,
                "order" => 0,
                "parentId" => null,
                "description" => "---"
            ]);
        }
    }


    public function getProductCategory()
    {
        return $this->model->whereHas('otherProducts')->get();
    }

    public function getDefault()
    {

        $categoryInstance = [
            "label" => config('constants.DEFAULT_PRODUCT_CATEGORY')
        ];


        return $this->model->firstOrCreate($categoryInstance, []);
    }

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return ProductCategory::class;
    }
}
