<?php

namespace App\Repositories;

use App\Models\StockOperation;
use App\Models\StockOperationType;
use App\Repositories\BaseRepository;

use Carbon\Carbon;

/**
 * Class StockOperationRepository
 * @package App\Repositories
 * @version August 26, 2019, 2:18 pm UTC
*/

class StockOperationRepository extends BaseRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [

    ];

    public function create($input)
    {

        $type = StockOperationType::where("tag", "planification")->first();

        if($input["typeId"] == $type->id) {

            $models = $this->allQuery([
                "typeId" => $input["typeId"],
                "magasinId" => $input["magasinId"],
                "planifiedFor" => $input["planifiedFor"]
            ]);

            $models->delete();

        }

        return $this->model->create([
            "typeId" => $input["typeId"],
            "magasinId" => $input["magasinId"],
            "planifiedFor" => $input["planifiedFor"],
            "userId" => $input["userId"]
        ]);

    }

    public function getPlanifiedProducts() {
        return $this->model->select("StockOperationProduct.productId AS productId")
            ->join('StockOperationProduct', 'StockOperation.id', '=', 'StockOperationProduct.operationId')
            ->join('StockOperationType', 'StockOperation.typeId', '=', 'StockOperationType.id')
            ->where('StockOperationType.tag', 'planification')
            // ->where('StockOperationProduct.quantity', '>', 0)
            ->groupBy('StockOperationProduct.productId')
            ->get()
            ->pluck("productId");
    }

    public function stockStatus($productList = null) {

        $planifiedOperations = $this->model->selectRaw("productId, SUM(StockOperationProduct.quantity * StockOperationType.multiplier) total")
            ->join('StockOperationProduct', 'StockOperation.id', '=', 'StockOperationProduct.operationId')
            ->join('StockOperationType', 'StockOperation.typeId', '=', 'StockOperationType.id')
            ->where('StockOperationType.tag', 'planification')
            ->groupBy('StockOperationProduct.productId')
            ->having('total', '>', 0);

        if($productList) $planifiedOperations = $planifiedOperations->whereIn('StockOperationProduct.productId', $productList)->get();
        else $planifiedOperations = $planifiedOperations->get();

        $dailyOperations = $this->model->selectRaw("productId, SUM(StockOperationProduct.quantity * StockOperationType.multiplier) total")
            ->join('StockOperationProduct', 'StockOperation.id', '=', 'StockOperationProduct.operationId')
            ->join('StockOperationType', 'StockOperation.typeId', '=', 'StockOperationType.id')
            ->where('StockOperationType.tag', '!=', 'planification')
            ->groupBy('StockOperationProduct.productId')
            ->having('total', '>', 0);

        if($productList) $dailyOperations = $dailyOperations->whereIn('StockOperationProduct.productId', $productList)->get();
        else $dailyOperations = $dailyOperations->get();

        return $planifiedOperations->map(function($planified) use ($dailyOperations) {
            $dailyOperation = $dailyOperations->first(function($operation) use ($planified){
                return $operation->productId == $planified->productId ? true : false;
            });
            if($dailyOperation) $planified->total += $dailyOperation->total;
            return $planified;
        });
    }

    /**
     * Return searchable fields
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }

    /**
     * Configure the Model
     **/
    public function model()
    {
        return StockOperation::class;
    }
}
