<?php

namespace App\Services;

use Illuminate\Http\Request;
use Carbon\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Response as GuzzleResponse;

class ApiCallServiceImpl implements Interfaces\ApiCallService {

    /*
     to send a request to the gateway and get the url of the microservices

     - Parameters:
     - functionName: the name of the service.
     - params: the parameters to generate url.

     - Returns: String which is the url of the MS.
    */
    public function getPath(string $functionName, array $params): String {

        $path = config("constants.ApiPathTags")[$functionName];

        foreach ($params as $key => $param) {
            $path = str_replace("{". ($key + 1) ."}", $param, $path);
        }

        return $path;

    }

    /*
     demand an url and call that url to get the response from another microservice
     this method allow a microservice to discover the other microservices and grab the data from them

     - Parameters:
     - container: an object which we can get from the request to make another request
     - method: the type of the method you wanna request GET/PUT/POST
     - decoder: the type of the object you want you result to convert to
     - functionName: the name of the service.
     - params: the parameters to generate url.

     - Returns: an object of the same type you put on the decoder
     */
    public function getMSdata(string $method, string $functionName, array $params, array $data = []): array {

        // $urlQuery = str_replace(\Request::url(), '', \Request::fullUrl());
        $urlQuery = "";
        $headers = app('request')->headers->all();
        $bearer = app('request')->input('token');
        array_push($headers, ["Authorization" => "Bearer ".$bearer]);

        return $this->requestMsdata(
            $method,
            $urlQuery,
            $functionName,
            $params,
            $data,
            $headers
        );

    }

    //TODO:add comments
    public function requestMsdata(string $method, string $urlQuery, string $functionName, array $params, array $data = [], array $headers = []): array {

        ///the url of the microservice
        $url = $this->getPath($functionName, $params);

        // $urlQuery = str_replace(\Request::url(), '', \Request::fullUrl());
        $requestedUrlQuery = $urlQuery != "" ? $urlQuery."&" : "";

        //the client route
        $client = new Client();

        $headers = [
            'Content-Type' => 'application/json',
            'User-Date' => array_key_exists("user-date", $headers) ? $headers["user-date"][0] : null,
            'External-Api-Authorization' => array_key_exists("External-Api-Authorization", $headers) ? $headers["External-Api-Authorization"][0] : null,
            'Authorization' => array_key_exists("authorization", $headers) ? $headers["authorization"][0] : null,
            'precisionIdentifier' => array_key_exists("precisionIdentifier", $headers) ? $headers["precisionIdentifier"][0] : null,
            'buildNumber' => array_key_exists("buildNumber", $headers) ? $headers["buildNumber"][0] : null
        ];

        $client = new Client([
            'headers'  => $headers
        ]);

        //generate the httprequest
        // $requested_url = str_replace("localhost", "192.168.1.5", $jsonResponse["url"]);
        $requested_url = $url;

        \Log::info("REQUESTING ---- " . $requested_url . $requestedUrlQuery);
        $msResponse = $client->request(
            $method,
            $requested_url . $requestedUrlQuery,
            [ "json" => $data ]
        );

        return json_decode($msResponse->getBody(), true);

    }
}