<?php

namespace App\Services;

use Illuminate\Http\Request;
use Carbon\Carbon;

use App\Repositories\PriceRepository;
use App\Repositories\CommandPriceRepository;
use App\Repositories\ClientRepository;
use App\Repositories\CommandRepository;
use App\Repositories\CommandReductionRepository;
use App\Repositories\ReductionCodeRepository;
use App\Repositories\ReductionIntervalRepository;

class CommandServiceImpl implements Interfaces\CommandService {

    protected $clientRepository;
    protected $priceRepository;
    protected $commandPriceRepository;
    protected $commandRepository;
    protected $commandReductionRepository;
    protected $reductionCodeRepository;
    protected $reductionIntervalRepository;

    public function __construct(
        PriceRepository $priceRepository,
        CommandPriceRepository $commandPriceRepository,
        ClientRepository $clientRepository,
        CommandRepository $commandRepository,
        CommandReductionRepository $commandReductionRepository,
        ReductionCodeRepository $reductionCodeRepository,
        ReductionIntervalRepository $reductionIntervalRepository
    ) {
        $this->priceRepository = $priceRepository;
        $this->commandPriceRepository = $commandPriceRepository;
        $this->clientRepository = $clientRepository;
        $this->commandRepository = $commandRepository;
        $this->commandReductionRepository = $commandReductionRepository;
        $this->reductionCodeRepository = $reductionCodeRepository;
        $this->reductionIntervalRepository = $reductionIntervalRepository;
    }

    function checkDeliveringPrice($commandId) {

        $command = $this->commandRepository->find($commandId);

        $total = $command->total + $command->invitedCommands->reduce(function ($carry, $item) {
            return $carry + $item->total;
        }, 0);

        if($total < setting('constants.DELIVERING_FREE_FROM_PRICE')) {

            $deliveringPrice = $this->priceRepository->getDeliveringPrice();

            $this->commandPriceRepository->create([
                "clientId" => $command->clientId,
                "commandId" => $command->id,
                "priceId" => $deliveringPrice->id,
                "quantity" => 1
            ]);

            $command = $this->commandRepository->find(
                $command->id
            );

        }

        return $command;

    }

    function addCommand($inputs) {

        $client = $this->clientRepository->find($inputs["clientId"]);

        $command = $this->commandRepository->create([
            'parentId' => array_key_exists("parentId", $inputs) ? $inputs["parentId"] : null,
            'adresseId' => $inputs["adresseId"],
            'stateId' => $inputs["stateId"],
            'paymentModeId' => $inputs["paymentModeId"],
            'clientId' => $inputs["clientId"],
            // 'planifiedFor' => $inputs["planifiedFor"],
            'comment' => $inputs["comment"],
            'country' => $inputs["country"],
            'city' => $inputs["city"],
            'region' => $inputs["region"],
            'code_postal' => $inputs["code_postal"],
            'totalAmount' => $inputs["totalAmount"]
        ]);

        if($client->user->entrepriseId) {
            $this->commandReductionRepository->createEntrepriseReduction([
                "commandId" => $command->id,
                "entrepriseId" => $client->user->entrepriseId
            ]);
        }

        if(array_key_exists("codePromo", $inputs) && $inputs["codePromo"]) {
            $this->commandReductionRepository->addCodeReduction([
                "clientId" => $client->id,
                "commandId" => $command->id,
                "code" => $inputs["codePromo"]
            ]);
        }

        // $reductionInterval = $this->reductionIntervalRepository->reductionByHour(
        //     $command->planifiedFor
        // );

        // if($reductionInterval) {
        //     $this->commandReductionRepository->createIntervalReduction([
        //         "commandId" => $command->id,
        //         "intervalReductionId" => $reductionInterval->id
        //     ]);
        // }

        return $command;

    }

}