<?php

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class AddForeignKeysConstraintTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {

        Schema::table('PriceCategory', function (Blueprint $table) {
            $table->foreign('parentId')->references('id')->on('PriceCategory')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('Price', function (Blueprint $table) {
            $table->foreign('categoryId')->references('id')->on('PriceCategory')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('PriceTagPrice', function (Blueprint $table) {
            $table->foreign('priceId')->references('id')->on('Price')
				->onDelete('cascade')->onUpdate('cascade');
            $table->foreign('priceTagId')->references('id')->on('PriceTag')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('ProductCategory', function (Blueprint $table) {
            $table->foreign('parentId')->references('id')->on('ProductCategory')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('ProductDetailKey', function (Blueprint $table) {
            $table->foreign('productId')->references('id')->on('Product')
				->onDelete('cascade')->onUpdate('cascade');
            $table->foreign('detailKeyId')->references('id')->on('DetailKey')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('ProductElement', function (Blueprint $table) {
            $table->foreign('productId')->references('id')->on('Product')
				->onDelete('cascade')->onUpdate('cascade');
            $table->foreign('elementId')->references('id')->on('Product')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('ProductDescription', function (Blueprint $table) {
            $table->foreign('brandId')->references('id')->on('Brand')
				->onDelete('cascade')->onUpdate('cascade');
            $table->foreign('categoryId')->references('id')->on('ProductCategory')
				->onDelete('cascade')->onUpdate('cascade');
            $table->foreign('supplierId')->references('id')->on('Supplier')
				->onDelete('cascade')->onUpdate('cascade');
        });

        Schema::table('Product', function (Blueprint $table) {
            $table->foreign('productDescriptionId')->references('id')->on('ProductDescription')
				->onDelete('cascade')->onUpdate('cascade');
        });
        Schema::table('InviteCode', function(Blueprint $table) {
			$table->foreign('commandId')->references('id')->on('Command')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Magasin', function(Blueprint $table) {
$table->foreign('adresseId')->references('id')->on('Adresse')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Command', function(Blueprint $table) {
$table->foreign('adresseId')->references('id')->on('Adresse')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('UserMagasin', function(Blueprint $table) {
$table->foreign('userId')->references('id')->on('users')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('UserMagasin', function(Blueprint $table) {
$table->foreign('magasinId')->references('id')->on('Magasin')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Client', function(Blueprint $table) {
$table->foreign('userId')->references('id')->on('users')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('users', function(Blueprint $table) {
$table->foreign('entrepriseId')->references('id')->on('Entreprise')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Adresse', function(Blueprint $table) {
$table->foreign('geographicPointId')->references('id')->on('GeographicPoint')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('ClientPrice', function(Blueprint $table) {
$table->foreign('priceId')->references('id')->on('Price')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('ClientPrice', function(Blueprint $table) {
$table->foreign('clientId')->references('id')->on('Client')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('ProductSourcing', function(Blueprint $table) {
$table->foreign('productId')->references('id')->on('Product')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('ProductSourcing', function(Blueprint $table) {
$table->foreign('sourcingId')->references('id')->on('Sourcing')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Command', function(Blueprint $table) {
$table->foreign('stateId')->references('id')->on('CommandState')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Command', function(Blueprint $table) {
$table->foreign('paymentModeId')->references('id')->on('PayementMode')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Command', function(Blueprint $table) {
$table->foreign('clientId')->references('id')->on('Client')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Command', function(Blueprint $table) {
$table->foreign('magasinId')->references('id')->on('Magasin')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CommandPrice', function(Blueprint $table) {
$table->foreign('clientId')->references('id')->on('Client')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CommandPrice', function(Blueprint $table) {
$table->foreign('commandId')->references('id')->on('Command')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CommandPrice', function(Blueprint $table) {
$table->foreign('priceId')->references('id')->on('Price')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CommandReduction', function(Blueprint $table) {
$table->foreign('commandId')->references('id')->on('Command')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('ReductionEntreprise', function(Blueprint $table) {
$table->foreign('priceEntrepriseId')->references('id')->on('PriceEntreprise')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('StockOperation', function(Blueprint $table) {
$table->foreign('magasinId')->references('id')->on('Magasin')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('StockOperation', function(Blueprint $table) {
$table->foreign('userId')->references('id')->on('users')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('StockOperation', function(Blueprint $table) {
$table->foreign('typeId')->references('id')->on('StockOperationType')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('StockOperationProduct', function(Blueprint $table) {
$table->foreign('operationId')->references('id')->on('StockOperation')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('StockOperationProduct', function(Blueprint $table) {
$table->foreign('productId')->references('id')->on('Product')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('Notification', function(Blueprint $table) {
$table->foreign('userId')->references('id')->on('users')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CombinedProduct', function(Blueprint $table) {
$table->foreign('productId')->references('id')->on('Product')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('CombinedProduct', function(Blueprint $table) {
$table->foreign('priceId')->references('id')->on('Price')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('PriceEntreprise', function(Blueprint $table) {
$table->foreign('priceId')->references('id')->on('Price')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

Schema::table('PriceEntreprise', function(Blueprint $table) {
$table->foreign('entrepriseId')->references('id')->on('Entreprise')
            ->onDelete('cascade')
            ->onUpdate('cascade');
});

    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('PriceCategory', function (Blueprint $table) {
            $table->dropForeign(['parentId']);
        });

        Schema::table('Price', function (Blueprint $table) {
            $table->dropForeign(['categoryId']);
        });

        Schema::table('PriceTag', function (Blueprint $table) {
            $table->dropForeign(['parentId']);
        });

        Schema::table('ProductCategory', function (Blueprint $table) {
            $table->dropForeign(['parentId']);
        });

        Schema::table('ProductDetailKey', function (Blueprint $table) {
            $table->dropForeign(['productId']);
            $table->dropForeign(['detailKeyId']);
        });

        Schema::table('ProductElement', function (Blueprint $table) {
            $table->dropForeign(['productId']);
            $table->dropForeign(['elementId']);
        });

        Schema::table('Product', function (Blueprint $table) {
            $table->dropForeign(['brandId']);
            $table->dropForeign(['categoryId']);
        });

        Schema::table('PriceTagPrice', function (Blueprint $table) {
            $table->dropForeign(['price_id']);
            $table->dropForeign(['price_tag_id']);
        });
        
        Schema::table('InviteCode', function(Blueprint $table) {
			$table->dropForeign('InviteCode_commandId_foreign');
		});
        
        Schema::table('InviteCode', function(Blueprint $table) {
			$table->dropForeign('InviteCode_userId_foreign');
		});
        
        Schema::table('GeographicPoint', function(Blueprint $table) {
			$table->dropForeign('GeographicPoint_magasinId_foreign');
		});
        
        Schema::table('UserMagasin', function(Blueprint $table) {
			$table->dropForeign('UserMagasin_userId_foreign');
		});
        
        Schema::table('UserMagasin', function(Blueprint $table) {
			$table->dropForeign('UserMagasin_magasinId_foreign');
		});
        
        Schema::table('Client', function(Blueprint $table) {
			$table->dropForeign('Client_userId_foreign');
		});
        
        Schema::table('User', function(Blueprint $table) {
			$table->dropForeign('User_entrepriseId_foreign');
		});
        
        Schema::table('Adresse', function(Blueprint $table) {
			$table->dropForeign('Adresse_geographicPointId_foreign');
		});
        
        Schema::table('ClientPrice', function(Blueprint $table) {
			$table->dropForeign('ClientPrice_priceId_foreign');
		});
        
        Schema::table('ClientPrice', function(Blueprint $table) {
			$table->dropForeign('ClientPrice_clientId_foreign');
		});
        
        Schema::table('Product', function(Blueprint $table) {
			$table->dropForeign('Product_categoryId_foreign');
		});
        
        Schema::table('ProductSourcing', function(Blueprint $table) {
			$table->dropForeign('ProductSourcing_productId_foreign');
		});
        
        Schema::table('ProductSourcing', function(Blueprint $table) {
			$table->dropForeign('ProductSourcing_sourcingId_foreign');
		});
        
        Schema::table('Command', function(Blueprint $table) {
			$table->dropForeign('Command_stateId_foreign');
		});
        
        Schema::table('Command', function(Blueprint $table) {
			$table->dropForeign('Command_paymentModeId_foreign');
		});
        
        Schema::table('Command', function(Blueprint $table) {
			$table->dropForeign('Command_clientId_foreign');
		});
        
        Schema::table('CommandPrice', function(Blueprint $table) {
			$table->dropForeign('CommandPrice_commandId_foreign');
		});
        
        Schema::table('CommandPrice', function(Blueprint $table) {
			$table->dropForeign('CommandPrice_priceId_foreign');
		});
        
        Schema::table('CommandReduction', function(Blueprint $table) {
			$table->dropForeign('CommandReduction_commandId_foreign');
		});
        
        Schema::table('ReductionEntreprise', function(Blueprint $table) {
			$table->dropForeign('ReductionEntreprise_priceEntrepriseId_foreign');
		});
        
        Schema::table('StockOperation', function(Blueprint $table) {
			$table->dropForeign('StockOperation_magasinId_foreign');
		});
        
        Schema::table('StockOperation', function(Blueprint $table) {
			$table->dropForeign('StockOperation_userId_foreign');
		});
        
        Schema::table('StockOperation', function(Blueprint $table) {
			$table->dropForeign('StockOperation_typeId_foreign');
		});
        
        Schema::table('StockOperationProduct', function(Blueprint $table) {
			$table->dropForeign('StockOperationProduct_operationId_foreign');
		});
        
        Schema::table('StockOperationProduct', function(Blueprint $table) {
			$table->dropForeign('StockOperationProduct_productId_foreign');
		});
        
        Schema::table('Notification', function(Blueprint $table) {
			$table->dropForeign('Notification_userId_foreign');
		});
        
        Schema::table('CombinedProduct', function(Blueprint $table) {
			$table->dropForeign('CombinedProduct_productId_foreign');
		});
        
        Schema::table('CombinedProduct', function(Blueprint $table) {
			$table->dropForeign('CombinedProduct_priceId_foreign');
		});
        
        Schema::table('PriceEntreprise', function(Blueprint $table) {
			$table->dropForeign('PriceEntreprise_priceId_foreign');
		});
        
        Schema::table('PriceEntreprise', function(Blueprint $table) {
			$table->dropForeign('PriceEntreprise_entrepriseId_foreign');
		});
    }
}
