@php
    $edit = !is_null($dataTypeContent->getKey());
    $add  = is_null($dataTypeContent->getKey());
@endphp

@extends('voyager::master')

@section('css')
    <meta name="csrf-token" content="{{ csrf_token() }}">
@stop

@section('page_title', __('voyager::generic.'.($edit ? 'edit' : 'add')).' '.$dataType->display_name_singular)

@section('page_header')
    <h1 class="page-title">
        <i class="{{ $dataType->icon }}"></i>
        {{ __('voyager::generic.'.($edit ? 'edit' : 'add')).' '.$dataType->display_name_singular }}
    </h1>
    @include('voyager::multilingual.language-selector')
@stop

@section('content')
    <div class="page-content edit-add container-fluid">
        <div class="row">
            <div class="col-md-12">

                <div class="panel panel-bordered">
                    <!-- form start -->
                    <form role="form"
                            class="form-edit-add"
                            action="{{ $edit ? route('voyager.'.$dataType->slug.'.update', $dataTypeContent->getKey()) : route('voyager.'.$dataType->slug.'.store') }}"
                            method="POST" enctype="multipart/form-data">
                        <!-- PUT Method if we are editing -->
                        @if($edit)
                            {{ method_field("PUT") }}
                        @endif

                        <!-- CSRF TOKEN -->
                        {{ csrf_field() }}

                        <div class="panel-body">

                            @if (count($errors) > 0)
                                <div class="alert alert-danger">
                                    <ul>
                                        @foreach ($errors->all() as $error)
                                            <li>{{ $error }}</li>
                                        @endforeach
                                    </ul>
                                </div>
                            @endif

                            <!-- Adding / Editing -->
                            @php
                                $dataTypeRows = $dataType->{($edit ? 'editRows' : 'addRows' )};
                            @endphp

                            @foreach($dataTypeRows as $row)
                                <!-- GET THE DISPLAY OPTIONS -->
                                @php
                                    $display_options = $row->details->display ?? NULL;
                                    if ($dataTypeContent->{$row->field.'_'.($edit ? 'edit' : 'add')}) {
                                        $dataTypeContent->{$row->field} = $dataTypeContent->{$row->field.'_'.($edit ? 'edit' : 'add')};
                                    }
                                @endphp
                                @if (isset($row->details->legend) && isset($row->details->legend->text))
                                    <legend class="text-{{ $row->details->legend->align ?? 'center' }}" style="background-color: {{ $row->details->legend->bgcolor ?? '#f0f0f0' }};padding: 5px;">{{ $row->details->legend->text }}</legend>
                                @endif

                                <div class="form-group @if($row->type == 'hidden') hidden @endif col-md-{{ $display_options->width ?? 12 }} {{ $errors->has($row->field) ? 'has-error' : '' }}" @if(isset($display_options->id)){{ "id=$display_options->id" }}@endif>
                                    {{ $row->slugify }}
                                    <label class="control-label" for="name">{{ $row->display_name }}</label>
                                    @include('voyager::multilingual.input-hidden-bread-edit-add')
                                    @if (isset($row->details->view))
                                        @include($row->details->view, ['row' => $row, 'dataType' => $dataType, 'dataTypeContent' => $dataTypeContent, 'content' => $dataTypeContent->{$row->field}, 'action' => ($edit ? 'edit' : 'add')])
                                    @elseif ($row->type == 'relationship')
                                        @include('voyager::formfields.relationship', ['options' => $row->details])
                                    @else
                                        {!! app('voyager')->formField($row, $dataType, $dataTypeContent) !!}
                                    @endif

                                    @foreach (app('voyager')->afterFormFields($row, $dataType, $dataTypeContent) as $after)
                                        {!! $after->handle($row, $dataType, $dataTypeContent) !!}
                                    @endforeach
                                    @if ($errors->has($row->field))
                                        @foreach ($errors->get($row->field) as $error)
                                            <span class="help-block">{{ $error }}</span>
                                        @endforeach
                                    @endif
                                </div>
                            @endforeach
                            <div class="form-group col-md-12" id="product-list">
                                <label class="control-label" for="name">
                                    Les Lignes de commande
                                </label>
                                <hr />
                                @foreach ($dataTypeContent->prices as $price)
                                    <div class="row">
                                        <div class="col-sm-2 col-form-label" style="margin-bottom: 0">
                                            {{ $price->pivot->client ? $price->pivot->client->user->name : "---" }}
                                            <br/>
                                            {{ $price->pivot->client->phone }}
                                        </div>
                                        <div class="col-sm-1 col-form-label" style="margin-bottom: 0">
                                            <input
                                                type="number"
                                                class="form-control"
                                                name="quantity[]"
                                                value="{{$price->pivot->quantity}}"
                                                placeholder="Quantité"
                                            >
                                        </div>
                                        <label class="col-sm-9" style="margin-bottom: 0">
                                            <input type="hidden" name="priceId[]" value="{{$price->id}}">
                                            {{ $price->products->implode('label', ' + ') }}
                                        </label>
                                    </div>
                                    <hr />
                                @endforeach
                                <div class="content-placeholder">

                                </div>
                                <button
                                    type="button"
                                    onclick="addProduct()"
                                    class="btn btn-success save"
                                >
                                    Ajouter une ligne
                                </button>
                            </div>
                        </div><!-- panel-body -->

                        <div class="panel-footer">
                            @section('submit-buttons')
                                <button type="submit" class="btn btn-primary save">{{ __('voyager::generic.save') }}</button>
                            @stop
                            @yield('submit-buttons')
                        </div>
                    </form>

                    <iframe id="form_target" name="form_target" style="display:none"></iframe>
                    <form id="my_form" action="{{ route('voyager.upload') }}" target="form_target" method="post"
                            enctype="multipart/form-data" style="width:0;height:0;overflow:hidden">
                        <input name="image" id="upload_file" type="file"
                                 onchange="$('#my_form').submit();this.value='';">
                        <input type="hidden" name="type_slug" id="type_slug" value="{{ $dataType->slug }}">
                        {{ csrf_field() }}
                    </form>

                </div>
            </div>
        </div>
    </div>

    <div class="modal fade modal-danger" id="confirm_delete_modal">
        <div class="modal-dialog">
            <div class="modal-content">

                <div class="modal-header">
                    <button type="button" class="close" data-dismiss="modal"
                            aria-hidden="true">&times;</button>
                    <h4 class="modal-title"><i class="voyager-warning"></i> {{ __('voyager::generic.are_you_sure') }}</h4>
                </div>

                <div class="modal-body">
                    <h4>{{ __('voyager::generic.are_you_sure_delete') }} '<span class="confirm_delete_name"></span>'</h4>
                </div>

                <div class="modal-footer">
                    <button type="button" class="btn btn-default" data-dismiss="modal">{{ __('voyager::generic.cancel') }}</button>
                    <button type="button" class="btn btn-danger" id="confirm_delete">{{ __('voyager::generic.delete_confirm') }}</button>
                </div>
            </div>
        </div>
    </div>
    <!-- End Delete File Modal -->
@stop

@section('javascript')
    <script src="https://unpkg.com/axios/dist/axios.min.js"></script>
    <script>
        var params = {};
        var $file;

        // function changeCommandLines(commandId) {
        //     var myWindow = window.open(
        //         "{{env('APP_URL')}}/login?commandId="+commandId,
        //         "Command Panier",
        //         "width=700,height=600"
        //     );
        //     return true
        // }

        function addProduct(){
            $("#product-list .content-placeholder").append(
                `<div class="row older-content">
                    <div class="col-sm-1 col-form-label" style="margin-bottom: 1rem">
                        <input
                            type="number"
                            class="form-control"
                            name="quantity[]"
                            value=""
                            placeholder="Quantité"
                        >
                    </div>
                    <label class="col-sm-3" style="margin-bottom: 1rem">
                        <select
                            class="form-control select2-ajax select2-category"
                            name="CategoryId[]"
                        >
                            <option value="">Aucun</option>
                        </select>
                    </label>
                    <label class="col-sm-8 product-place-holder" style="margin-bottom: 1rem">
                            <select
                                class="form-control"
                                name="priceId[]"
                            >
                            </select>
                    </label>
                </div>`
            );
            $("#product-list .content-placeholder select").select2()
            getCategory()
        }

        function getCategory(categoryId){
            $("#voyager-loader").fadeIn();
            return axios.get(`/admin/select/category`).then(function (response) {
                $("#voyager-loader").fadeOut();
                let categories = response.data
                let element = $(".select2-category")
                categories.map(el => {
                    element.append(`
                        <option class="category-option-${el.id}" value="${el.id}">${el.text}</option>
                    `)
                })
                element.select2()
            }).catch(function (error) {
                $("#voyager-loader").fadeOut();
                return []
            });
        }

        function getPriceByCategory(categoryId){
            $("#voyager-loader").fadeIn();
            return axios.get(`/admin/select/price?categoryId=${categoryId}`).then(function (response) {
                $("#voyager-loader").fadeOut();
                return response.data
            }).catch(function (error) {
                $("#voyager-loader").fadeOut();
                return []
            });
        }

        $(document).on("change", ".select2-category", function(){
            let categoryId = $(this).val()
            // $(`.category-option-${categoryId}`).remove()
            $(".select2-category").select2()
            let futureProducts = getPriceByCategory(categoryId)
            let element = $(this).parent().parent().find(".product-place-holder select")
            element.html("")
            futureProducts.then(function (data) {
                data.map((el, key) => {
                    element.append(`
                        <option class="product-option-${el.id}" value="${el.id}">
                            ${el.products.reduce((carry, el) => carry ? " + " : "" + el.label  ,null)}
                        </option>
                    `)
                })
            })
        });

        function deleteHandler(tag, isMulti) {
          return function() {
            $file = $(this).siblings(tag);

            params = {
                slug:   '{{ $dataType->slug }}',
                filename:  $file.data('file-name'),
                id:     $file.data('id'),
                field:  $file.parent().data('field-name'),
                multi: isMulti,
                _token: '{{ csrf_token() }}'
            }

            $('.confirm_delete_name').text(params.filename);
            $('#confirm_delete_modal').modal('show');
          };
        }

        $('document').ready(function () {
            $('.toggleswitch').bootstrapToggle();

            //Init datepicker for date fields if data-datepicker attribute defined
            //or if browser does not handle date inputs
            $('.form-group input[type=date]').each(function (idx, elt) {
                if (elt.type != 'date' || elt.hasAttribute('data-datepicker')) {
                    elt.type = 'text';
                    $(elt).datetimepicker($(elt).data('datepicker'));
                }
            });

            @if ($isModelTranslatable)
                $('.side-body').multilingual({"editing": true});
            @endif

            $('.side-body input[data-slug-origin]').each(function(i, el) {
                $(el).slugify();
            });

            $('.form-group').on('click', '.remove-multi-image', deleteHandler('img', true));
            $('.form-group').on('click', '.remove-single-image', deleteHandler('img', false));
            $('.form-group').on('click', '.remove-multi-file', deleteHandler('a', true));
            $('.form-group').on('click', '.remove-single-file', deleteHandler('a', false));

            $('#confirm_delete').on('click', function(){
                $.post('{{ route('voyager.'.$dataType->slug.'.media.remove') }}', params, function (response) {
                    if ( response
                        && response.data
                        && response.data.status
                        && response.data.status == 200 ) {

                        toastr.success(response.data.message);
                        $file.parent().fadeOut(300, function() { $(this).remove(); })
                    } else {
                        toastr.error("Error removing file.");
                    }
                });

                $('#confirm_delete_modal').modal('hide');
            });
            $('[data-toggle="tooltip"]').tooltip();
        });
    </script>
@stop
