@php
    $edit = !is_null($dataTypeContent->getKey());
    $add  = is_null($dataTypeContent->getKey());
@endphp

@extends('voyager::master')

@section('css')
    <meta name="csrf-token" content="{{ csrf_token() }}">
    <style>
        .flex-container {
            display: flex;
            flex-wrap: wrap;
            padding: 2rem;
            background-color: #f7f7f7;
            max-height: 40rem;
            overflow-y: auto;
        }
        .item {
            flex-grow: 1;
            width: 18rem;
            max-width: 18rem;
            white-space: nowrap;
            overflow: hidden;
            text-overflow: ellipsis;
        }
        .form-radio {
            -webkit-appearance: none;
            -moz-appearance: none;
            appearance: none;
            display: inline-block;
            position: relative;
            background-color: #f1f1f1;
            color: #666;
            top: 10px;
            height: 30px;
            width: 30px;
            border: 0;
            border-radius: 50px;
            cursor: pointer;
            margin-right: 7px;
            outline: none;
        }
        .form-radio.no-radius {
            border-radius: 0;
        }

        .form-radio:checked::before {
            position: absolute;
            font: 13px/1 'Open Sans', sans-serif;
            left: 11px;
            top: 7px;
            content: '\02143';
            transform: rotate(40deg);
        }

        .form-radio:hover {
            background-color: #f7f7f7;
        }

        .form-radio:checked {
            background-color: #ffa821;
            color: white;
        }

        .form-radio.no-radius:checked {
            background-color: #59d772;
            color: white;
        }

        .label {
            font: 15px/1.7 'Open Sans', sans-serif;
            color: #959595;
            -webkit-font-smoothing: antialiased;
            -moz-osx-font-smoothing: grayscale;
            cursor: pointer;
        }

    </style>
@stop

@section('page_title', __('voyager::generic.'.($edit ? 'edit' : 'add')).' '.$dataType->display_name_singular)

@section('page_header')
    <h1 class="page-title">
        <i class="{{ $dataType->icon }}"></i>
        {{ __('voyager::generic.'.($edit ? 'edit' : 'add')).' '.$dataType->display_name_singular }}
    </h1>
    @include('voyager::multilingual.language-selector')
@stop

@section('content')
    <div class="page-content edit-add container-fluid">
        <div class="row">
            <div class="col-md-12">

                <div class="panel panel-bordered">
                    <!-- form start -->
                    <form role="form"
                            class="form-edit-add"
                            action="{{ $edit ? route('voyager.'.$dataType->slug.'.update', $dataTypeContent->getKey()) : route('voyager.'.$dataType->slug.'.store') }}"
                            method="POST" enctype="multipart/form-data">
                        <!-- PUT Method if we are editing -->
                        @if($edit)
                            {{ method_field("PUT") }}
                        @endif

                        <!-- CSRF TOKEN -->
                        {{ csrf_field() }}

                        <div class="panel-body">

                            @if (count($errors) > 0)
                                <div class="alert alert-danger">
                                    <ul>
                                        @foreach ($errors->all() as $error)
                                            <li>{{ $error }}</li>
                                        @endforeach
                                    </ul>
                                </div>
                            @endif

                            <!-- Adding / Editing -->
                            @php
                                $dataTypeRows = $dataType->{($edit ? 'editRows' : 'addRows' )};
                            @endphp

                            @foreach($dataTypeRows as $row)
                                <!-- GET THE DISPLAY OPTIONS -->
                                @php
                                    $display_options = $row->details->display ?? NULL;
                                    if ($dataTypeContent->{$row->field.'_'.($edit ? 'edit' : 'add')}) {
                                        $dataTypeContent->{$row->field} = $dataTypeContent->{$row->field.'_'.($edit ? 'edit' : 'add')};
                                    }
                                @endphp
                                @if (isset($row->details->legend) && isset($row->details->legend->text))
                                    <legend class="text-{{ $row->details->legend->align ?? 'center' }}" style="background-color: {{ $row->details->legend->bgcolor ?? '#f0f0f0' }};padding: 5px;">{{ $row->details->legend->text }}</legend>
                                @endif

                                <div class="form-group @if($row->type == 'hidden') hidden @endif col-md-{{ $display_options->width ?? 12 }} {{ $errors->has($row->field) ? 'has-error' : '' }}" @if(isset($display_options->id)){{ "id=$display_options->id" }}@endif>
                                    {{ $row->slugify }}
                                    <label class="control-label" for="name">{{ $row->display_name }}</label>
                                    @include('voyager::multilingual.input-hidden-bread-edit-add')
                                    @if (isset($row->details->view))
                                        @include($row->details->view, ['row' => $row, 'dataType' => $dataType, 'dataTypeContent' => $dataTypeContent, 'content' => $dataTypeContent->{$row->field}, 'action' => ($edit ? 'edit' : 'add')])
                                    @elseif ($row->type == 'relationship')
                                        @include('voyager::formfields.relationship', ['options' => $row->details])
                                    @else
                                        {!! app('voyager')->formField($row, $dataType, $dataTypeContent) !!}
                                    @endif

                                    @foreach (app('voyager')->afterFormFields($row, $dataType, $dataTypeContent) as $after)
                                        {!! $after->handle($row, $dataType, $dataTypeContent) !!}
                                    @endforeach
                                    @if ($errors->has($row->field))
                                        @foreach ($errors->get($row->field) as $error)
                                            <span class="help-block">{{ $error }}</span>
                                        @endforeach
                                    @endif
                                </div>
                            @endforeach

                                    <div class="form-group col-md-4 ">
                                        <input
                                            onchange="boxChanged('entreprises')"
                                            type="radio"
                                            class="form-radio"
                                            name="radio-box"
                                            id="reduction-code-entreprises"
                                        />
                                        <label
                                            class="label"
                                            for="reduction-code-entreprises"
                                        >
                                            Pour Une entreprise ?
                                        </label>
                                    </div>
                                    <div class="form-group  col-md-4 ">
                                        <input
                                            onchange="boxChanged('clients')"
                                            type="radio"
                                            class="form-radio"
                                            name="radio-box"
                                            id="reduction-code-clients"
                                        />
                                        <label
                                            class="label"
                                            for="reduction-code-clients"
                                        >
                                            Pour Un client ?
                                        </label>
                                    </div>
                                    <div class="form-group  col-md-4 ">
                                        <input
                                            onchange="boxChanged(null)"
                                            type="radio"
                                            class="form-radio"
                                            name="radio-box"
                                            id="reduction-code-null"
                                        />
                                        <label
                                            class="label"
                                            for="reduction-code-null"
                                        >
                                            Pour tout le monde ?
                                        </label>
                                    </div>

                                    <div class="col-md-12">
                                        <div id="data-placeholder" class="flex-container">
                                        </div>
                                    </div>


                        </div><!-- panel-body -->

                        <div class="panel-footer">
                            @section('submit-buttons')
                                <button type="submit" class="btn btn-primary save">{{ __('voyager::generic.save') }}</button>
                            @stop
                            @yield('submit-buttons')
                        </div>
                    </form>

                    <iframe id="form_target" name="form_target" style="display:none"></iframe>
                    <form id="my_form" action="{{ route('voyager.upload') }}" target="form_target" method="post"
                            enctype="multipart/form-data" style="width:0;height:0;overflow:hidden">
                        <input name="image" id="upload_file" type="file"
                                 onchange="$('#my_form').submit();this.value='';">
                        <input type="hidden" name="type_slug" id="type_slug" value="{{ $dataType->slug }}">
                        {{ csrf_field() }}
                    </form>

                </div>
            </div>
        </div>
    </div>

    <div class="modal fade modal-danger" id="confirm_delete_modal">
        <div class="modal-dialog">
            <div class="modal-content">

                <div class="modal-header">
                    <button type="button" class="close" data-dismiss="modal"
                            aria-hidden="true">&times;</button>
                    <h4 class="modal-title"><i class="voyager-warning"></i> {{ __('voyager::generic.are_you_sure') }}</h4>
                </div>

                <div class="modal-body">
                    <h4>{{ __('voyager::generic.are_you_sure_delete') }} '<span class="confirm_delete_name"></span>'</h4>
                </div>

                <div class="modal-footer">
                    <button type="button" class="btn btn-default" data-dismiss="modal">{{ __('voyager::generic.cancel') }}</button>
                    <button type="button" class="btn btn-danger" id="confirm_delete">{{ __('voyager::generic.delete_confirm') }}</button>
                </div>
            </div>
        </div>
    </div>
    <!-- End Delete File Modal -->
@stop

@section('javascript')
    <script src="https://unpkg.com/axios/dist/axios.min.js"></script>
    <script>
        var params = {};
        var $file;

        function boxChanged(tag) {
            if(tag == "entreprises") {
                getEntreprises("", true)
            }
            else if(tag == "clients") {
                getClients("", true)
            }
            else if (tag == null) {
                $("#data-placeholder").html("")
            }
        }

        @if($dataTypeContent->entreprises()->count() > 0)
            $("#reduction-code-entreprises").prop("checked", true)
            getEntreprises().then(function() {
                @foreach ($dataTypeContent->entreprises() as $entreprise)
                    $("#entreprise-id-{{$entreprise->id}}").prop("checked", true)
                @endforeach
            });
        @elseif($dataTypeContent->clients->count() > 0)
            $("#reduction-code-clients").prop("checked", true)
            getClients().then(function() {
                @foreach ($dataTypeContent->clients as $client)
                    $("#client-id-{{$client->id}}").prop("checked", true)
                @endforeach
            });
        @else
            $("#reduction-code-null").prop("checked", true)
        @endif


        function addClientElement(elementId, elementLabel, attr = "") {
            return $("#data-placeholder").append(`<div class="item">
                <input class="form-radio no-radius" name="clients[]" id="client-id-${elementId}" type="checkbox" value="${elementId}" ${attr}>
                <label for="client-id-${elementId}">${elementLabel}</label>
            </div>`)
        }

        function addEntrepriseElement(elementId, elementLabel, attr = "") {
            return $("#data-placeholder").append(`<div class="item">
                <input class="form-radio no-radius" name="entreprises[]" id="entreprise-id-${elementId}" type="checkbox" value="${elementId}" ${attr}>
                <label for="entreprise-id-${elementId}">${elementLabel}</label>
            </div>`)
        }

        function getClients(keyword = ""){
            $("#voyager-loader").fadeIn();
            $("#data-placeholder").html("")
            return axios.get(`/admin/select/client?keyword=${keyword}`).then(function (response) {
                $("#voyager-loader").fadeOut();
                const elements = response.data;
                for (let index = 0; index < elements.length; index++) {
                    addClientElement(elements[index].id, elements[index].user.name);
                }
            }).catch(function (error) {
                $("#voyager-loader").fadeOut();
                return []
            });
        }

        function getEntreprises(keyword = ""){
            $("#voyager-loader").fadeIn();
            $("#data-placeholder").html("")
            return axios.get(`/admin/select/entreprise?keyword=${keyword}`).then(function (response) {
                $("#voyager-loader").fadeOut();
                const elements = response.data;
                for (let index = 0; index < elements.length; index++) {
                    addEntrepriseElement(elements[index].id, elements[index].name);
                }
            }).catch(function (error) {
                $("#voyager-loader").fadeOut();
                return []
            });
        }

        function deleteHandler(tag, isMulti) {
          return function() {
            $file = $(this).siblings(tag);

            params = {
                slug:   '{{ $dataType->slug }}',
                filename:  $file.data('file-name'),
                id:     $file.data('id'),
                field:  $file.parent().data('field-name'),
                multi: isMulti,
                _token: '{{ csrf_token() }}'
            }

            $('.confirm_delete_name').text(params.filename);
            $('#confirm_delete_modal').modal('show');
          };
        }

        $('document').ready(function () {
            $('.toggleswitch').bootstrapToggle();

            //Init datepicker for date fields if data-datepicker attribute defined
            //or if browser does not handle date inputs
            $('.form-group input[type=date]').each(function (idx, elt) {
                if (elt.type != 'date' || elt.hasAttribute('data-datepicker')) {
                    elt.type = 'text';
                    $(elt).datetimepicker($(elt).data('datepicker'));
                }
            });

            @if ($isModelTranslatable)
                $('.side-body').multilingual({"editing": true});
            @endif

            $('.side-body input[data-slug-origin]').each(function(i, el) {
                $(el).slugify();
            });

            $('.form-group').on('click', '.remove-multi-image', deleteHandler('img', true));
            $('.form-group').on('click', '.remove-single-image', deleteHandler('img', false));
            $('.form-group').on('click', '.remove-multi-file', deleteHandler('a', true));
            $('.form-group').on('click', '.remove-single-file', deleteHandler('a', false));

            $('#confirm_delete').on('click', function(){
                $.post('{{ route('voyager.media.remove') }}', params, function (response) {
                    if ( response
                        && response.data
                        && response.data.status
                        && response.data.status == 200 ) {

                        toastr.success(response.data.message);
                        $file.parent().fadeOut(300, function() { $(this).remove(); })
                    } else {
                        toastr.error("Error removing file.");
                    }
                });

                $('#confirm_delete_modal').modal('hide');
            });
            $('[data-toggle="tooltip"]').tooltip();
        });
    </script>
@stop
