@extends('voyager::master')

@section('css')
    <meta name="csrf-token" content="{{ csrf_token() }}">
    <link rel="stylesheet" href="https://unpkg.com/tableexport/dist/css/tableexport.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.css">
    <style>
        legend {
            display: initial;
            width: initial;
            margin-bottom: initial;
            font-size: initial;
            line-height: initial;
            color: initial;
            border-bottom: initial;
        }
        fieldset, legend {
            padding: initial;
            border: initial;
        }
        fieldset {
            min-width: initial;
            margin: initial;
        }
        fieldset {
            margin: 0 2px;
            border: 1px solid silver;
            padding: .35em .625em .75em;
        }
        .row > [class*="col-"]{
            margin: 15px 0;
        }
        .voyager .btn {
            background: #2ecc71;
            color: #fff;
            border: 0;
            border-radius: 3px;
            opacity: .9;
        }
        canvas {
			-moz-user-select: none;
			-webkit-user-select: none;
			-ms-user-select: none;
		}
        input[type="date"].form-control, input[type="datetime-local"].form-control, input[type="month"].form-control, input[type="time"].form-control{
            line-height: initial;
        }
    </style>
@stop

@section('page_title', 'L\'exportation des rapports')

@section('page_header')
    <h1 style="margin: 0 40px" class="page-title">
        <i class="voyager-receipt"></i>
        La derniére synchronisation a eu lieu le : {{ Voyager::setting("site.last_sync_date") }}
    </h1>
    @include('voyager::multilingual.language-selector')
@stop


@section('content')
    <div style="margin: 0 40px" class="container d-flex justify-content-center align-items-center">
        <form id="syncDataForm" method="POST" action="{{route("sync_catalogue_route")}}">
            @csrf
            <input class="form-control" style="max-width: 350px" type="password" name="authPassword" placeholder="Entrez votre mot de passe">
            <button type="button" id="syncData" class="btn btn-sm btn-primary">
                Synchroniser
            </button>
        </form>
    </div>
@stop

@section('javascript')

    <script src="https://cdn.jsdelivr.net/npm/file-saver@2.0.2/dist/FileSaver.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/xlsx/0.15.1/xlsx.core.min.js"></script>
    <script src="https://unpkg.com/tableexport/dist/js/tableexport.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.js"></script>
    <script src="https://unpkg.com/axios/dist/axios.min.js"></script>

    <script>
        var ExportButtons = document.getElementById('export-buttons-table');

        var instance = new TableExport(ExportButtons, {
            formats: ['xls'],
            exportButtons: false
        });
        var exportData = instance.getExportData()['export-buttons-table']['xls'];
        $("#export-table").on("click", function(){
            instance.export2file(exportData.data, exportData.mimeType, exportData.filename, exportData.fileExtension);
        })
    </script>

    <script>

        $('#syncData').on('click', function ()
        {  
            $("#voyager-loader").fadeIn();
            $(this).attr('disabled', true)
            $('#syncDataForm').submit()
        });

        let chiffreDatasets = {
            label: 'Le chiffre affaire',
            backgroundColor: "#75c3ad",
            borderColor: "#5d9988",
            fill: false,
            data: []
        }

        let budgetDatasets = {
            label: 'Le budget',
            backgroundColor: "#22a7f0",
            borderColor: "#187db5",
            fill: false,
            data: []
        }

        let labels = []

        @if(count($reportedRecap) > 1)
            @foreach ($reportedRecap as $recap)
                chiffreDatasets.data.push({
                    x: '{{ $recap["createdAt"] }}',
                    y: '{{ $recap["chiffreAffaire"] }}'
                })
                budgetDatasets.data.push({
                    x: '{{ $recap["createdAt"] }}',
                    y: '{{ $recap["budget"] }}'
                })
            @endforeach
        @endif

        let config = {
            type: 'line',
            data: {
                labels: chiffreDatasets.data.map(el => el.x),
                datasets: [
                    chiffreDatasets,
                    budgetDatasets
                ]
            }
        };

        window.onload = function() {
            var ctx = document.getElementById('canvas').getContext('2d');
            window.myLine = new Chart(ctx, config);
        };

    </script>
@stop
