<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::group(['prefix' => 'auth'], function() {

    Route::post('login', 'Auth\LoginController@loginWithToken');
    Route::post('forgot', 'Auth\LoginController@forgotPassword');

});

///home routes and urls
Route::group(['prefix' => 'home'], function() {
    //cache
    Route::post('products/guests', 'Api\HomeController@getProductsGuests');//->middleware('cache.response:5');
    Route::get('products', 'Api\HomeController@getProducts')->middleware('jwt');
    Route::post('products/search', 'Api\HomeController@findProducts');
    //cache
    Route::get('products/home', 'Api\HomeController@getProductsHome'); //->middleware('cache.response:15');
    Route::get('products/home/{description_id}/{color_id}', 'Api\HomeController@getSelectedProduct');
});

Route::group(['prefix' => 'checkout'], function() {

    Route::get('products/simple/guests', 'Api\CheckoutController@getRecommendedProductsGuests');
    Route::get('promo/{promoCode}/guests', 'Api\CheckoutController@getUnauthentifiedPromoCodeReduction');
    Route::post('client/register', 'Api\CheckoutController@addClient');
    Route::get('promo/{promoCode}', 'Api\CheckoutController@getPromoCodeReduction');
    Route::group(['middleware' => 'jwt'], function () {
        Route::get('products/simple', 'Api\CheckoutController@getRecommendedProducts');
        Route::post('invite/collegues', 'Api\CheckoutController@inviteCollegues');
        Route::post('cart/submit', 'Api\CheckoutController@addCartcheckOut');
        Route::post('command/{commandId}/cancel', 'Api\CheckoutController@cancelCommand');
        Route::post('/add-payment-file', 'Api\CheckoutController@addPaymentFile');
        Route::post('/track-commands', 'Api\CheckoutController@getAllCommands');
    });
    Route::post('/command-details', 'Api\CheckoutController@getCommandDetails');
    Route::get('/shipping-products', 'Api\CheckoutController@getShippingPrices');

});

Route::group(['prefix' => 'delivery', 'middleware' => 'jwt'], function() {
    Route::get('commands', 'Api\DeliveryController@getCommands');
    Route::post('command/{commandId}', 'Api\DeliveryController@trackDeliverMan');
    Route::post('command/{commandId}/delivered', 'Api\DeliveryController@markCommandDelivered');
    Route::get('command/{commandId}/position', 'Api\DeliveryController@getCommandPosition');
});

Route::group(['prefix' => 'profile', 'middleware' => 'jwt'], function() {
    Route::get('commands', 'Api\ProfileController@getCommands');
    Route::get('favorite/products', 'Api\ProfileController@getFavoritProducts');
    Route::get('address', 'Api\ProfileController@getAddresses');
    Route::delete('address/{id}', 'Api\ProfileController@deleteAddress');
    Route::get('command/{id}', 'Api\ProfileController@getCommand');
    Route::patch('user/update', 'Api\ProfileController@updateProfile');
});

Route::group(['prefix' => 'app'], function() {
    Route::post('contact', 'Api\RootAppController@addContact');
    Route::get('init_data', 'Api\RootAppController@getInitData');
    Route::get('stock_status', 'Api\RootAppController@getStockStatus');
    Route::get('reductions', 'Api\RootAppController@getReductions');
    //cache
    Route::get('category/roots', 'Api\RootAppController@getRootCategories');//->middleware('cache.response:1440');
    Route::get('category/all', 'Api\RootAppController@getAllCategory');//->middleware('cache.response:1440');
    //cache
    Route::get('slider', 'Api\RootAppController@getSliderByTag')->middleware('cache.response:1440');
    Route::post('product_stock_status', 'Api\RootAppController@getProductStockStatus');
});

Route::group(['prefix' => 'shop'], function() {
    //cache
    Route::get('filters', 'Api\CollectionController@getFilteres')->middleware('cache.response:1440');
    Route::get('products', 'Api\CollectionController@getProducts');
    Route::get('ingredients', 'Api\CollectionController@getIngredients');
});

Route::group(['prefix' => 'product'], function() {
    // Route::get('/sync', 'Api\ProductController@syncData');
    Route::get('{priceId}', 'Api\ProductController@getProductDetail');
    Route::post('{priceId}/review', 'Api\ProductController@addReview');
    //cache
    Route::post('detailKeys', 'Api\HomeController@getAllProductsDetailKeys');//->middleware('cache.response:1440');
    Route::get('detailKeys/{id}', 'Api\HomeController@getProductsDetailKeys')->middleware('cache.response:1440');
});

Route::group(['prefix' => 'app', 'middleware' => 'jwt'], function() {
    Route::get('user', 'Api\RootAppController@getAuthUser');
    Route::get('delivery/user', 'Api\RootAppController@getDeliveryAuthUser');
    Route::get('command/hash', 'Api\RootAppController@getCommandByHash');
    Route::post('price/{priceId}', 'Api\RootAppController@favoritProduct');
    Route::post('user/delete-account', 'Api\RootAppController@deleteAccount');
});

Route::group(['prefix' => 'blog'], function() {
    Route::get('', 'Api\BlogController@getBlogs');
    Route::get('/{slug}', 'Api\BlogController@getBlogDetail');
    Route::post('/send-comment/{blog_id}', 'Api\BlogController@addComment');
});